$LARGE
c**********************************************************************
c*** File name: PREP6.FOR (Source code for the program DATAPREP)
c**********************************************************************
c	 P u t   E l e m e n t       (subroutine)
c**********************************************************************
      SUBROUTINE Put Element (Data Unit,Col,Index,Varnum,Value)
 
c*** This routine replaces a sample datum into a scratch file with 
c*** the variable 'Value'. 
c***
c*** VARIABLES:
c*** Data Unit - Unit number of scratch file.
c*** Col       - variable requested from infile.
c*** Index     - data item requested of 'Col'.
c*** Varnum    - number of variables.
c*** Value     - sample datum replacement.

      REAL*4     Value
      INTEGER*4  Varnum, Data Unit, Col, J, Index

c***  Calculate the record number containing the sample to be replaced.
c***  Each record of a scratch file contains only one sample.
      J = Col + ((Index - 1) * Varnum) 

c***  Write the new sample datum(Value) to that record.
      WRITE (Data Unit, REC = J) Value

      RETURN
      END
c**********************************************************************
c        C h e c k  f o r  S a m e  N a m e  V a r i a b l e s (subroutine)
c**********************************************************************
      SUBROUTINE Check for Same Name Variables(Error, Tnvar, K)

c***  This routine compares all the variable names contained in the
c***  global array 'Varnam' for non-unique names.  If two variable
c***  names are non-unique then an error message is displayed and
c***  'Error' is returned .true..  
c***
c***  VARIABLES:
c***  Error - logical that returns .true. if non-unique variable 
c***          names exist.
c***  Tnvar - total number of variables.
c***  K	    - integer variable that indicates which error message to
c***          print.

$INCLUDE:'Dataprep.dec'
$INCLUDE:'Screen.str'
	
      LOGICAL*1   Error
      INTEGER*4	  Tnvar, I, J, K


c***  The following nested do loops compare each variable name
c***  to another searching for non-unique names.
      DO 10 I = 1, Tnvar
	DO 15 J = Tnvar, I+1, -1
	  IF ( Varnam(I) .EQ. Varnam(J) ) THEN
c***        If non-unique variable names exist then display appropriate
c***        error message, set 'Error' condition, then return to calling
c***        routine.
	    IF (K .EQ. 1) THEN
              CALL Flash (20, 2,
     +    'ERROR - Similar named variables in file 1...(Press any key)')
	    ELSE
              CALL Flash (20, 2,
     +    'ERROR - Similar named variables in file 2...(Press any key)')
	    ENDIF
 	    Error = .TRUE.
	    RETURN
	  ENDIF
  15    CONTINUE
  10  CONTINUE

      RETURN
      END
c**********************************************************************
c        G e t   F i l e n a m e      (subroutine)
c**********************************************************************
      SUBROUTINE Get Name of File(Fieldname, Filename, Error, Check)

c***   This routine prompts the user for the name of the file and
c***   checks if the filename is a blank, if so 'Error'=true is returned.
c***   If the variable 'Check' = 1 then the following is performed:
c***   if the filename is not blank it is checked for existence, if non-
c***   existent then 'Error'=true is returned.   If the file exists then  
c***   'Error'=false is returned. 
c***   If the variable 'Check' = 2 then the following is performed:
c***   if the filename exists, the user is asked overwriting is permissable,
c***   if no is the response then the output file name is deleted from
c***   the screen and 'Error'=true.
c***
c***   VARIABLES:
c***   Fieldname - contains the name of the screen field to be accessed.
c***   Filename	 - contains the name of the file to be accessed.
c***   Error	 - logical that returns .true. if the file does not
c***               exist or if the user does not wish to overwrite an
c***               existing output file.
c***   Check	 - if 1 then a check is performed for in input file,
c***               if 2 then a check is performed for an output file.
c***   Prefile	 - global variable that is assigned the prefix + filename
c***               during the execution of the 'Build Prefix Filename' 
c***               routine.
c***   Yes	 - logical variable. 
c***   Exist	 - logical that is .true. if the file in question exists.

$INCLUDE:'Dataprep.dec'
$INCLUDE:'Screen.str'
		   
      CHARACTER   Blank*14, Fieldname*8, Filename*14,Readmess*16
      INTEGER*4	  Check
      LOGICAL*1   Exist, Error, Yes, Get Yes No

      Readmess	  = ' Reading data...'
c***  Initialize 'Error'.
      Error = .FALSE.
      Blank = '              '
c***  Prompt user for filename. ***
      Next Field = FLD Index (Fieldname)
      CALL Get Field
      CALL Get Character Value (Fieldname, Filename)
      CALL Put Str (Readmess, 16, 20, 2, 2)
c***  If  filename is blank issue message and return Error=true.
      IF (Filename .EQ. Blank) THEN
	Error = .TRUE.
	CALL Flash(20, 2, 
     +     'ERROR - filename is blank...(press any key)')
      ELSE
c***  If file does not exist then issue message and return Error=true.
c***  Perform this check only if 'Check' = 1  where 1 means in input file.
	IF (Check .EQ. 1) THEN
          CALL Build Prefix Filename (Filename)
          INQUIRE (FILE = Prefile, EXIST = Exist)
	  IF (.NOT. Exist) THEN
	    Error = .TRUE.
	    CALL Flash(20, 2, 
     +       'ERROR - file does not exist...(press any key)')
	  ENDIF
c***    Check if output file exists.
	ELSE IF (Check .EQ. 2) THEN
          CALL Build Prefix Filename (Filename)
          INQUIRE (FILE = Prefile, EXIST = Exist)
	  IF (Exist) THEN
     	    Yes = Get Yes No (20, 2,
     +	    ' Output file exists, do you wish to overwrite?...<Y/N>')
	    IF (.NOT. Yes) THEN
c***          If the user does not wish to overwrite then blank out
c***          the filename from the display.
              Error = .TRUE.
              CALL Set Character Value (Fieldname, Blank)
	      CALL Display One Field(Fieldname)
	    ENDIF
	  ENDIF
 	ENDIF
      ENDIF

c***  Erase 'Reading data' message.
      CALL Erase(20,2,30)
      RETURN
      END
c**********************************************************************
c        S e t   F i l e n a m e  B l a n k    (subroutine)
c**********************************************************************
      SUBROUTINE Set Filename Blank(Fieldname)

c***  This routine sets the field of the named screen field
c***  blank. 
c***  
c***   VARIABLES:
c***   Fieldname - contains the name of the screen field to be accessed.
c***   Blank	 - character string of blanks.

$INCLUDE:'Screen.str'
	
	CHARACTER Blank*14, Fieldname*8

	Blank = '              '
c***    Set specified screen field to blank characters.
	CALL Set Character Value (Fieldname, Blank)

	RETURN
	END
c**********************************************************************
c        G e t   D o s  F i l e n a m e      (subroutine)
c**********************************************************************
      SUBROUTINE Get Dos Filename(Fieldname,Filename,Error,Check,Option)

c***   This routine prompts the user for the file name used in the DOS 
c***   command and checks if the filename is a blank, if so 'Error'=true 
c***   is returned.  If the variable 'Check' = 1 then the following is 
c***   performed:
c***   If the filename is not blank it is checked for existence, if non-
c***   existent then 'Error'=true is returned.   If the file exists then  
c***   'Error'=false is returned. 
c***   If the variable 'Check' = 2 then the following is performed:
c***   if the filename exists, the user is asked overwriting is permissable,
c***   if no is the response then the output file name is deleted from
c***   the screen and 'Error'=true.
c***
c***   VARIABLES:
c***   Fieldname - contains the name of the screen field to be accessed.
c***   Filename	 - contains the name of the file to be accessed.
c***   Error	 - logical that returns .true. if the file does not
c***               exist or if the user does not wish to overwrite an
c***               existing output file.
c***   Check	 - if 1 then a check is performed for in input file,
c***               if 2 then a check is performed for an output file.
c***   Option    - the option number represents the following DOS commands:
c***                   1: COPY
c***                   2: DELETE
c**	               3: PRINT
c***	               4: RENAME
c***	               5: LIST
c***   Prefile	 - global variable that is assigned the prefix + filename
c***               during the execution of the 'Build Prefix Filename' 
c***               routine.
c***   Yes	 - logical variable. 
c***   Exist	 - logical that is .true. if the file in question exists.


$INCLUDE:'Dataprep.dec'
$INCLUDE:'Screen.str'
		   
      CHARACTER   Blank*14, Fieldname*8, Filename*14
      INTEGER*4	  FLD Index, Check, Option
      LOGICAL*1   Exist, Error, Yes, Get Yes No

      Error = .FALSE.
      Blank = '              '
c***  Prompt user for filename ***
      Next Field = FLD Index (Fieldname)
      CALL Get Field
      CALL Get Character Value (Fieldname, Filename)
c***  If  filename is blank issue message and return error=true.
      IF (Filename .EQ. Blank) THEN
	Error = .TRUE.
	CALL Flash(20, 2, 
     +     'ERROR - filename is blank...(press any key)')
      ELSE
c***    If file does not exist then issue message and return Error=true.
c***    Perform this check only if 'Check' = 1  where 1 means in input
c***    file.  'Check' = 2 means output file check.
	IF (Check .EQ. 1) THEN
c***      The next routine affixes the prefix(if any) to the filename
c***      with the results contained in 'Prefile'.
          CALL Build Prefix Filename (Filename)
          INQUIRE (FILE = Prefile, EXIST = Exist)
c***      If the file does not exist then set Error=true and display the 
c***      error message indicated by the option number.
	  IF (.NOT. Exist) THEN
	    Error = .TRUE.
	    IF (Option .EQ. 1) THEN
	       CALL Flash(20, 2, 
     +    'ERROR - file to be copied does not exist...(press any key)')
	    ELSE IF (Option .EQ. 2) THEN
	       CALL Flash(20, 2, 
     +    'ERROR - file to be deleted does not exist...(press any key)')
	    ELSE IF (Option .EQ. 3) THEN
	       CALL Flash(20, 2, 
     +    'ERROR - file to be printed does not exist...(press any key)')
	    ELSE IF (Option .EQ. 4) THEN
	       CALL Flash(20, 2, 
     +    'ERROR - file to be renamed does not exist...(press any key)')
	    ELSE IF (Option .EQ. 5) THEN
	       CALL Flash(20, 2, 
     +    'ERROR - file to be listed does not exist...(press any key)')
	    ENDIF
	  ENDIF
c***    Perform the check for an output file.
	ELSE IF (Check .EQ. 2) THEN
          CALL Build Prefix Filename (Filename)
          INQUIRE (FILE = Prefile, EXIST = Exist)
	  IF (Exist) THEN
	    IF (Option .EQ. 1) THEN
     	      Yes = Get Yes No (20, 2,
     +	      ' Output file exists, do you wish to overwrite?...<Y/N>')
	      IF (.NOT. Yes) THEN
                Error = .TRUE.
                CALL Set Character Value (Fieldname, Blank)
	        CALL Display One Field(Fieldname)
	      ENDIF
	    ELSE IF (Option .EQ. 4) THEN
              Error = .TRUE.
	      CALL Flash(20, 2, 
     +    'ERROR - this filename already exists...(press any key)')
	    ENDIF
	  ENDIF
 	ENDIF
      ENDIF

      RETURN
      END
c**********************************************************************
c        C e n t e r  F m t       (subroutine)  
c**********************************************************************
      SUBROUTINE Center Fmt(Count, End Col, V1, Centerln)

c***  This routine right justifies text within a character string  
c***  giving the appearance of a centered format in the 'Report' listing.
c***  The text right justified is either a variable name from the
c***  global array Varnam or a measurement description from the global
c***  array Units.  The character string becomes an element in the 
c***  array Centerln.
c***  
c***  VARIABLES:
c***  Count    - index of global array Startcol (which contains the indices 
c***             of the global arrays Varnam and Units) indicating
c***             first element to be right justified.
c***  End Col  - index of global array Startcol (which contains the indices
c***             of the global arrays Varnam and Units) indicating
c***             last element to be right justified.
c***  V1       - logical that is true if Varnam element is to be right 
c***             justified, false if Units element is to be justified.
c***  Centerln - array whose elements contain the justified text.
c***  Varnam   - global array containing the variable names.
c***  Startcol - global array containing index numbers relating to 
c***             Varnam or Units elements..
c***  Units    - global array containing the unit description of each 
c***             variable.
c***  N        - is an index for the array Centerln.

$INCLUDE:'Dataprep.dec'
$INCLUDE:'Screen.str'

      CHARACTER*10    Centerln(4)
      LOGICAL*1       V1
      INTEGER*4	      Count, End Col, J, N

c***  N is an index for the array Centerln.  It is incremented after
c***  each iteration of the DO 35 loop.  
      N = 1
c***  The following loop performs the justification process on the character
c***  strings.  Start with the variable(column) indicated by Count
c***  and end with the variable(column) indicated by End Col.  The loop
c***  can center up to 4 variables.
      DO 35 J = Count, End Col
c***    V1 is true if the variable name is to be justified.
c***    V1 is false if the measurement description is to be justified.
	  IF (V1) THEN
c***        Assign variable name to Centerln array.
	    Centerln(N) = Varnam(Startcol(J))
	  ELSE
c***        Assign measurement description to Centerln array.
	    Centerln(N) = Units(Startcol(J))
	  ENDIF
c***    Justify the character string.
        CALL Right Justify(Centerln(N),10)
c***    Increment Centerln array index.
        N = N + 1
  35  CONTINUE

      RETURN
      END
c**********************************************************************
c        F m t  B u i l d e r     (subroutine)  
c**********************************************************************
      SUBROUTINE Fmt Builder(Count, Option, End Col)

c***  This routine constructs the output format specifier used in
c***  generating the 'Report' listing of variable samples.  The format
c***  specifier contains specifiers for up to 4 variables since the
c***  'Report' listing can generate up to 4 variables per line.
c***
c***  VARIABLES:
c***  Count    - index of global array 'Varnam' indicating
c***             first element of row.
c***  Option   - logical that is true if observation number is to be  
c***             included, false if excluded.
c***  End Col  - index of global array (Varnam or Units) indicating
c***             last element of row.
c***  Fmt      - array containing the output format specifier.
c***  Formats  - array containing the output format for each variable.
c***  Startcol - array containing specific element indices for the
c***             Formats array.
c***  J        - index counter for the array 'Fmt'.
c***  Value    - temporary variable that contains the integer value
c***             of the width field of the format specifier in question.

$INCLUDE:'Dataprep.dec'

      LOGICAL*1 Option
      INTEGER*4 End Col, Count, Value

c***  Insert sequence number specifier if sequence option is chosen.
c***  Set J to current element in the 'Fmt' array.
      IF (Option) THEN
        Fmt(1:15) = '(3X,I4,''. '',2X,'
	J = 15
      ELSE
c***    Insert spaces normally filled with sequence number.
        Fmt(1:5) = '(11X,'
	J = 5
      ENDIF
c***  The following DO 10 loop constructs the 'Fmt' array for up to
c***  three variables.  The last variable specifier is added after this
c***  loop. 
      DO 10 I = Count, End Col - 1
c***    Assign the format specifier for a variable (contained in 
c***    'Formats' array) and assign to 'Fmts' array.  The following
c***    code determines the length of the character string containing
c***    the specifier and assigns each character to the 'Fmts' element. 
	K = LENSTR (Formats(Startcol(I)))
	DO 15 L = 1, K
c***      Set J to next element in the 'Fmt' array.
	  J = J + 1
	  Fmt(J:J) = Formats(Startcol(I))(L:L)
  15	CONTINUE
	J = J + 1
	Fmt (J:J) = ','
c***    Convert width field from a character to an integer and
c***    assign to 'Value'.
c***    Convert first digit of width field.
	Value = ICHAR(Formats(Startcol(I))(2:2)) - ICHAR('0')
c***    Convert second digit(if any) width of field.
	IF (Formats(Startcol(I))(4:4) .EQ. '.')THEN 
    	  Value = ICHAR(Formats(Startcol(I))(3:3)) - ICHAR('0')
     +            + (Value * 10)
	ENDIF
c***    Calculate the number of spaces to insert so that each column
c***    occupies 16 spaces.
	Value = 16 - Value
	  J = J + 1
c***    Convert 'number of spaces to insert' to a character type.
c***    Convert a one digit field.
	IF (Value .LT. 10) THEN
	  Fmt(J:J) = CHAR(ICHAR('0') + Value)
	  J = J + 1
	ELSE
c***      Convert a two digit field.
	  Fmt(J:J) = CHAR(ICHAR('0')  + Value/10)
	  J = J + 1
	  Fmt(J:J) = CHAR(ICHAR('0') + (Value - ((Value/10)*10)))
          J = J + 1
	ENDIF
c***    Add the 'X' to indicate that spaces are skipped, ie., '3X'.
        Fmt(J:J+2) = 'X,'
	J = J + 2
  10  CONTINUE
c***  Include specifier for last element of row.
      K = LENSTR (Formats(Startcol(End Col)))
      DO 11 L = 1, K
        J = J + 1
	Fmt(J:J) = Formats(Startcol(End Col))(L:L)
  11  CONTINUE
      J = J + 1
c***  Include closing parenthesis for the format specifier.
      Fmt(J:J) = ')'

      RETURN
      END
c**********************************************************************

