$LARGE
c**********************************************************************
c*** File name: PREP8.FOR (Source code for the program DATAPREP)
c**********************************************************************
c        S o r t  P r o m p t s     (subroutine)
c**********************************************************************
      SUBROUTINE Sort Prompts

c***  This routine constructs the Sort screen display and menu, prompts
c***  the user for a menu selection, and then processes that selection 
c***  by calling the appropriate subroutines.
c***  
c***  VARIABLES:
c***  Menu Choice - contains the name of the Sort menu option selected 
c***                by the user.
c***  Menu	  - contains the field name of the Sort menu.
c***  Vardone	  - logical that is .true. if the Variable option was
c***                accessed.
c***  Error	  - logical that is .true. if for any reason an operation
c***                could not be performed such as accessing a file or
c***                reading data from a file.
c***  Var	  - contains the variable(column) number of the variable
c***                selected by the user.
c***  Recskip     - contains the number of the first record in the Geo-EAS
c***                data file that contains sample data.
c***  Infil       - global variable that contains the name of the Geo-EAS
c***                input data file.
c***  Outfil      - global variable that contains the name of the Geo-EAS
c***                output data file.


$INCLUDE:'Dataprep.dec'
$INCLUDE:'Screen.str'
		   
      CHARACTER   Menu Choice*20, Menu*10, File*8
      CHARACTER   Toggle Field*8, Readmess*17, Oldfile*14
      CHARACTER   Label1*11,Label2*12,Label3*17
      CHARACTER   Label4*13, Label5*17, Label6*22
      INTEGER*4   FLD Index, Var, Toggle
      LOGICAL*1   Vardone, Error

c***  Erase the File Operations screen.
      CALL Erase Message Divider
      CALL ClearRows(1,18)
      CALL Draw Middle Horizontal Line(4)
      CALL Draw Middle Horizontal Line(6)
      CALL Draw Middle Horizontal Line(9)

      Menu  = 'ST MENU   '
      Vardone = .FALSE.
      Error = .TRUE.

c***  Initialize and diaplay labels.
      Readmess = ' Processing data '
      Label1 = 'Input File:'
      Label2 = 'Output File:'
      Label3 = 'Variable to sort:'
      Label4 = 'SORT FUNCTION'
      Label5 = 'Data File Prefix:'
      Label6 = 'Input\Output Variables'
      CALL Put Str (Label1, 11,  5,  2, 0)
      CALL Put Str (Label2, 12,  5, 39, 0)
      CALL Put Str (Label3, 17,  8,  2, 0)
      CALL Put Str (Label4, 13,  2, 33, 0)
      CALL Put Str (Label5, 17,  3,  2, 0)
      CALL Put Str (Prefix, 55,  3, 20, 0)
      CALL Put Str (Label6, 22, 10, 28, 0)
c***  Initialize and display file name screen fields.
      CALL Set Character Value ('INFILE1 ', Infil)
      CALL Set Character Value ('OUTFILE1 ', Outfil)
c***  Display Sorting menu and prompt user for choice.
      CALL Put Menu (Menu)

  10  CALL Get Menu Choice (Menu, Menu Choice)
      IF (Menu Choice      .EQ. 'Quit                ') THEN
	CALL Erase Middle Horizontal Line(4)
	CALL Erase Middle Horizontal Line(6)
	CALL Erase Middle Horizontal Line(9)
	CALL Clear Rows(1,18)
c***    Erase the variables heading.
	CALL Erase (1, 35, 9)
  	RETURN
      ELSE IF (Menu Choice .EQ. 'Files               ') THEN
	CALL Erase Variable List
c***    Initialize Vardone to false to indicate that the variable option
c***    has not been accessed.
	Vardone = .FALSE.
c***    Prompt user for input file ***
  	File = 'INFILE1 '
	CALL Get Name of File(File,Infil,Error,1)
c***    If error return to menu.
	IF (Error) GOTO 10
c***    Prompt user for outfile.
	File = 'OUTFILE1'
	CALL Get Name of File(File, Outfil, Error,2)
      ELSE IF (Menu Choice .EQ. 'Variable            ') THEN
c***    If legitimate files have been specified by the user then proceed
c***    with variable option an error message and return to the menu.
	IF (.NOT. Error) THEN
c***      Indicate the variable list has been accessed before the
c***      execution option.
	  Vardone = .TRUE.
	  Oldfile = Infil
c***      Read the variable names from the Geo-EAS data file and store
c***      in Scratch File 1 by making the next call.
	  CALL File Set Up (Error, Nvar)
	  Recskip = Nvar + 3
c***      If no error in reading header list of input file then load 
c***      toggle fields and then prompt the user for the variable to 
c***      be sorted.
          IF (.NOT. Error) THEN
	    CALL Load All Toggle Fields (Nvar)
	    CALL Erase Variable List
	    CALL Display Variables(Nvar)
	    Toggle Field = 'VTOGGLE2'
	    Var = Toggle (Toggle Field) + 1
 	  ENDIF
	ELSE
c***    Inform user that legitimate filenames have not been provided.
c***    and return to the menu. 
	    CALL Beep
	    CALL Flash(20, 2, 
     +     'ERROR - Filenames must be entered...(press any key)')
	ENDIF
      ELSE IF (Menu Choice .EQ. 'Execute             ') THEN
	IF (Error) THEN
c***      If filenames have not been entered then inform the user 
c***      and return to the menu.
	  CALL Beep
	  CALL Flash(20, 2, 
     +       'ERROR - Filenames must be entered...(press any key)')
	  GOTO 10
	ELSE IF ((Oldfile .NE. Infil).OR.(.NOT. Vardone)) THEN
c***      Check if variable list option's infile is same as 
c***      file option's infile.  This check is to determine if the
c***      variable list option has been opted before the execution process.
	  CALL Beep
	  CALL Flash(20, 2, 
     +     'ERROR - Choice of variable not provided...(press any key)')
	  GOTO 10
	ENDIF
        Vardone = .FALSE.
        CALL Put Str (Readmess, 17, 20, 2, 2)
c***    Open scratch file to contain 'Data1' 'structure'. ***
        CALL Open Scratch Files
c***    Read the samples of the input file.
        CALL Read Numerical Data(1,Infil,Nvar,Nvar,Error,Recskip)
c***    If no error in reading 'sample' data then display processing 
c***    message and proceed with processing.
	IF (.NOT. Error) THEN
          CALL Put Str (Readmess, 17, 20, 2, 2)
     	  CALL Rank Var (Var, Outfil)
	ENDIF
        CALL Close Scratch Files
      ENDIF

      GOTO 10
      RETURN
      END
c**********************************************************************
c        R a n k  V a r                        (subroutine)
c**********************************************************************

      SUBROUTINE Rank Var (Col, Outfile)

c*** This routine sorts a specified variable in the order of low to
c*** highest.  The corresponding data record for that ordered variable
c*** is then written to the specified output file in that order.
c*** Any missing values found in the specified variable are placed
c*** at the end of the list.  The corresponding records associated with
c*** these missing values, although placed at the end, are presented
c*** in the order as originally encountered in the input file. Ie., if
c*** record 32 contained a missing value in the specified variable and
c*** the next missing value of the specified variable appeared in record
c*** 40 then these records would appear at the end of the listing with
c*** record 32 immediately preceding record 40.
c*** 
c*** VARIABLES:
c*** Col     - this integer value represents the position(element) of the
c***           specified variable to be sorted.  It must contain the 
c***           position when passed to this subroutine.
c*** Outfile - this variable contains the name of the output file to
c***           which the processed data is to be written.  It also must
c***           contain a filename and cannot be blank.
c*** A1	     - a real array which stores the sample data of the variable
c***           to be sorted.
c*** P	     - an integer array which stores the original positions of 
c***           corresponding sample datum. 
c*** Nvar    - global variable which contains the number of variables in
c***           the file.
c*** Ndata   - global variable which contains the number of sample records
c***           contained in the file.
c*** Value   - temporary variable that contains the a sample datum.

c*** The input file must have already been read and the data structures
c*** such as 'Data', 'Varnam', and the variables Nvar and Ndata initialized.

$INCLUDE:'Dataprep.dec'

      CHARACTER*14  Outfile
      REAL*4        Get Element, Value, A1(MAXDATA), SortVar
      INTEGER*4     I, J, K, N, Col, Countr
      INTEGER*2	    P (MAXDATA)
      N = Ndata

c*** Initialize Scratch File 2 with Scratch File 1 data.  
c*** The Scratch File 1 was initialized with the input file 
c*** data in the Read Numerical Data routine.
      DO 10 I = 1, Nvar
	DO 20 J = 1, Ndata
	  Value = Get Element(1,I,J,Nvar)
	  CALL Put Element(2,I,J,Nvar,Value)
  20	CONTINUE
  10  CONTINUE

c***  Initialize the arrays to be used in the subroutine Qsort ***
      DO 30 I = 1, Ndata
	A1 (I) = Get Element (1,Col,I,Nvar)
	P (I) = I
  30  CONTINUE


c***  Sort the specified variable's data stored in A1 in the ascending
c***  order.  Upon return A1 will contain the ordered sample data, and P 
c***  will contain the original position of that sample datum.  N is 
c***  assigned the value contained in 'Ndata' before this call. 

      CALL Qsort (A1, P, N)

c***  Store the entire sample data record relative to the sorted 
c***  variable to Scratch File 1.
      DO 60 I = 1, Nvar
          Countr = 0
          DO 65 J = 1, Ndata
            SortVar = Get Element (2,Col,INT4 (P (J)),Nvar)
c***        If the chosen variable's sample does not have a missing
c***        value then assign the entire record to the appropriate
c***        position.
	    IF (.NOT.((ABS(1.E+31 - SortVar) .LE. 0.0001) .AND.
     +         (SortVar .GE. 0.0))) THEN
	       Countr = Countr + 1
	       Value = Get Element (2, I, INT4 (P (J)), Nvar)
	       CALL Put Element(1,I,Countr,Nvar,Value)
	    ENDIF
  65	  CONTINUE
  60  CONTINUE

c***  The following DO loop sorts the records corresponding to the missing
c***  values, if any, in the order that they were encountered in the input 
c***  file.  

c***  If missing values exist then 'sort' ***
c***  'Countr' contains the total of non-missing samples in the 
c***  sorted variable.
      IF (Countr .LT. Ndata) THEN
c***    Search for missing values in the order they appeared in the 
c***    input file.
        DO 80 J = 1, Ndata
c***      For a missing value assign the corresponding record to 
c***      the sorted records but inserting them where the 'missing value' 
c***      records were grouped at the end of the sorted list.
	  SortVar = Get Element(2,Col,J,Nvar) 
	  IF ((ABS(1.E+31 - SortVar) .LE. 0.0001) .AND.
     +       (SortVar .GE. 0.0)) THEN
	    Countr = Countr + 1
	    DO 85 I = 1, Nvar
	        Value = Get Element (2,I,J,Nvar)
	        CALL Put Element (1,I,Countr,Nvar,Value)
  85	    CONTINUE
	  ENDIF
  80    CONTINUE
      ENDIF

c***  Write results to data file.
      Outfil = Outfile
      CALL Write to Output File 

      RETURN
      END

c**********************************************************************
c        A p p e n d   P r o m p t s     (subroutine)
c**********************************************************************
      SUBROUTINE Append Prompts

c***  This routine constructs the Append screen and displays the
c***  Append menu.  The user is prompted for a menu selection.  The
c***  selection is then processed via subroutine calls.
c***  
c***  VARIABLES:
c***  Menu Choice - contains the user menu selection.
c***  Menu	  - contains the fieldname of the Append menu.
c***  Equal Var	  - number of variables that exist in both Infile1
c***                and Infile2 simultaneously.
c***  Infile1	  - global variable that contains the name of the file 
c***                to append to.
c***  Infile2	  - global variable that contains the name of the file 
c***                to be appended.
c***  Outfile1	  - contains the name of the file whose contents
c***                are the appended files.
c***  Infil	  - global variable that contains the name of the Geo-EAS
c***                input data file.
c***  Outfil	  - global variable that contains the name of the Geo-EAS
c***                output data file.

$INCLUDE:'Dataprep.dec'
$INCLUDE:'Screen.str'
		   
      CHARACTER   Menu Choice*20, Menu*10, Blank*14, File*10
      CHARACTER   Toggle Field*8, Readmess*17,Label5*17,Label6*16
      CHARACTER   Label1*50,Label2*50, Label3*50, Label4*15
      INTEGER*4   FLD Index, I, J, K, L, Equal Var
      LOGICAL*1   Yes, Error

c***  Erase the File Operations screen.
      CALL Erase Message Divider
      CALL ClearRows(1,18)
      Blank = '              '
      Error = .TRUE.
      Menu  = 'AP MENU   '

c***  Initialize and display screen labels.
      Readmess = ' Processing data '
      Label1 = 'Name of file 1 (to which file 2 will be appended):'
      Label2 = 'Name of file 2 (which will be appended to file 1):'
      Label3 = 'Name of output file                              :'
      Label4 = 'APPEND FUNCTION'
      Label5 = 'Data File Prefix:'
      Label6 = 'Output Variables'
      CALL Put Str (Prefix, 55, 4, 20, 0)
      CALL Put Str (Label1, 50, 6, 2, 0)
      CALL Put Str (Label2, 50, 7, 2, 0)
      CALL Put Str (Label3, 50, 8, 2, 0)
      CALL Put Str (Label4, 15, 2, 32, 0)
      CALL Put Str (Label5, 17, 4,  2, 0)
      CALL Put Str (Label6, 16, 10, 31, 0)
c***  Draw horizontal line at row 5 across the screen.
      CALL Draw Middle Horizontal Line(5)
c***  Initialize and display file names.
      CALL Set Character Value ('FILE1A ', Infil)
      CALL Set Character Value ('FILE2A ', Infile2)
      CALL Set Character Value ('OUTFILE2 ', Outfil)
      CALL Draw Middle Horizontal Line(9)
c***  Display Append menu and prompt user for choice.
      CALL Put Menu (Menu)
  10  CALL Get Menu Choice (Menu, Menu Choice)
      IF (Menu Choice      .EQ. 'Quit                ') THEN
	Choice = 'UNITS     '
	CALL Set Character Value ('UNITS ', Blank)
	CALL Erase Middle Horizontal Line(5)
	CALL Erase Middle Horizontal Line(9)
	CALL Clear Rows(1,18)
c***    Erase the variables heading ***
	CALL Erase (1, 35, 9)
  	RETURN
      ELSE IF (Menu Choice .EQ. 'Files               ') THEN
	CALL Erase Variable List
c***    Prompt user for the file to append to.
  	File = 'FILE1A  '
	CALL Get Name of File(File,Infile1,Error,1)
c***    If error return to menu.
	IF (Error) GOTO 10
c***    Prompt user for the file to append.
	File = 'FILE2A  '
	CALL Get Name of File(File, Infile2, Error,1)
c***    If error return to menu.
	IF (Error) GOTO 10
c***    If input files are the same then issue message and return to menu.
	IF (Infile1 .EQ. Infile2) THEN
          CALL Flash (20, 2,
     +    'ERROR - Input file is same as output file...(Press any key)')
	  GOTO 10
	ENDIF
c***    Prompt user for the name of the output file. 
	File = 'OUTFILE2  '
	CALL Get Name of File(File, Outfile1, Error,2)
	Infil = Infile1
	Outfil = Outfile1
      ELSE IF (Menu Choice .EQ. 'Execute             ') THEN
	IF (Error) THEN
c***    If filenames have not been entered then inform the user 
c***    and return to the menu.
	  CALL Beep
	  CALL Flash(20, 2, 
     +       'ERROR - Filenames must be entered...(press any key)')
	ELSE
          CALL Put Str (Readmess, 17, 20, 2, 2)
c***      Yes is .true. when Infile2's has not yet been stored into
c***      Scratch File 2.
	  Yes = .TRUE.
	  CALL Append (Infile1, Infile2, Outfile1,
     +               Error, Equal Var, Yes)
        ENDIF
      ENDIF

      GOTO 10
      RETURN
      END
C**********************************************************************
C        A p p e n d     (subroutine)
C**********************************************************************
      SUBROUTINE Append (InfileA, InfileB, OutfileA,
     +                   Error, Equal Var, Yes)

c***  This routine appends InfileB to InfileA and writes the results
c***  to OutfileA.  If an error should occur during processing then an
c***  error message is displayed and the logical 'Error' is returned 
c***  as .true..
c***
c***  VARIABLES:
c***  InfileA	- Name of infile 1 which is the primary file.
c***  InfileB	- Name of infile 2 which is appended to infile 1.
c***  OutfileA	- Name of output file which contains the appended files.
c***  Error	- Logical set to .true. if processing cannot be performed.
c***  Equal Var - Integer variable that returns the number of variables
c***              that exist in both InfileA and InfileB.
c***  Yes	- If this logical is .true. than InfileB's data must be
c***              read.   
c***  Nvar	- Global variable that contains the number of variables
c***              in InfileA.
c***  Ndata	- Global variable that contains the number of data records
c***              in InfileA.
c***  Nvar2	- Global variable that contains the number of variables
c***              in InfileB.
c***  Ndata2	- Global variable that contains the number of data records
c***              in InfileB.
c***  Infil     - Global variable that contains the name of the file to
c***              be read.
c***  Outfil    - Global variable that contains the name of the file to
c***              which the data is saved. 
c***  Tndata    - A temporary variable that contains the number of records
c***              to be stored in the appended file.
c***  Index     - A temporary variable used to determine the number of 
c***              variables to be stored in the appended file.
c***  Tarray    - Logical array used to indicate which variables appear
c***              in both InfileA and InfileB.
c***  Var Count - Maintains the number of unique variables in the output file.
c***  Recskip   - Global variable containing first record with sample data. 

$INCLUDE:'Dataprep.dec'
$INCLUDE:'Screen.str'
	
      CHARACTER*14  InfileA, InfileB, OutfileA
      CHARACTER     Readmess*17
      INTEGER*4     Index, Tndata, Var Count, Equal Var, I, J, K
      REAL*4        Get Element, Value
      LOGICAL*1     Yes, Error, Common Var

      Readmess = ' Processing data '
      Var Count = 0
c***  If Yes=.true. then read InfileB else InfileB data has already been
c***  stored in Scratch File 2. 
      IF (Yes) THEN
c***    Read InfileB ,ie., the file to be attached to InfileA. 
c***    Set global variable Infil.
        Infil = InfileB
	CALL Open Scratch Files
c***    Read header data for InfileB.
        CALL File Set Up (Error, Nvar2)
c***    Check for non-unique variables in InfileB.
        CALL Check for Same Name Variables(Error, Nvar2, 2)
c***    Assign to 'Recskip' the first record with sample data.
        Recskip = Nvar2 + 3
c***    Read all sample data records for InfileB.
        IF (.NOT. Error) 
     +  CALL Read Numerical Data(2,Infil,Nvar2,Nvar2,Error,Recskip)
        IF (.NOT. Error) THEN
c***      Display 'processing' message.
          CALL Put Str (Readmess, 17, 20, 2, 2)
c***      Copy InfileB data over to InfileB header data structures.
	  DO 30 I = 1,Nvar2
	    Varnam2(I) = Varnam(I)
	    Units2(I)  = Units(I)
	    Formats2(I) = Formats(I)
  30      CONTINUE
c***      Assign number of data records for InfileB to Ndata2.
c***      (Ndata is a global variable that is assigned the number of
c***      data records in the file just read and must be reassigned 
c***      to the correct variable which in this case is 'Ndata2') 
	  Ndata2 = Ndata
	ENDIF
      ENDIF
      K = 0
      IF (.NOT. Error) THEN
c***    Read InfileA ie., the file to which InfileB is appended and  
c***    retain info in InfileA data structures. 
c***    set global variable Infil.
	Infil = InfileA
	CALL File Set Up (Error, Nvar)
c***    Check for non-unique variables in InfileA.
        CALL Check for Same Name Variables(Error, Nvar, 1)
	IF (.NOT. Error) THEN
          CALL Put Str (Readmess, 17, 20, 2, 2)
c***      Check for non-unique variables in both InfileA and InfileB.
	  DO 10 I = 1, Nvar
            DO 15 J = 1, Nvar2
	      IF (Varnam(I) .EQ. Varnam2(J)) K = K + 1
  15	    CONTINUE
  10	  CONTINUE
	  Index = Nvar + Nvar2 - K
	  IF (Index .GT. MAXVAR) THEN
	    Var Count = 999
c***        Display error message via label 85.
	    GOTO 85
	  ELSE
c***        Read InfileA sample data.
            Recskip = Nvar + 3
c***        Read sample data from InfileB.
            CALL Read Numerical Data(1,Infil,Nvar,Index,Error,Recskip)
 	  ENDIF
	ENDIF

	IF (.NOT. Error) THEN
          CALL Put Str (Readmess, 17, 20, 2, 2)
c***      Initialize Tarray elements to .false. ***
	  DO 45 I = 1, MAXVAR
	    Tarray(I) = .FALSE.
  45	  CONTINUE
c***      Set Tndata no greater than MAXDATA.  Tndata represents the number
c***	  of sample records to be stored in the output file.
 	  IF(( Ndata+Ndata2) .GT. MAXDATA) THEN
	    Tndata = MAXDATA - Ndata
	  ELSE
	    Tndata = Ndata2
	  ENDIF
c***      Check if same variable exists in both files, if so then add
c***      InfileB variable to InfileA and set Tarray element representing
c***      that variable to true (indicates this variable exists in InfileB).
	  Var Count = 0
	  Equal Var = 0
	  DO 50 I = 1, Nvar
	    Common Var = .FALSE.
	    DO 55 J = 1, Nvar2
	      IF (Varnam(I) .EQ. Varnam2(J)) THEN
		Common Var = .TRUE.
		Equal Var = Equal Var + 1
		Var Count = Var Count + 1
		Tarray(J) = .TRUE.
c***            Add InfileB records to InfileA, for the common variable ***
	        DO 60 K = 1, Tndata
		  Value = Get Element(2,J,K,Nvar2)
		  CALL Put Element(1,I,Ndata+K,Index,Value)
  60		CONTINUE
 	      ENDIF
  55	    CONTINUE
c***        If InfileA variable is not in file 2 then assign missing values 
c***        to all records succeeding InfileA's original records. 
	    IF (.NOT. Common Var) THEN
	      Var Count = Var Count + 1
              DO 65 K = 1, Tndata
	        CALL Put Element(1,I,Ndata+K,Index,Missing)
  65	      CONTINUE
	    ENDIF
  50      CONTINUE
c***      Add all variables in InfileB that were not included in InfileA.
	  DO 70 I = 1, Nvar2
	    IF (.NOT. Tarray(I))THEN
c***          Var Count maintains the total distinct variables of both files.
	      Var Count = Var Count + 1
	      IF (Var Count .LE. MAXVAR) THEN
		Varnam(Var Count) = Varnam2(I)
		Units(Var Count)  = Units2(I)
		Formats(Var Count)= Formats2(I)
c***            Assign Missing values to the uncommon InfileB variable up to 
c***            Ndata records.
		DO 75 K = 1, Ndata
		  CALL Put Element(1,Var Count,K,Index,Missing)
  75		CONTINUE
c***            Assign uncommon InfileB var values to file1 from Ndata+1 to 
c***            Ndata2 records.
		DO 80 K = 1, Tndata
		  Value = Get Element(2,I,K,Nvar2)
		  CALL Put Element(1,Var Count,Ndata+K,Index,Value)
  80		CONTINUE
	      ELSE
c***          If total variables exceed MAXVAR, then set Var Count=999.
		Var Count = 999
		GOTO 85
	      ENDIF
	    ENDIF
  70      CONTINUE
	  Nvar = Var Count
	  Ndata = Ndata + Tndata
c***      Write results of the appending to the designated output file.
	  IF (.NOT. Error) THEN
	    Outfil = OutfileA
c***        Display variable names on the screen.
	    CALL Display Variables(Nvar)
	    CALL Write to Output File 
	  ENDIF
	ENDIF
      ENDIF

  85  IF (Var Count .EQ. 999) THEN
         CALL Flash (20, 2,
     +     'ERROR - Too many variables in files...(Press any key)')
      ENDIF
      CALL Close Scratch Files

      RETURN
      END
C**********************************************************************

