$LARGE
C*********************************************************************
C*** File Name: HPLIB.FOR (HPGL plotting routines for the program 
C                          HPPLOT)
C**********************************************************************
C SUBROUTINE ZSETUP (OUTPUT FILE, UNIT, IERR, TRUE SCALE)
C
C Formal Arguments:
C  OUTPUT FILE  - Name of the HPGL file.
C  UNIT         - External unit specifier.
C  IERR         - Integer variable set to 1 if an error occurs.
C  TRUE SCALE   - Indicates whether or not graph is set for true
C                 scale.
C
C This routine uses the HPGL commands 'IP' and 'IW' to initialize
C the minimum and maximum coordinates for the plotter.
C
	SUBROUTINE Zsetup (Output File, Unit, Ierr, True Scale)

	PARAMETER (MAXBUF = 1024)

	COMMON	/COMBLOCK/ Output Unit, Comport
	COMMON  /PENUPDN/  Up
	COMMON  /XYCURSOR/ Xcursor, Ycursor
	COMMON  /DEVICE /  MinX, MaxX, MinY, MaxY
	COMMON  /OUTBUF/   Buffer, Buffer Length, Buffer Size

	CHARACTER*1024	Buffer
	INTEGER*4	Buffer Length, Buffer Size
	INTEGER*4	MinX, MaxX, MinY, MaxY
	REAL*4		Xcursor, YCursor
	CHARACTER	Output File*65, Tbuffer*32
	INTEGER*4	Output Unit
	LOGICAL		Up, ComPort, True Scale 
	INTEGER*4	Ierr, Unit, P1X, P2X, P1Y, P2Y

        DATA p1X /250/
        DATA P1y /596/
        DATA p2x /10250/
        DATA p2y /7796/

	Output Unit 	= Unit
	Up 		= .TRUE.
	Xcursor		= 0.0
	Ycursor		= 0.0
	Ierr 		= 0
	Buffer Length 	= 0


  5	IF (Output File(1:4) .EQ. 'COM1' .OR. 
     +      Output File (1:4) .EQ. 'com1') THEN
  	   CALL Flash (20, 2, 'Get plotter Ready, then press any key')
	   OPEN (UNIT = Output Unit, FILE = 'COM1', STATUS = 'OLD',
     +           ERR = 999)
	   ComPort 	= .TRUE.
	   Buffer Size 	= MAXBUF
	ELSE
	   OPEN (UNIT = Output Unit, FILE = Output File, ERR = 999)
	   ComPort 	= .FALSE.
	   Buffer Size 	= 75 
	ENDIF

	IF (ComPort) THEN
 10	   WRITE (Output Unit, *) ' ',CHAR(27)//'.J'
	   WRITE (Output Unit, *)  ' OP;'
	   READ  (Output Unit, *, ERR = 888) MinX, MinY, MaxX, MaxY
	   WRITE (Output Unit, *)  CHAR(27)//'.L'
	   READ  (Output Unit, *, ERR = 888) Buffer Size
	   IF (Buffer Size .GT. MAXBUF) Buffer Size = MAXBUF
	ELSE
 888	   CONTINUE
c           MinX = P1X
c           MaxX = p2X
c           MinY = p1Y
c           MaxY = p2y
           IF (True Scale) THEN
c              MaxX = p1x + (p2y - p1y)
c              p2x = MaxX
               p2x = p1x + (p2y - p1y)
           ENDIF
	ENDIF

	CALL Buffer Out (' IN; VS5;',9)
	
        WRITE (TBuffer, 15) p1x, p1y, p2x, p2y
 15	FORMAT ('  IP',3(I6,','),I6,';')
	CALL Buffer Out (Tbuffer, 32)

c	WRITE (TBuffer, 20) MinX, MinY, MaxX, MaxY
c 20	FORMAT ('  IW',3(I6,','),I6,';')
c	CALL Buffer Out (Tbuffer, 32)
	
        CALL Zlocat (p1x, p2x, p1y, p2y)

        RETURN

 999	Ierr = 1
	
        RETURN
	END	

C**********************************************************************
C SUBROUTINE BUFFER OUT (TBUFFER, LENGTH)
C
C Formal Arguments:
C  TBUFFER   -  Temporary buffer.
C  LENGTH    -  Number of characters in the command.
C 
C This routine loads a plotting instruction into the output buffer  
C for execution.
C
	SUBROUTINE Buffer Out (Tbuffer, Length)
	
        COMMON	/COMBLOCK/ Output Unit, Comport
	COMMON  /OUTBUF/   Buffer, Buffer Length, Buffer Size

	CHARACTER*1024	Buffer
	CHARACTER	Tbuffer*(*)
	INTEGER*4	Buffer Length, Buffer Size, Length
	INTEGER*4	Output Unit, P1, P2
	LOGICAL 	ComPort

C *** check if buffer should be sent to plotter, or file
	IF ((Buffer Length + Length) .GT. Buffer Size) THEN
           CALL Flush Buffer
	   Buffer Length = 0
	ENDIF

C *** store plotting instructions into the buffer
	P1 = Buffer Length + 1
	P2 = P1 + Length - 1
	Buffer (P1:P2) = Tbuffer (1:Length)
	Buffer Length = P2
	
        RETURN
	END
	

C**********************************************************************
C SUBROUTINE FLUSH BUFFER
C
C This routine flushes the output buffer.
C
	SUBROUTINE Flush Buffer

	COMMON	/COMBLOCK/ Output Unit, Comport
	COMMON  /OUTBUF/   Buffer, Buffer Length, Buffer Size

	CHARACTER*1024	Buffer
	INTEGER*4	Buffer Length 
        LOGICAL 	ComPort
	INTEGER*4	Output Unit, Status

	CALL Compress (Buffer, Buffer Length)
	IF (Buffer Length .EQ. 0) RETURN

	IF (.NOT. Comport) THEN
	   WRITE (Output Unit, *) Buffer (1:Buffer Length)
	   RETURN
	ELSE

  5	   DO 10 I = 1, 32000
 10	   CONTINUE

 15	   WRITE (Output Unit, *, ERR = 15) CHAR(27)//'.O'
	   READ  (Output Unit, *) Status

	   IF (Status .EQ. 8) THEN
	      WRITE (Output Unit, *, ERR = 15) Buffer (1:Buffer Length)
	      RETURN
           ELSE
	      GOTO 5
	   ENDIF                  
	ENDIF

	END	

C**********************************************************************
C SUBROUTINE COMPRESS (BUF, BLENGTH)
C
C Formal Arguments:
C  Buf	    - character string 
C  Blength  - length of character string
C
C  This routine removes embedded blanks (if any) from the
C  character string Buf, and modifies Blength to contain the
C  number of compressed characters.
C 
	SUBROUTINE COMPRESS (BUF, BLENGTH)
	CHARACTER Buf*(*), BufCopy*80 
	INTEGER*4 Blength, NewLength

	NewLength = 0
	BufCopy   = ' '
	DO 10 I = 1, BLength
	   IF (Buf (I:I) .NE. ' ') THEN
	       NewLength = NewLength + 1
	       BufCopy (NewLength:NewLength) = Buf (I:I)
	   ENDIF
 10	CONTINUE
	Buf     = BufCopy
	Blength = NewLength 
	RETURN
	END

C**********************************************************************
C SUBROUTINE ZLOCAT (X1, X2, Y1, Y2)
C
C Formal Arguments:
C  X1, Y1   - Minimum X and Y coordinates.
C  X2, Y2   - Maximum X and Y coordinates.
C 
C This routine uses the HPGL command 'IW' to set the minimum 
C and maximum coordinates for the plotter.
C

	SUBROUTINE Zlocat (X1, X2, Y1, Y2)

c	COMMON	/COMBLOCK/ Output Unit, Comport
	COMMON  /DEVICE /  MinX, MaxX, MinY, MaxY

c	LOGICAL 	ComPort
c	INTEGER*4	Output Unit
        INTEGER*4       X1, X2, Y1, Y2
	INTEGER*4	MinX, MaxX, MinY, MaxY
	CHARACTER*32	Tbuffer

        MinX = X1
        MaxX = X2
        MinY = Y1
        MaxY = Y2
               
	WRITE (TBuffer, 20) MinX, MinY, MaxX, MaxY
 20	FORMAT ('  IW',3(I6,','),I6,';')

	CALL Buffer Out (Tbuffer, 32)

	RETURN
	END
	

C**********************************************************************
C SUBROUTINE GETDEV (X1, X2, Y1, Y2)
C
C Formal Arguments:
C  X1, X2    - Minimum and maximum  X coordinates.
C  Y1, Y2    - Minimum and maximum  Y coordinates.
C 
C This routine gets the minimum and maximum device coordinates.
C
	SUBROUTINE GetDev (X1, X2, Y1, Y2)

	COMMON  /DEVICE /	MinX, MaxX, MinY, MaxY
	INTEGER*4  		MinX, MaxX, MinY, MaxY
	INTEGER*4		X1,   X2,   Y1,   Y2

	X1 = MinX 
	X2 = MaxX 
	Y1 = MinY 
	Y2 = MaxY 
 
	RETURN
	END

C**********************************************************************
C SUBROUTINE GETWOR (XMIN, XMAX, YMIN, YMAX)
C
C Formal Arguments:
C  XMIN, YMIN   - Minimum X and Y coordinates.
C  XMAX, YMAX   - Maximum X and Y coordinates.
C
C This routine gets the user coordinate system extents.
C

	SUBROUTINE GetWor (Xmin, Xmax, Ymin, Ymax)

        COMMON  /WORLDEXT/ XminG, XmaxG, YminG, YmaxG

	REAL*4		Xmin, Xmax, Ymin, Ymax
	REAL*4		XminG, XmaxG, YminG, YmaxG

        Xmin = XminG
        Ymin = YminG
        Xmax = XmaxG
        Ymax = YmaxG

        RETURN
        END

C**********************************************************************
C SUBROUTINE ZSCALE (XMIN, XMAX, YMIN, YMAX)
C
C Formal Arguments:
C  XMIN, YMIN  - Minimum X and Y coordinates.
C  XMAX, YMAX  - Maximum X and Y coordinates.
C
C This routine uses the HPGL command 'SC' to initialize the 
C user coordinate system.
C
	SUBROUTINE Zscale (Xmin, Xmax, Ymin, Ymax)

c	COMMON	/COMBLOCK/ Output Unit, Comport
	COMMON  /XYCURSOR/ Xcursor, Ycursor
        COMMON  /WORLDEXT/ XminG, XmaxG, YminG, YmaxG

	REAL*4		Xcursor, YCursor
c	LOGICAL 	ComPort
c	INTEGER*4	Output Unit
	REAL*4		Xmin, Xmax, Ymin, Ymax
	REAL*4		XminG, XmaxG, YminG, YmaxG
	CHARACTER*32	Tbuffer

        XminG = Xmin
        YminG = Ymin
        XmaxG = Xmax
        YmaxG = Ymax

	WRITE (TBuffer, 10) Xmin, Xmax, Ymin, Ymax
 10	FORMAT ('  SC',3(F6.1,','),F6.1,';')

	CALL Buffer Out (Tbuffer, 32)

	Xcursor = Xmin
	Ycursor = Ymin

	RETURN
	END
	

C**********************************************************************
C SUBROUTINE ROTATE (ANGLE)
C
C Formal Arguments:
C  ANGLE    - Angle in degrees.
C
C This routine uses the HPGL command 'RO' to rotate the coordinate
C system by the specified angle.
C

	SUBROUTINE Rotate (Angle)
   
c	COMMON	/COMBLOCK/ Output Unit, Comport

c	LOGICAL 	ComPort
c	INTEGER*4	Output Unit
	REAL*4		Angle
	CHARACTER*8	Tbuffer

	WRITE (TBuffer, 10) Angle
 10	FORMAT (' RO',F4.0,';')

	CALL Buffer Out (Tbuffer, 8)

	RETURN
	END
	
C**********************************************************************
C SUBROUTINE ZWHERE (X,Y)
C
C Formal Arguments:
C  X,Y    -  X and Y coordinates of the pen location.
C
C This routine returns the X and Y coordinates of the current
C pen location.
C
	SUBROUTINE Zwhere (X, Y)

c	COMMON	/COMBLOCK/ Output Unit, Comport
c	COMMON  /PENUPDN/  Up
	COMMON  /XYCURSOR/ Xcursor, Ycursor

	REAL*4	    Xcursor, YCursor, X, Y
c	LOGICAL     ComPort, Up
c	INTEGER*4   Output Unit, Pen

	X = Xcursor
	Y = Ycursor
        
	RETURN
	END

C**********************************************************************
C SUBROUTINE ZCOLOR (PEN)
C
C Formal Arguments:
C  PEN       - Number of the plotter pen to select.
C
C This routine uses the HPGL command 'SP' to select a new pen color.
C

	SUBROUTINE Zcolor (Pen)

c	COMMON	/COMBLOCK/ Output Unit, Comport

c	LOGICAL 	ComPort
c	INTEGER*4	Output Unit, Pen
	INTEGER*4	Pen
	CHARACTER*8	Tbuffer

	WRITE (TBuffer, 10) Pen
 10	FORMAT (' SP',I4,';')

	CALL Buffer Out (Tbuffer, 8)
				 
	RETURN
	END

C**********************************************************************
C SUBROUTINE SET SPEED (SPEED)
C
C Formal Arguments:
C  SPEED   -  Pen speed.
C
C This routine uses the HPGL command 'VS' to set the velocity of 
C the plotter pen.
C
        SUBROUTINE Set Speed (Speed)

c	COMMON	/COMBLOCK/ Output Unit, Comport

c	LOGICAL 	ComPort
c	INTEGER*4	Output Unit
c	LOGICAL		Up
	REAL*4		Speed
	CHARACTER*8	Tbuffer

	WRITE (TBuffer, 10) Speed
 10	FORMAT (' VS',F4.0,';')

	CALL Buffer Out (Tbuffer, 8)

	RETURN
	END

C**********************************************************************
C SUBROUTINE LINE TYPE (TYPE)
C
C Formal Arguments:
C  TYPE     - Number of the line pattern.
C
C This routine uses the HPGL command 'LT' to set the current line type.
C

	SUBROUTINE Line Type (Type)

c	COMMON	/COMBLOCK/ Output Unit, Comport

c	LOGICAL 	ComPort
c	INTEGER*4	Output Unit
	INTEGER*4	Type
	CHARACTER*8	Tbuffer

	WRITE (TBuffer, 10) Type
 10	FORMAT (' LT',I4,';')

	CALL Buffer Out (Tbuffer, 8)

	RETURN
	END

C**********************************************************************
C SUBROUTINE ZLBDIR (ANGLE)
C
C Formal Arguments:
C  ANGLE     - Angle to plot label.
C
C This routine uses the HPGL command 'DR' to set the direction 
C to plot the label.
C

	SUBROUTINE ZlbDir (Angle)

	PARAMETER (DTR = 0.01745329)

	COMMON	/COMBLOCK/ Output Unit, Comport

c	LOGICAL 	ComPort
c	INTEGER*4	Output Unit
	REAL*4		Angle, Rise, Run
	CHARACTER*30	Tbuffer
	
	Rise = SIN (Angle*DTR)
	Run  = COS (Angle*DTR)

	WRITE (TBuffer, 10) Run, Rise
 10	FORMAT ('DR',F6.0,',',F6.0,';') 

	CALL Buffer Out (TBuffer, 16)
 
	RETURN
	END
	

C**********************************************************************
C SUBROUTINE PEN UP
C
C This routine uses the HPGL command 'PU' to raise the plotter pen.
C
	SUBROUTINE Pen Up

c	COMMON	/COMBLOCK/ Output Unit, Comport
	COMMON  /PENUPDN/  Up

	LOGICAL 	Up
c	LOGICAL 	ComPort, Up
c	INTEGER*4	Output Unit

	CALL Buffer Out (' PU;', 4)
	Up = .TRUE.		 

	RETURN
	END

C**********************************************************************
C SUBROUTINE PEN DOWN
C
C This routine uses the HPGL command 'PD' to lower the plotter pen.
C
	SUBROUTINE Pen Down

c	COMMON	/COMBLOCK/ Output Unit, Comport
	COMMON  /PENUPDN/  Up

c	LOGICAL 	ComPort, Up
	LOGICAL 	Up
c	INTEGER*4	Output Unit

	CALL Buffer Out (' PD;', 4)
	Up = .FALSE.

	RETURN
	END

C**********************************************************************
C SUBROUTINE ZPLOT (X, Y)
C
C Formal Arguments:
C  X, Y    - Coordinates to plot line.
C
C This routine uses the HPGL command 'PA' to plot a line.
C
 	SUBROUTINE Zplot (X, Y)

c	COMMON	/COMBLOCK/ Output Unit, Comport
	COMMON  /PENUPDN/  Up
	COMMON  /XYCURSOR/ Xcursor, Ycursor

	REAL*4		Xcursor, YCursor
c	LOGICAL 	ComPort, Up
	LOGICAL 	Up
c	INTEGER*4	Output Unit
	REAL*4		X, Y
	CHARACTER*16	TBuffer

	IF (Up) CALL Pen Down
	
        WRITE (TBuffer, 10) X, Y
 10	FORMAT ('PA',F6.1,',',F6.1,';') 

	CALL Buffer Out (TBuffer, 16)
	
        Xcursor = X
	Ycursor = Y

	RETURN
	END
				
C**********************************************************************
C SUBROUTINE ZMOVE (X,Y) 
C
C Formal Arguments:
C  X,Y   - Location to move pen.
C
C This routine uses the HPGL command 'PA' to move the plotter 
C pen to the specified location.
C
	SUBROUTINE Zmove (X, Y)

c	COMMON	/COMBLOCK/ Output Unit, Comport
	COMMON  /PENUPDN/  Up
	COMMON  /XYCURSOR/ Xcursor, Ycursor

	REAL*4		Xcursor, YCursor
c	LOGICAL 	ComPort, Up
	LOGICAL 	Up
c	INTEGER*4	Output Unit
	REAL*4		X, Y
	CHARACTER*16	TBuffer

	IF (.NOT. Up) CALL Pen Up

	WRITE (TBuffer, 10) X, Y
 10	FORMAT ('PA',F6.1,',',F6.1,';') 

	CALL Buffer Out (TBuffer, 16)

	Xcursor = X
	Ycursor = Y

	RETURN
	END
	
C**********************************************************************
C SUBROUTINE ZLABEL (TEXT)
C
C Formal Arguments:
C  Text   - Information to plot.
C
C This routine writes text using the plotter's hardware font.
C

	SUBROUTINE ZLABEL (Text)
 
c	COMMON	/COMBLOCK/ Output Unit, Comport

c	LOGICAL 	ComPort
c	INTEGER*4	Output Unit
	CHARACTER	Text*(*), TBuffer*128
	INTEGER*4	N			

        
	N = INDEX (Text, CHAR(0))
	IF (N .EQ. 0) N = INDEX (Text, '\')
	IF (N .LE. 1) RETURN
	IF (N .GT. 125) N = 125

	TBuffer = ' LB'//Text (1:N-1)//CHAR(3)
	CALL Buffer Out (TBuffer, N+3)
		    
	RETURN
	END


 	


