$LARGE
C*********************************************************************
C*** File Name: MHPPLOT.FOR (Source code for the program HPPLOT)
C**********************************************************************
C SUBROUTINE MHPPLOT (INPUT FILE, STAT, OUTPUT FILE, TRUE SCALE)
C 
C Formal Arguments:
C  INPUT FILE   -  Name of the metacode file.
C  STAT         -  Indicates the status of the input file.
C  OUTPUT FILE  -  Name of the HPGL file to be created.
C  TRUE SCALE   -  Indicates whether or not the graph is in true scale.
C
C This program translates NCAR metacode to graphic output on one of  
C the HP family plotters.
C
        SUBROUTINE MHPPLOT (Input File,Stat,Output File,True Scale)

        COMMON	/COM/ 		Comport
        COMMON	/CENTER/ 	ICENT
        COMMON	/DASHES/ 	IDASH
        COMMON	/PTMODE/ 	PTMODE,MARKER
        COMMON	/XY/ 		MX,MY
        COMMON	/ENDHP/ 	EXITHP

        INTEGER*4      IERR
        CHARACTER*3    Stat
        CHARACTER*65   Input File, Output File	
        INTEGER*2      IWORD
        LOGICAL        PTMODE, FEXIST, COMPORT,EXITHP, True Scale
        LOGICAL*1      Try Again, GetYesNo


C *** Set current beam/pen position to (0,0), turn points mode off,
C *** define left-justified characters, and define solid dash pattern.
        MX 	= 0
        MY 	= 0
        PTMODE 	= .FALSE.
        ICENT 	= 0
        IDASH 	= 0

C *** Open the NCAR metacode file.
        CALL OPENMC (Input File, Stat)

C *** Open Hershy character lookup file. 
        CALL HCINIT (10)

C *** Set up the plotting parameters, and begin processing metacode
        CALL FRAME (Output File, True Scale)
        EXITHP = .FALSE.

C *** Process a four-byte, two-byte, or multi-byte metacode instruction.
   10   CALL GET16(IWORD)
        IF (.NOT. EXITHP ) THEN
           IF (IWORD .EQ. #A020) GOTO 10
           ITYPE = IBITS (IWORD,14,2)
           IF (ITYPE .LE. 1) THEN
              CALL FOUR (IWORD)
           ELSE IF (ITYPE .EQ. 2) THEN
              CALL TWO (IWORD)
           ELSE IF (ITYPE .NE. 0) THEN
              CALL MULTI (IWORD)
           ENDIF

C *** Process the next metacode instruction.
           GO TO 10
      ENDIF
      CALL  Flush Buffer	
      CLOSE (8)
      CLOSE (12)
      RETURN
 999  STOP
      END

C********************************************************************
C SUBROUTINE COLOR (LETTER)
C
C Formal Arguments:
C  LETTER          - Color to set.
C
C Local Variables:
C  COLORS          - Character string of the colors available.
C  ICOLOR          - Array of integers representing the colors.
C
C This routine identifies the color to set and then calls Zcolor to 
C change pen colors.
C
        SUBROUTINE COLOR (LETTER)

        COMMON/PENS/ IPEN1,IPEN2
        CHARACTER COLORS*36,LETTER*3,PCOLOR*3
        INTEGER*4 KCOLOR
        DIMENSION ICOLOR(9)

        DATA COLORS/'BLA:RED:BLU:GRE:YEL:BRO:ORA:PUR:MAG:'/
        DATA ICOLOR/   1,  2,  3,  4,  5,  6,  1,  2,  3/


        I = (INDEX (Colors, Letter) / 4) + 1
        IF (I .LE. 1 .OR. I .GT. 9)  THEN
	   Kcolor = 1
        ELSE
           Kcolor  = Icolor (i) 
        ENDIF
        CALL Zcolor (KCOLOR)		

        RETURN
        END

C**********************************************************************
C SUBROUTINE DASH
C
C This routine converts a NCAR binary dash pattern into a PLOT 10 
C decimal dash pattern.
C
        SUBROUTINE DASH
 
        INTEGER*2 IWORD
        LOGICAL SOLID
        COMMON/DASHES/ IDASH

C *** Get the NCAR dash pattern.
        CALL GET16(IWORD)

C *** Build the PLOT 10 dash pattern.
        SOLID = .TRUE.
        IDASH = 0
        DO 10 I=0,7
           IBIT = IBITS(IWORD,I,1)
           IF (IBIT .EQ. 0) THEN
              SOLID = .FALSE.
              IDASH = IDASH * 10 + 6
           ELSE
              IDASH = IDASH * 10 + 3
           ENDIF
   10   CONTINUE

C *** Check for the special case of a solid line.
        IF (SOLID) IDASH = 0

        RETURN
        END

C**********************************************************************
C SUBROUTINE FINISH (ISTAT)
C
C Formal Arguments:
C  ISTAT        - Indicates status.
C
C This routine terminates plotting. A message is issued if the 
C metacode file was not properly formatted or not properly closed.
C
        SUBROUTINE FINISH(ISTAT)
        
        COMMON /ENDHP/ EXITHP

        LOGICAL   EXITHP

        EXITHP = .TRUE.
        CALL CLEAR SCREEN

C *** Issue message if metacode file was not properly formatted or not
C *** properly closed.

      	IF (ISTAT. EQ. -1) THEN
          WRITE(6,*) 'Metacode file is not properly formatted.',CHAR(7)
      	ELSE IF (ISTAT .EQ. -2) THEN
          WRITE(6,*) 'Unexpected end of metacode file.',CHAR(7)
      	ELSE IF (ISTAT .EQ. -3) THEN
          WRITE(6,*) 'Error opening metacode file.',CHAR(7)
      	ENDIF

      	CALL ZCOLOR (0)

      	RETURN
      	END

C**********************************************************************
C SUBROUTINE FOUR (IWORD)
C
C Formal Arguments:
C  IWORD            - Metacode instruction to process.
C
C This routine processes a four-byte (absolute position) 
C metacode instruction by getting the X and Y coordinates and the
C pen control values, then moving or drawing to the specified position.
C 
        SUBROUTINE FOUR (IWORD)

        INTEGER*2 IWORD
        COMMON/XY/ MX,MY

C *** Get the x-coordinate, y-coordinate, and pen control values.
        MX = IWORD
        CALL GET16 (IWORD)
        MY = IBITS (IWORD,0,15)
        IPEN = IBITS (IWORD,15,1)

C *** Move or draw to the specified position.
        CALL PLOT (MX,MY,IPEN)

        RETURN
        END

C**********************************************************************
C SUBROUTINE FRAME (OUTPUT FILE, TRUE SCALE)
C
C Formal Arguments:
C  OUTPUT FILE    - Name of the HPGL file.
C  TRUE SCALE     - Indicates whether the graph is in true scale.
C
C This routine advances to the next metacode frame.
C
        SUBROUTINE FRAME (Output File, True Scale)

        PARAMETER (MAXBUF = 1440) 

        COMMON/COM/ Comport
        COMMON/IPTR/ IPTR
        COMMON/XY/ MX,MY

        INTEGER*4     IERR
        CHARACTER*65  Output File
        LOGICAL       ComPort, True Scale
        LOGICAL*1     Try Again, GetYesNo

C *** Reset the current beam/pen position to (0,0).
        MX = 0
        MY = 0

C *** Metacode instructions for the next frame begin in the next record.
C *** Force the next record to be read the next time GET16 is called.
        IPTR=MAXBUF

C *** Get the name of the com1port or file to be used
        ComPort = .FALSE.
        IF (Output File .EQ. 'COM1' .OR. Output File .EQ. 'com1') 
     +     ComPort = .TRUE.
        CLOSE (12)
        CALL ZSETUP (Output File, 12, IERR, True Scale)
        IF (IERR .EQ. 1) THEN
           CALL FLASH (20, 2, 'Error opening '//Output File(1:20))
           CALL Clear Screen
           RETURN
      	ENDIF

        IF (ComPort) CALL GRWAIT

C *** Re-initialize internal plotting parameters
        CALL INIT (True Scale)     

        RETURN
        END

C**********************************************************************
C SUBROUTINE GET16 (IWORD)
C
C Formal Arguments:
C  IWORD          - Metacode instruction to process.
C
C This routine gets the next 16 bits of metacode instructions to 
C process.
C
        SUBROUTINE GET16 (IWORD)

C *** Get the next 16 bits of metacode.
C
        PARAMETER (MAXBUF = 1440) 
        INTEGER*2 IWORD, IWORD2
        INTEGER*1 META (MAXBUF), IBYTE1, IBYTE2

        COMMON /ENDHP/ EXITHP
        COMMON/IPTR/ IPTR
        COMMON STARTED
        LOGICAL STARTED

        IF (.NOT. STARTED) IPTR = MAXBUF 

C *** If necessary, read the next metacode record.  Check for the "end
C *** of metacode" signal (first 16 bits are zero) and also for the
C *** flag which indicates that this is metacode (next 4 bits are 0010).
        IF (IPTR .EQ. MAXBUF) THEN
           READ(8,END=99,ERR=98) (META(I),META(I-1),I=2,MAXBUF,2)
           IF (IWORD2 (META(1),META(2)) .EQ. 0) THEN
              CALL FINISH (1)
              RETURN
           ENDIF
           IF (IWORD2 (META(3),META(4)) .NE. #2800) THEN
              CALL FINISH (-1)
              RETURN
           ENDIF
           IPTR = 4
        ENDIF


C *** Return the next 16-bit metacode array element.
        IPTR	= IPTR + 1
        IBYTE1 	= META (IPTR)
        IPTR	= IPTR + 1
        IBYTE2 	= META (IPTR)  	
        IWORD	= IWORD2 (IBYTE1, IBYTE2)
        STARTED = .TRUE.
        
        RETURN

C *** If an error occurred during the read operation, the metacode file
C *** is not formatted properly.
   98   CALL FINISH (-2)
        RETURN

C *** If the end-of-file was reached on the metacode file, the user's
C *** application program did not terminate normally.
   99   CALL FINISH(1)
        RETURN

        END

C**********************************************************************
C SUBROUTINE GRWAIT
C 
C This routine waits for a keypress after a frame has been plotted..
C
C
	SUBROUTINE GRWAIT

	CALL FLASH (20, 2, 'Get the plotter ready ... (press any key')

	RETURN
	END

C**********************************************************************
C INTEGER*2 FUNCTION IWORD2 (IBYTE1, IBYTE2)
C
C Formal Arguments:
C  IBYTE1,IBYTE2    - Metacode instructions.
C
C This routine combines two bytes (IBYTE1,IBYTE2) into an INTEGER*2 word 
C (IBYTE2 will be in bits 0-7).
C
C
	INTEGER*2 FUNCTION IWORD2 (IBYTE1, IBYTE2)

        INTEGER*1 IBYTE1, IBYTE2 
	INTEGER   JBYTE1, JBYTE2

	JBYTE2 = IBYTE2
	JBYTE2 = IAND (JBYTE2, #00FF)
        JBYTE1 = IBYTE1
        JBYTE1 = IAND (ISHC (JBYTE1, 8), #FF00)
       
	IWORD2  = IOR(JBYTE1, JBYTE2)
	
        RETURN
	END
        
C**********************************************************************
C FUNCTION IBITS (IWORD, ISTART, NBITS)
C
C Formal Arguments:
C  IWORD       - Metacode instruction.
C  ISTART      - Position to start.
C  NBITS       - Number of bits.
C
C This routine extracts from IWORD the bits from ISTART, to
C ISTART+NBITS-1.
C
C
	FUNCTION IBITS (IWORD, ISTART, NBITS)
	
        INTEGER*2 IWORD
	INTEGER*4 ISTART, NBITS, JWORD

	JWORD = 0
	DO 10 I = 1, NBITS
	   IF (BTEST (IWORD, ISTART+I-1)) THEN
               JWORD = IBSET (JWORD, I-1)
	   ENDIF
 10	CONTINUE
	IBITS = JWORD
	
        RETURN
	END 

C**********************************************************************
C SUBROUTINE INIT (TRUE SCALE)
C
C Formal Arguments:
C  TRUE SCALE   - Indicates whether  the graph is in true scale.
C
C This routine initializes the plotting routines.
C
        SUBROUTINE INIT (True Scale)

        COMMON /CHAR/   ASPECT, FACTOR, VSIZE
        COMMON /DEVICE/ IDEV
        COMMON /PENS/   IPEN1,IPEN2

        LOGICAL True Scale
        REAL*4	XMIN, XMAX, YMIN, YMAX
        DATA	XMIN /0./, YMIN /0./, XMAX /3277./, YMAX /3277./

        FACTOR = 1.0
	IF (True Scale) THEN
           ASPECT = 1.
	ELSE
	   ASPECT = 1.2
	ENDIF

        CALL ZCOLOR (1)

C *** Scale virtual window to metacode limits.
        CALL ZSCALE (XMIN, XMAX, YMIN, YMAX)

C *** Establish default character width as 128 metacode units.
        IW = 128
        VSIZE = (REAL (IW)/10.) / ASPECT

C *** Reset hershey character defaults
        CALL HCSIZE (REAL (IW) / 10.)
        CALL HCROT  (0.0)
        CALL HCFONT (2)
        CALL HCASP  (ASPECT)	

        RETURN
        END

C**********************************************************************
C SUBROUTINE MULTI (IWORD)
C
C Formal Arguments:
C  IWORD       -  Metacode instruction to process.
C
C This routine processes a multi-byte (control function)
C metacode instruction.
C
        SUBROUTINE MULTI (IWORD)
       
        INTEGER*2 IWORD,OPCODE
        LOGICAL PTMODE
        COMMON/PTMODE/ PTMODE,MARKER

C *** Get the instruction opcode and the number of bytes it contains.
        OPCODE = IBITS (IWORD,8,6)
        NBYTES = IBITS (IWORD,0,8)

C *** Process "plot character" instruction.
        IF (OPCODE .EQ. 33) THEN
           CALL PLOTCH (NBYTES)

C *** Process "frame advance" instruction.
        ELSE IF (OPCODE .EQ. 34) THEN
          CALL Finish (1)

C *** Process "set option" instruction.
        ELSE IF (OPCODE .EQ. 35) THEN
           CALL OPTION

C *** Ignore "range information" instruction.
        ELSE IF (OPCODE .EQ. 39) THEN
           CALL GET16 (IWORD)
           CALL GET16 (IWORD)
           CALL GET16 (IWORD)
           CALL GET16 (IWORD)

C *** Process "points mode" instruction.
        ELSE IF (OPCODE .EQ. 40) THEN
           CALL GET16 (IWORD)
           PTMODE = (IBITS(IWORD,8,8) .GT. 0)
           IF (PTMODE) MARKER = IBITS (IWORD,0,8)
        ENDIF
      
        RETURN
        END

C**********************************************************************
C SUBROUTINE OPENMC (FILE NAME, STAT)
C
C Formal Arguments:
C  FILE NAME  - Name of the metacode file.
C  STAT       - Status of the metacode file.
C
C Opens and checks the status of the metacode file.
C
      SUBROUTINE OPENMC	(File Name, Stat)

      CHARACTER	FILE NAME*(*), Stat*3	
      INTEGER*1 META(4)
      INTEGER*2 IWORD2	

C *** Check if the metacode file exists.
      OPEN(UNIT=8,STATUS=Stat,FILE=File Name,
     +     FORM='BINARY',RECL=1440,ERR=30)

C *** Check if the file contains metacode.
      READ(8,END=20,ERR=10) META(2),META(1),META(4),META(3)
      IF(IWORD2(META(3),META(4)).NE. #2800) CALL FINISH(-1)

C *** Metacode file appears to be correct.
      REWIND 8
      RETURN

C *** Error reading first record of metacode file.
   10 CALL FINISH(-1)

C *** Metacode file is empty.
   20 CALL FINISH(-2)

C *** Metacode file could not be opened.
   30 CALL FINISH(-3)

      END

C**********************************************************************
C SUBROUTINE OPTION
C
C This routine processes a "set option" metacode instruction.
C
        SUBROUTINE OPTION

        CHARACTER	LETTER*3
        INTEGER*2 	IWORD, ICH
        COMMON/CENTER/ 	ICENT
        COMMON/CHAR/ 	ASPECT,FACTOR,VSIZE
        COMMON/DASHES/ 	IDASH

C *** Extract option code.
        CALL GET16 (IWORD)
        IOPT = IBITS (IWORD,8,8)

C *** Process "character orientation" option code.
        IF (IOPT .EQ. 3) THEN
           CALL GET16 (IWORD)
           ANGLE = REAL (IWORD)
	   CALL HCROT (ANGLE)

C *** Process "character width" option code.
        ELSE IF (IOPT .EQ. 4) THEN
           CALL GET16 (IWORD)
           VSIZE = (REAL(IWORD)/10.0) / ASPECT
	   CALL HCSIZE (REAL(IWORD)/10.)

C *** Process "font" option code.
        ELSE IF (IOPT .EQ. 5) THEN
           ITALIC = IBITS(IWORD,0,8)
	   IF (ITALIC .EQ. 1) THEN
              CALL HCFONT (9)
	   ELSE
	      CALL HCFONT (2)
	   ENDIF 

C *** Process "dash pattern" option code.
        ELSE IF (IOPT .EQ. 6) THEN
           CALL DASH

C *** Ignore third and fourth bytes of "spot size" option code.
        ELSE IF (IOPT .EQ. 7) THEN
           CALL GET16 (IWORD)

C *** Process "centering" option code.
        ELSE IF (IOPT .EQ. 8) THEN
           ICENT = IBITS (IWORD,0,8)

C *** Process "color" option code.
        ELSE IF (IOPT .EQ. 9) THEN
           ICH = IBITS (IWORD,0,8)
	   WRITE (LETTER(1:1),'(A1)') ICH
           CALL GET16 (IWORD)
           ICH = IBITS (IWORD,8,8)
	   WRITE (LETTER(2:2),'(A1)') ICH
           ICH = IBITS (IWORD,0,8)
	   WRITE (LETTER(3:3),'(A1)') ICH
           CALL COLOR (LETTER)
        ENDIF

        RETURN
        END

C**********************************************************************
C SUBROUTINE PLOT (MX,MY,IPEN)
C
C Formal Arguments:
C  MX, MY      - Coordinates of the position.
C  IPEN        - Line pattern.
C
C  This routine processes an "absolute position" metacode instruction.
C
        SUBROUTINE PLOT (MX,MY,IPEN)
    
        LOGICAL		PTMODE
        COMMON/DASHES/ 	IDASH
        COMMON/PTMODE/ 	PTMODE,MARKER
        COMMON/CHAR/ 	ASPECT,FACTOR,VSIZE

C *** Scale the metacode coordinates (MX,MY) to virtual window
C *** coordinates (X,Y).
        X = REAL(MX)/10.
        Y = REAL(MY)/10.

C *** Move or draw to the specified position.  Draw with a dashed line
C *** if a dash pattern has been defined.
        IF (IPEN .EQ. 0) THEN
           CALL ZMOVE (X,Y)
        ELSE IF (IDASH .EQ. 0) THEN
           CALL ZPLOT (X,Y)
        ELSE
           CALL ZPLOT (X,Y)
        ENDIF

C *** If points mode is on, plot a point (MARKER = 0) or marker
C *** (MARKER > 0) at the specified position.
        IF (PTMODE) THEN
           IF (MARKER .EQ. 0) THEN
              CALL ZMOVE (X,Y)
	      CALL ZPLOT (X,Y)
	      CALL ZMOVE (X,Y)
           ELSE
	      CALL HCFONT (32)
 	      CALL HCASP (1.0)
              CALL ZMOVE (X,Y)
              CALL HCPOS (CHAR(MARKER),1,1)
	      CALL ZMOVE (X,Y)
	      CALL HCFONT (2)
	      CALL HCASP (ASPECT)
           ENDIF
        ENDIF

        RETURN
        END

C**********************************************************************
C SUBROUTINE PLOTCH (NCHAR)
C
C Formal Arguments:
C  NCHAR   - Number of characters.
C
C This routine plots the specified number of characters.  The 
C characters are packed two per 16-bit metacode word.
C
        SUBROUTINE PLOTCH (NCHAR)

        INTEGER*2 	IWORD
        DIMENSION 	IBUFF(256)
        CHARACTER*64   	CBUFF
        COMMON/CENTER/ 	ICENT
        COMMON/CHAR/ ASPECT, FACTOR, VSIZE
      

C *** Set the appropriate label origin code. Left-justified (ICENT=0),
C     centered (ICENT=1), or right-justified (ICENT=2).

        IF (ICENT .EQ. 0) THEN
	   LBORG = 0
        ELSE IF (ICENT .EQ. 1) THEN
	   LBORG = 1
        ELSE
	   LBORG = 2
        ENDIF

C *** Unpack the characters and store them in the buffer.
        DO 10 I = 1,NCHAR
           IF (MOD(I,2) .EQ. 1) THEN
              CALL GET16 (IWORD)
              ICH = IBITS (IWORD,8,8)
           ELSE
              ICH = IBITS (IWORD,0,8)
           ENDIF
           CBUFF(I:I) = CHAR (ICH)
   10   CONTINUE

C *** Plot the characters.
        CALL ZWHERE (TX, TY)

C check for centered symbol set
	IF (CBUFF (1:4) .EQ. '#032') THEN
          CALL ZMOVE  (TX, TY)
	ELSE
          CALL ZMOVE  (TX, TY-VSIZE*.5)
	ENDIF

        CALL HCPOS  (CBUFF, NCHAR, LBORG)
        CALL ZMOVE  (TX, TY)
       
        RETURN
        END

C**********************************************************************
C SUBROUTINE TWO (IWORD)
C
C Formal Arguments:
C  IWORD  - Metacode instruction to process.
C
C This routine processes a two-byte (relative position) metacode 
C instruction.
C
        SUBROUTINE TWO (IWORD)

        INTEGER*2 IWORD
        COMMON/XY/ MX,MY

C *** Get the x-coordinate, y-coordinate, and pen control values.
        MXINC = IBITS (IWORD,8,6)
        MYINC = IBITS (IWORD,0,6)
        IPEN = IBITS(IWORD,7,1)

C *** Add the relative coordinates to the current position.
        MX = MX + MXINC - 32
        MY = MY + MYINC - 32

C *** Move or draw to the specified position.
        CALL PLOT (MX,MY,IPEN)

        RETURN
        END      

