$LARGE
C*********************************************************************
C*** File Name: POSTPLOT.FOR (Source code for the program POSTPLOT)
C*********************************************************************
C                      PROGRAM POSTPLOT
C
C This program produces a plot of (2D) saple locations and values for 
C variables in a Geo-EAS data file.  Sample locations may be marked 
C with a symbol, value, or both.  The format for the value to be
C plotted may be specified by setting a scaling factor and the number 
C of decimal places to be used in the plotted value.  Options allow
C control of axes parameters, and titles.  A file called a 'metacode'
C file is created for redisplay or to produce a hard copy
C
C Postplot attempts to produce  'true-scale' graph on the screen, but
C for some data configurations, and internal NCAR routine overrides
C true scaling and produces a somewhat distorted graph.
C
C The main program performs the following:
C   Reads in the system defaults
C   Initializes program variables
C   Sets up the screen
C   Calls Get Screen Fld to display the screens and process the
C    user's requests
C   Writes to the system default file
C
C**********************************************************************
C Interface  statements for video status function

	interface to integer*2 function vstats[C]
     1		(c_adap[reference],ega_mon[reference],mode[reference],
     1		 at[reference],sec_mon[reference])
	integer*2 c_adap,ega_mon,mode,at,sec_mon
	end
C**********************************************************************

        PROGRAM POSTPLOT

$INCLUDE:'postplot.cmn'

      	LOGICAL Exists
          
C ***  The commented lines are used when debugging the screen 
C ***  definition file.
c      	INQUIRE	(FILE = 'postplot.scr', EXIST = Exists)
c      	IF (.NOT. Exists) THEN
c	    CALL Clear Screen
c	    WRITE (*, 10) 
c	    CALL FLASH (20, 2, 'Press any key to continue ... ')
c       ELSE
           CALL Clear Screen
           CALL Grafinit
c          CALL Read Structure ('postplot.scr')   
c 	   CALL Clear Screen
c          k = tOTAL NUMBER OF SCREENS
c          WRITE (*, *) '# fields: ',End Screen (K)
c          WRITE (*, *) '# labels: ',End Label  (K)
c          K = End Label  (Total number of Screens)
c	   WRITE (*, *) 'Pointer: ',LB Pointer (K)+LBLength(K)-1
c	   write (*, *)
c          call flash (20, 2, '*****')

           CALL Read System Defaults
           CALL Initialize
           CALL Set Up Screen
           IF (Intro) CALL Intro Screen
           CALL Get Screen Fld
           CALL Write System Defaults
           CALL Clear Screen
c	ENDIF


 10   	FORMAT (' POSTPLOT cannot operate without the file postplot.scr ',
     +	/,/,	'   This file could not be located in the default   ',
     +	/,	' directory.  Please copy this file into the current',
     +	/,	' directory and try again.',//)
	 
        END

C**************************************************************************
C SUBROUTINE GRAFINIT 
C
C  This routine calls Vstats to initialize graphics parameters and
C  sets the  global variable Devcode to the type of graphics device
C  in use.
C
	SUBROUTINE GrafInit 
	
        IMPLICIT INTEGER*4 (A-Z)
	PARAMETER (OTHER = 0, HERC = 1, CGA = 2, EGA = 3)

$INCLUDE: 'postplot.cmn'

	INTEGER*2	    Vstats, Adaptor, EgaMon, Mode
	INTEGER*2	    ATflag, Mon2, Rcode

C Get device type , and graphics mode
        RCode = Vstats (Adaptor, Ega Mon, Mode, Atflag, Mon2)
	IF (Adaptor .EQ. CGA) THEN
           Devcode  = CGA
        ELSE IF (Adaptor .EQ. EGA) THEN
	   Devcode  = EGA
        ELSE IF (Adaptor .EQ. HERC) THEN
	   Devcode  = HERC
        ELSE
           Devcode = OTHER
	ENDIF

        RETURN
	END

C************************************************************************
C SUBROUTINE INTRO SCREEN 
C
C  This routine displays the screen frame/labels/fields for the 
C  introductory screen.           

        SUBROUTINE Intro Screen

        CALL Clear Screen
        CALL Set Screen(1)
        CALL Display Frame
        CALL Display Labels
        CALL Flash (20, 2, 'Press any key to begin the program ')

        RETURN
        END

C************************************************************************
C SUBROUTINE READ SYSTEM DEFAULTS 
C 
C This routine reads the system defaults file 'GEOEAS.DEF' and  
C initializes the system defaults.
C
        SUBROUTINE Read System Defaults
        
        IMPLICIT INTEGER*4 (A-Z)

$INCLUDE: 'POSTPLOT.CMN'

        CHARACTER   String*80

        OPEN (UNIT = 10, FILE = 'GEOEAS.DEF', STATUS = 'UNKNOWN',
     +        ERR= 999)

        READ (10, '(A)', ERR = 999) File Prefix
        READ (10, '(A)', ERR = 999) Data File
        READ (10, '(A)', ERR = 999) String

C *** Intro flag embedded as INTRO=ON, INTRO=anything else
C *** Intro is a global logical
        Intro = (String (7:8) .EQ. 'ON' .OR.
     +           String (7:8) .EQ. 'on')

        CALL Set Character Value ('PREFIX    ',File Prefix)
        CALL Set Character Value ('DATA FILE ',Data File)

        CLOSE (10)
        RETURN
         
 999    CLOSE (10)
        File Prefix = ' '
        Intro = .TRUE.

        RETURN
        END

C************************************************************************
C SUBROUTINE WRITE SYSTEM DEFAULTS 
C
C This routine writes the system defaults to the system default file 
C 'GEOEAS.DEF'.
C
        SUBROUTINE Write System Defaults
        
        IMPLICIT INTEGER*4 (A-Z)

$INCLUDE: 'POSTPLOT.CMN'


        OPEN (UNIT = 10, FILE = 'GEOEAS.DEF', STATUS = 'UNKNOWN',
     +        ERR= 999)

        WRITE (10, '(A)', ERR = 999) File Prefix
        WRITE (10, '(A)', ERR = 999) Data File

        IF (Intro) THEN
           WRITE (10, '(A)', ERR = 999) 'INTRO=ON'
        ELSE
           WRITE (10, '(A)', ERR = 999) 'INTRO=OFF'
        ENDIF


 999    CLOSE (10)

        RETURN
        END

C************************************************************************
C SUBROUTINE INITIALIZE 
C
C This routine initializes global varaibles to their default values and
C then calls Reset Values to set the screen fields.
C
        SUBROUTINE Initialize
        
        IMPLICIT INTEGER*4 (A-Z)
 
$INCLUDE:'postplot.cmn'

	Xaxis Args (1) = 'F (''X/MINIMUM.'',0.)'
	Xaxis Args (2) = 'F (''X/MAXIMUM.'',0.)'
	Xaxis Args (3) = 'F (''AXIS/BOTTOM/MAJOR/TYPE.'',1.E36)'
	Xaxis Args (4) = 'F (''AXIS/BOTTOM/MAJOR/BASE.'',1.E36)'
	Xaxis Args (5) = 'F (''AXIS/BOTTOM/NUMERIC/TYPE.'',1.E36)'
	Xaxis Args (6) = 'F (''AXIS/BOTTOM/NUMERIC/EXPONENT.'',1.E36)'
	Xaxis Args (7) = 'F (''AXIS/BOTTOM/NUMERIC/FRACTION.'',1.E36)'
       
	Yaxis Args (1) = 'F (''Y/MINIMUM.'',0.)'
	Yaxis Args (2) = 'F (''Y/MAXIMUM.'',0.)'
	Yaxis Args (3) = 'F (''AXIS/LEFT/MAJOR/TYPE.'',1.E36)'
 	Yaxis Args (4) = 'F (''AXIS/LEFT/MAJOR/BASE.'',1.E36)'
	Yaxis Args (5) = 'F (''AXIS/LEFT/NUMERIC/TYPE.'',1.E36)'
	Yaxis Args (6) = 'F (''AXIS/LEFT/NUMERIC/EXPONENT.'',1.E36)'
	Yaxis Args (7) = 'F (''AXIS/LEFT/NUMERIC/FRACTION.'',1.E36)'
        
        CALL Reset Values

        RETURN
        END

C**********************************************************************   
C SUBROUTINE RESET VALUES 
C
C This routine resets the parameters for the graph options menu to
C their default values.
C

        SUBROUTINE Reset Values
         
        IMPLICIT INTEGER*4 (A-Z)

        CALL Set Character Value ('TIC BAS X', '   Auto   ')
        CALL Set Character Value ('TIC BAS Y', '   Auto   ')

        CALL Set Toggle Position ('TIC TYP X  ', 4)
        CALL Set Toggle Position ('TIC TYP Y  ', 4)
        
        CALL Set Toggle Position ('TIC LAB X  ', 0)
        CALL Set Toggle Position ('TIC LAB Y  ', 0)
        
        CALL Set Character Value ('TIC EXP X', '   Auto   ')
        CALL Set Character Value ('TIC EXP Y', '   Auto   ')
        
        CALL Set Character Value ('TIC FRC X', '   Auto   ')
        CALL Set Character Value ('TIC FRC Y', '   Auto   ')
	  
        CALL Set Toggle Position ('AXES     ', 1)
        
        CALL Set Character Value ('METACODE ', 'Metacode.met  ')

        RETURN
        END

   
C**********************************************************************   
C SUBROUTINE GET NEW SCREEN (SCREEN NUMBER) 
C
C Formal Arguments:
C  SCREEN NUMBER  - Stores the screen number.  
C
C This routine sets the current screen to screen number and then displays
C the frame and labels for that screen. 
C

        SUBROUTINE Get New Screen (Screen Number)
         
        IMPLICIT INTEGER*4 (A-Z)
       
        INTEGER*4   Screen Number        

        CALL Clear Screen
        CALL Set Screen (Screen Number)
        CALL Display Frame
        CALL Display Labels

        RETURN
        END

C*********************************************************************
C INTEGER*4 FUNCTION TOGGLE POSITION (FIELD NAME) 
C 
C Formal Arguments:
C  FIELD NAME  - Stores the toggle field name. 
C
C This routine returns the toggle position for the specified toggle
C field.
C

        INTEGER*4 FUNCTION Toggle Position (Field Name)
       
        IMPLICIT INTEGER*4   (A-Z)

$INCLUDE:'screen.str'
      
        CHARACTER*(*)   Field Name
        
        f = Fld Index (Field Name)
        Toggle Position = SC Position (f)

        RETURN
        END
 
C*********************************************************************
C SUBROUTINE ERASE MESSAGE 
C
C This routine erases messages displayed in the message area of the 
C screen.
C

        SUBROUTINE Erase Message

        IMPLICIT INTEGER*4 (A-Z)


	CALL Erase (9,42,35)
	CALL Erase (10,42,35)
	CALL Erase (11,42,35)
	CALL Erase (12,42,35)
	CALL Erase (13,42,35)
	CALL Erase (14,42,35)
	CALL Erase (15,42,35)
	CALL Erase (16,42,35)
	CALL Erase (17,42,35)
	CALL Erase (18,42,35)
        
        RETURN 
        END

C*********************************************************************
C SUBROUTINE DISPLAY FLDS 
C
C This routine displays the current screen fields.
C

        SUBROUTINE Display Flds

        IMPLICIT INTEGER*4 (A-Z)

$INCLUDE:'screen.str'

        INTEGER*4   Main Menu Field, Menu Field
  
        PARAMETER (Main Menu Field = 3)
        PARAMETER (Menu Field      = 4)


        DO 10 i = Begin Screen (Current Screen),
     +            End Screen (Current Screen)
           IF (SC Field Type (i) .NE. Main Menu Field .AND.
     +         SC Field Type (i) .NE. Menu Field)
     +        CALL Write Field (i, SC Attribute (i))
10      CONTINUE

        RETURN
        END           

C***************************************************************************
C SUBROUTINE GET FIELD GROUP 
C
C This routine gets the value for a group of screen fields.
C
  	SUBROUTINE Get Field Group 
	
        IMPLICIT INTEGER*4 (A-Z)

	PARAMETER (Main Menu Field = 3)
	PARAMETER (Menu Field      = 4)

$INCLUDE:'screen.str'
	
 10	CALL Get Field	
	IF (SC Field Type (Next Field) .NE. Main Menu Field .AND.
     +      SC Field Type (Next Field) .NE. Menu Field) GOTO 10

	RETURN
	END

C***********************************************************************
C SUBROUTINE GET SCREEN ARGS 
C
C This routine extracts the screen values and stores them into their 
C global variables. It calls Get Axis Args to get the X and Y axis
C parameters. 
C                

        SUBROUTINE Get Screen Args

        IMPLICIT INTEGER*4 (A-Z)

$INCLUDE:'postplot.cmn'

        CHARACTER  XY


C *** Get Option values
        f = Toggle Position ('VALUES    ')
        CALL Get Toggle Value ('VALUES     ', f, Val sel)

        Vsize = Toggle Position ('VAL SIZE    ')

        f = Toggle Position ('SCALE     ')
        CALL Get Toggle Value ('SCALE      ', f, Scal sel)

        f = Toggle Position ('DECIMALS  ')
        CALL Get Toggle Value ('DECIMALS   ', f, Dec sel)

        f = Toggle Position ('SYMBOLS   ')
        IF ( f .EQ. 0 ) THEN
           Sym sel = 'Yes'
        ELSEIF ( f .EQ. 1) THEN
           Sym sel = 'One'
 	ELSE
           Sym sel = 'No'
	ENDIF

        IF ( Val sel .EQ. 'No' .AND. Sym sel .EQ. 'No') THEN
           CALL Set Toggle Position ('SYMBOLS   ', 0) 
           CALL Display One Field ('SYMBOLS   ')
           Sym sel = 'Yes'
        ENDIF
           
        Ssize = Toggle Position ('SYM SIZE  ')

        IF (Scal sel .NE. ' 0') THEN
           Scaled  = 'Values scaled by 10#006D%'//Scal Sel
        ENDIF      

C *** Get backround
	
	f = Toggle Position ('AXES      ')
	CALL Get Toggle Value ('AXES      ', f , Ax sel)

	IF (Ax sel .EQ. 'Full') THEN
	   Background = 1
	ELSE IF (Ax sel .EQ. 'Grid') THEN
	   Background = 2
	ELSE
	   Background = 3
	ENDIF

C *** Get Axes arguments

	XY = 'X'
	CALL Get Axis Args (XY)

	XY = 'Y'
	CALL Get Axis Args (XY)

C *** Title and label arguments

	CALL Get Character Value ('TITLE     ', Titl)
	CALL Get Character Value ('SUBTITLE1 ', STit1)
	CALL Get Character Value ('SUBTITLE2 ', STit2)
	CALL Get Character Value ('X LABEL   ', Xlab)
	CALL Get Character Value ('Y LABEL   ', Ylab)

	RETURN
	END

C***********************************************************************
C SUBROUTINE GET AXIS ARGS (XY)
C
C Formal Arguments:
C  XY  - Stores which axis value to extract.
C
C This routine extracts the screen values for the X and Y axes and 
C stores them in their global variables. 
C               

        SUBROUTINE Get Axis Args (XY)

        IMPLICIT INTEGER*4 (A-Z)

$INCLUDE:'postplot.cmn'
	
        CHARACTER    XY, Ctlab*6
	CHARACTER*10 Fldnam, Ctbas, Cttyp, Ctexp, Ctfrac
	CHARACTER*13  Cmin, Cmax

	REAL*4       Nmin, Nmax

C *** Min/Max 
	  
        WRITE (Fldnam, '(A4,A1)') 'MIN ', XY
        CALL Get 4 Floating Point Value (Fldnam, Nmin)
	WRITE ( Cmin, '(F13.3)') Nmin

	WRITE (Fldnam, '(A4,A1)') 'MAX ', XY
        CALL Get 4 Floating Point Value (Fldnam, Nmax)
	WRITE ( Cmax, '(F13.3)') Nmax

	IF (XY .EQ. 'X' ) THEN
    	   WRITE (Xaxis Args (1)(17:50), '(A13,A1)') Cmin, ')'
	   WRITE (Xaxis Args (2)(17:50), '(A13,A1)') Cmax, ')'
           Xmin= Nmin
	   Xmax= Nmax
	ELSE
    	   WRITE (Yaxis Args (1)(17:50), '(A13,A1)') Cmin, ')'
	   WRITE (Yaxis Args (2)(17:50), '(A13,A1)') Cmax, ')'
	   Ymin= Nmin
	   Ymax= Nmax
	ENDIF

C *** Tic Type

        Write (Fldnam,'(A8,A1)') 'TIC TYP ', XY
	Position = Toggle Position (Fldnam)
	CALL Get Toggle Value (Fldnam, Position, Cttyp)
	Cfttyp = Toggle Position (Fldnam)

        IF (Cttyp .EQ. '   Auto  ') THEN
	   Cttyp = '1.E36'
	   IF (XY .eq. 'X') THEN
	      Write (Xaxis Args(3)(30:50),'(A10,A1)') Cttyp, ')'
	      CALL Remove Blanks (Xaxis Args(3))
	   ELSE
	      Write (Yaxis Args(3)(28:50),'(A10,A1)') Cttyp, ')'
	      CALL Remove Blanks (Yaxis Args(3))
	   ENDIF
	ELSE
	   IF (XY .eq. 'X') THEN
	      Write (Xaxis Args(3)(30:50),'(I1,A2)') Cfttyp, '.)'
	      CALL Remove Blanks (Xaxis Args(3))
	   ELSE
	      Write (Yaxis Args(3)(28:50),'(I1,A2)') Cfttyp, '.)'
	      CALL Remove Blanks (Yaxis Args(3))
	   ENDIF
	ENDIF

C *** Tic Base

 	WRITE (Fldnam, '(A8,A1)') 'TIC BAS ', XY
	CALL Get Character Value (Fldnam, Ctbas)
				 
	IF (Ctbas .EQ. '   Auto  ') Ctbas = '1.E36'
   
        IF (XY .EQ. 'X' ) THEN
           WRITE (Xaxis Args (4)(30:50), '(A10,A1)') Ctbas, ')'
           CALL Remove Blanks (Xaxis Args (4))
	ELSE
    	   WRITE (Yaxis Args (4)(28:50), '(A10,A1)') Ctbas, ')'
           CALL Remove Blanks (Yaxis Args (4))
	ENDIF

C *** Tic Labels

	WRITE (Fldnam, '(A8,A1)') 'TIC LAB ', XY
        f = Toggle Position (Fldnam)
	CALL Get Toggle Value (Fldnam, f, Ctlab)

	IF (Ctlab .EQ. ' Auto ') Ctlab = '1.E36'
	IF (Ctlab .EQ. ' None ') Ctlab = '0.'
	IF (Ctlab .EQ. 'Scien.') Ctlab = '1.'
	IF (Ctlab .EQ. 'Expon.') Ctlab = '2.'
	IF (Ctlab .EQ. 'No-Exp') Ctlab = '3.'

	IF (XY .EQ. 'X') THEN
	   WRITE (Xaxis Args (5)(32:50), '(A6,A1)') Ctlab,')'
	   CALL Remove Blanks (Xaxis Args(5))
	ELSE
	   WRITE (Yaxis Args (5)(30:50), '(A6,A1)') Ctlab, ')'
	   CALL Remove Blanks (Yaxis Args(5))
	ENDIF
        
C ** Tic exponent

	Write (Fldnam,'(A8,A1)') 'TIC EXP ', XY
	CALL Get Character Value (Fldnam,Ctexp)
        IF (Ctexp .eq. '   Auto   ')  Ctexp = '1.E36'

        IF (XY .eq. 'X') THEN
	   Write (Xaxis Args(6)(36:47),'(A10,A1)') Ctexp, ')'
	   CALL Remove Blanks (Xaxis Args(6))
	ELSE
	   Write (Yaxis Args(6)(34:47),'(A10,A1)') Ctexp, ')'
	   CALL Remove Blanks (Yaxis Args(6))
	ENDIF

C ** Tic fraction

	Write (Fldnam,'(A8,A1)') 'TIC FRC ', XY
	CALL Get Character Value (Fldnam,Ctfrac)

	IF (Ctfrac .EQ. '   Auto   ')  Ctfrac = '1.E36'

        IF (XY .EQ. 'X') THEN
	   Write (Xaxis Args(7)(36:47),'(A10,A1)') Ctfrac, ')'
	   CALL Remove Blanks (Xaxis Args(7))
	ELSE
	   Write (Yaxis Args(7)(34:47),'(A10,A1)') Ctfrac, ')'
	   CALL Remove Blanks (Yaxis Args(7))
	ENDIF


        RETURN
	END

C***********************************************************************
C SUBROUTINE SET GRAPH LIMITS
C
C This routine determines the quartile cut off values, then determines
C which data points are to be plotted by checking the graph limit.
C

	SUBROUTINE Set Graph Limits

	IMPLICIT INTEGER*4 (A-Z)

$INCLUDE:'postplot.cmn'
                                           

C *** Determine cut offs
        IF (Ndata .EQ. 0) THEN
           DO 10 i = 1, Ndatafile
              Iptr(i) = i
 10        CONTINUE
	   CALL Qsort (Parr, Iptr, Ndatafile)
           Cutoff(1) = Parr(Ndatafile * .25)
           Cutoff(2) = Parr(Ndatafile * .50)
           Cutoff(3) = Parr(Ndatafile * .75)
           
        ENDIF

C *** Initialize variables
	DO 5 i = 1,Ndata
	   Xcor(i) = 0.
	   Ycor(i) = 0.
	   Pval(i) = 0.
  5     CONTINUE
        Ndata = 0

C *** Determine value to use in plotting
	DO 15 j = 1, Ndatafile
           IF (((Xarr(Iptr(j)) .GE. Xmin) .AND. 
     +          (Xarr(Iptr(j)) .LE. Xmax)) .AND.
     +         ((Yarr(Iptr(j)) .GE. Ymin) .AND.
     +          (Yarr(Iptr(j)) .LE. Ymax))) THEN
	      Ndata = Ndata + 1
              Xcor (Ndata) = Xarr (Iptr(j))
	      Ycor (Ndata) = Yarr (Iptr(j))
	      Pval (Ndata) = Parr (j)
	   ENDIF
  15    CONTINUE

	RETURN

999     CLOSE(10)
        RETURN
	END


C***********************************************************************
C SUBROUTINE SET VALUE 
C
C  This routine calculates the scale factor to use, then computes the
C  format to plot the values. This routine is called only if the user
C  selects to plot the value. 
C

	SUBROUTINE Set Value 

	IMPLICIT INTEGER*4 (A-Z)

$INCLUDE:'postplot.cmn'

	CHARACTER Fmt*8 
	INTEGER*4  Scal
	REAL*4  X,Factor

C *** Compute the scale factor   
	WRITE (Fmt, '(A2,I2,A1,A2,A1)') '(F', 15,'.',Dec sel, ')'
        READ (Scal sel, '(I2)') Scal
        Factor = 10**(ABS(Scal))

C *** Compute format for the labels
	DO 10 i = 1, Ndata
           IF (Scal .LT. 0) THEN
              X = Pval(i)/Factor
           ELSE
	      X = Pval(i)*Factor
	   ENDIF
           WRITE (Value(i), Fmt) X
           CALL Left Justify (Value(i))
 10	CONTINUE
	RETURN
	END
  
C***********************************************************************
C SUBROUTINE OPEN FILE (NUNIT, NAMFIL, STAT, OPENERR)
C
C Formal Arguments:
C  NUNIT     -  File unit number.
C  NAMFIL    -  File name.
C  STAT      -  File status.
C  OPENERR   -  Logical variable set to ture if error occurs.
C
C This routine opens Nunit with Namfil and Stat parameters and returns 
C Openerr = 0, if successful.
C

        SUBROUTINE Open File (Nunit, Namfil, Stat, Openerr)

        IMPLICIT INTEGER*4 (A-Z)

	CHARACTER Namfil*(*), Stat*3

	Openerr = 0
	IF (Stat .EQ. 'OLD') THEN
	   OPEN (UNIT = Nunit,
     +           FILE = Namfil,
     +           STATUS = Stat,
     +           ERR = 99)
	ELSE
           OPEN (UNIT = Nunit,
     +           FILE = Namfil,
     +           STATUS = Stat,
     +           ERR = 99)

	ENDIF

	RETURN

 99	Openerr = 1

	RETURN
	END

C***********************************************************************
C SUBROUTINE REMOVE BLANKS (ARG)
C
C Formal Arguments:
C  ARG  - Character string to remove blanks.
C
C This routine checks for blanks in a given argument and shifts to
C the left as needed. 

	SUBROUTINE Remove Blanks (Arg)

	IMPLICIT INTEGER*4  (A-Z)

	CHARACTER*(*)   Arg

	ls = Lenstr(Arg)

	DO 10 k = 3, ls
	   m = Lenstr (Arg)
	   IF (Arg (k:k) .EQ. ' ') THEN
	      DO 15 l = k,m
	         IF (Arg (l:l) .NE. ' ') THEN
	            Arg(k:ls) = Arg(l:ls)
	         ENDIF
  15          CONTINUE
	   ENDIF
 10 	CONTINUE

	RETURN
	END

C***********************************************************************
C FUNCTION LENCMA (STRING)
C 
C Formal Arguments:
C  STRING  - Character sting. 
C
C This routine returns the location of the right-most comma in a string.
C Used for parsing Autograph arguments.
C 
        FUNCTION Lencma (String)

	IMPLICIT INTEGER*4  (A-Z)
 
	CHARACTER String*(*)

	n = LEN(String)

	DO 10 i = n,1,-1
	   IF (String(i:i) .EQ. ',') THEN
	      Lencma = i
	      RETURN
	   ENDIF
 10	CONTINUE

	Lencma = 0

	RETURN
	END



         
