$LARGE
c**********************************************************************
c*** File name: TRANSC.FOR (Source code for the program TRANS)
C**********************************************************************
c        F i l e   S e t   U p		       (subroutine)
c**********************************************************************
      SUBROUTINE File Set Up (Error, Nvar temp)

c*** This routine prompts the user for a filename. 
c*** If the file exists then a call to 'Get Header Data' 
c*** reads in the the variable number, title, and variable names.  
c*** The Variable box is cleared and the new variables
c*** are displayed.  The appropriate toggle fields are 're'loaded.
c*** If the file does not exist then a warning is issued and control
c*** returned to the calling subroutine.

c*** FORMAL ARGUMENTS:
c*** Error     - .TRUE. if any fatal errors occurred in accessing the
c***             information from the file.
c*** Nvar temp - number of variables in Geo-EAS input data file. 
c***
c*** VARIABLES:
c*** Title Field - stores the field name of the 'Title'.
c*** Noinfile	 - global logical variable.
c*** Infil	 - global variable that stores the name of the Geo-EAS
c***               input data file.
c*** Prefile     - global variable.
c*** NORMAL	 - parameter defined as zero in Screen.str.

$INCLUDE:'Trans.dec'
$INCLUDE:'Screen.str'

      CHARACTER*10  Title Field, Fname
      INTEGER*4     Nvar temp
      LOGICAL*1     Exist, Error

      Title Field = 'TITLE     '
      Fname       = 'INFILE    '
      Error = .FALSE.
      Noinfile = .TRUE.
      CALL Build Prefix Filename (Infil)
      INQUIRE (FILE = Prefile, EXIST = Exist)
      IF (.NOT. Exist) THEN
	 Error = .TRUE.
	 CALL Flash (20, 2,
     +   'ERROR - No such File Exists ... (Press any key)')
      ELSE
	CALL Get Header Data(Infil, Error, Nvar temp)
	IF (.NOT. Error) THEN
	  CALL Clear Variable Box
c***      Assign title to 'Title' variable.
	  CALL Set Character Value (Title Field, Title)
	  CALL Write Field (FLD Index (Title Field), NORMAL)
          CALL Load New Toggle Values 
c***      Set logical 'Noinfile' to indicate that a legitimate 
c***      file exists.
	  Noinfile = .FALSE.
	ENDIF
      ENDIF
   
      RETURN
      END
c**********************************************************************
c	 D i s p l a y  V a r i a b l e s   (subroutine)
c**********************************************************************
      SUBROUTINE Display Variables

c*** This subroutine displays variable names up to six in a row. ***
c***
c*** VARIABLES:
c*** Row    - stores the row number at which to display the variable 
c***          name.
c*** Col    - stores the column number at which to display the 
c***          variable name.
c*** Varnam - global array that stores the variable names.

$INCLUDE:'Trans.dec'
$INCLUDE:'Screen.str'

      Integer*4 I, J, Row, Col

c***  Set parameters to display first variable in desired row and 
c***  column. 
      Row = 11
      Col = 2
      CALL Clear Variable Box
      DO 10 I = 1, Nvar
	IF ( MOD(I,6) .EQ. 0) THEN
c***      Display the sixth variable and increment row counter to 
c***      next row.
     	  CALL Put Str(Varnam(I), 10, Row, Col, 0)
	  Row = Row + 1
	  Col = 2
        ELSE
c***      Display five variables in a row.  
	  CALL Put Str(Varnam(I), 10, Row, Col, 0)
c***      Increment column counter to next column.
	  Col = Col + 12
        ENDIF
  10  CONTINUE

      RETURN
      END
c**********************************************************************
c	 C l e a r  V a r i a b l e  B o x    (subroutine)
c**********************************************************************
      SUBROUTINE Clear Variable Box

c*** This subroutine clears the box that displays the variables.
 
$INCLUDE:'Screen.str'

      DO 10 I = 11,18
        CALL Erase (I, 2, 75)
  10  CONTINUE

      RETURN
      END
C**********************************************************************
c        G e t  F o r m a t                 (subroutine)
c**********************************************************************
      SUBROUTINE Get Format(Value)	

c*** Prompt user for the output format width and decimal size.
c*** If input format is illegal then issue message and use
c*** G16.9 as the default.

c*** FORMAL ARGUMENTS:
c*** Value - This value is displayed in the screen field and used by 
c***         default as the width for the FORTRAN output format.
c*** GLOBAL VARIABLES:
c*** Fwidth    - width for FORTRAN output format.
c*** Fdecimals - number of decimal places for FORTRAN output format.
 
$INCLUDE:'Trans.dec'
$INCLUDE:'Screen.str'

      CHARACTER*10 Choice,Heading1*5,Heading2*8
      CHARACTER*8  Value
      INTEGER*4    W1, D1

c***  prompt user for format width.
      Choice = 'FWIDTH    '
      CALL Set Character Value (Choice, Value)
      Next Field = FLD Index (Choice)
      CALL Get Field
c***  Erase width field from screen.
      CALL Erase (20,56,2)
      CALL Erase Field (Choice)
      CALL Get Character Value (Choice, Value)
      CALL Left Justify (Value)
c***  determine if format entered by user is within legal range.
c***  if not then issue warning and use G16.9 by default,
c***  else use the format specified by the user.
c***  The expression used for the FORTRAN output format is
c***  Gx + 7.x where x is Fwidth and 1<=Fwidth<=12.
c	IF ((Fwidth .GE. 1) .AND. (Fwidth .LE. 12)) THEN
c	  Value = Fwidth
c	  Fdecimals = Fwidth
c	  Fwidth = Fwidth + 7
c 	ELSE
c	  CALL Flash(20, 2, 
c     +    'Illegal format, 9 used by default (G16.9)...<Press any key>')
c	  Fwidth = 16
c	  Fdecimals = 9
c	  Value = 9
c        ENDIF
      RETURN
      END
C**********************************************************************
c        C o n v e r t  I n t  t o  C h a r  F o r m a t  (subroutine)
c**********************************************************************
      SUBROUTINE Convert Int to Char Format(Col)

c*** Convert the format output width and decimal variables(integer) to
c*** character type and insert into the array 'Formats'.  The 
c*** formats always use the G format.

c*** FORMAL ARGUMENT:
c*** Col - variable column used to identify the variable in question.
c***  
c*** VARIABLES:
c*** Formats   - global array that stores the output format for each
c***             variable.
c*** Fwidth    - global variable that stores the width field of an
c***             output format.
c*** Fdecimals - global variable that stores the decimal field of
c***             an output format.

$INCLUDE:'Trans.dec'

      INTEGER*4 Col, I

      Formats(Col) = '      '
      Formats(Col)(1:1) = 'G'
      IF (Fwidth .LT. 10) THEN
c***    Convert a one digit width field.
c***    If width field is < than 10, process.
	Formats(Col)(2:2) = CHAR(ICHAR('0') + Fwidth)
	Formats(Col)(3:3) = '.'
	IF (Fdecimals .LT. 10) THEN
c***      Convert a one digit decimal field.
	  Formats(Col)(4:4) = CHAR(ICHAR('0') + Fdecimals)
	ELSE
c***      Convert a two digit decimal field.
	  I = INT(Fdecimals/10)
	  Formats(Col)(4:4) = CHAR(ICHAR('0') + I)
	  I = MOD (Fdecimals, 10)
	  Formats(Col)(5:5) = CHAR (ICHAR('0') + I)
	ENDIF
      ELSE
c***    Convert a two digit width field.
c***    If width field is > than 9, process.
	I = INT(Fwidth/10)
	Formats(Col)(2:2) = CHAR(ICHAR('0') + I)
	I = MOD (Fwidth, 10)
	Formats(Col)(3:3) = CHAR (ICHAR('0') + I)
	Formats(Col)(4:4) = '.'
	IF (Fdecimals .LT. 10) THEN
c***      Convert a one digit decimal field.
	  Formats(Col)(5:5) = CHAR(ICHAR('0') + Fdecimals)
	ELSE
c***      Convert a two digit decimal field.
	  I = INT(Fdecimals/10)
	  Formats(Col)(5:5) = CHAR(ICHAR('0') + I)
	  I = MOD (Fdecimals, 10)
	  Formats(Col)(6:6) = CHAR (ICHAR('0') + I)
	ENDIF
      ENDIF
      RETURN
      END
C**********************************************************************
C        G e t   F i l e n a m e      (subroutine)
C**********************************************************************
      SUBROUTINE Get Name of File(Fieldname, Filename, Error, Check)

c***  This routine prompts the user for the name of the file and
c***  checks if the filename is a blank, if so error=true is returned.
c***  If the variable 'Check' = 1 then the following is performed.
c***  If the filename is not blank it is checked for existence, if non-
c***  existent then error=true is returned.   If the file exists then  
c***  error=false is returned.  If the 'Check' = 2 then if the output 
c***  file exists, then the user is asked if he or she wishes to 
c***  overwrite the existing file.  

c***  FORMAL ARGUMENTS:
c***  Fieldname - Screen fieldname.
c***  Filename  - Name of Geo-EAS input data file.
c***  Error     - .TRUE. if file does not exist.
c***  Check     - If 1 check for input file, if 2 check for output file.  
c***
c***  VARIABLE:
c***  Prefile	- global variable. 

$INCLUDE:'Trans.dec'
$INCLUDE:'Screen.str'
		   
      CHARACTER   Blank*14, Fieldname*8, Filename*14,Readmess*16
      INTEGER*4	  Check
      LOGICAL*1   Exist, Error, Yes, Get Yes No

      Readmess	  = ' Reading data...'
      Error = .FALSE.
      Blank = '              '
c***  prompt user for filename ***
      Next Field = FLD Index (Fieldname)
      CALL Get Field
      CALL Get Character Value (Fieldname, Filename)
      CALL Put Str (Readmess, 16, 20, 2, 2)
c***  if  filename is blank issue message and return error=true.
      IF (Filename .EQ. Blank) THEN
	Error = .TRUE.
	CALL Flash(20, 2, 
     +     'ERROR - filename is blank...(press any key)')
      ELSE
c***  if file does not exist then issue message and return error=true.
c***  perform this check only if 'Check' = 1  where 1 means in input file.
	IF (Check .EQ. 1) THEN
          CALL Build Prefix Filename (Filename)
          INQUIRE (FILE = Prefile, EXIST = Exist)
	  IF (.NOT. Exist) THEN
	    Error = .TRUE.
	    CALL Flash(20, 2, 
     +       'ERROR - file does not exist...(press any key)')
c***        reset Title field to blanks and display this blank field.
	    DO 10 I = 1,66
	      Title(I:I) = ' '
  10	    CONTINUE
            Fieldname= 'TITLE   '
	    CALL Set Character Value(Fieldname, Title) 
	    CALL Display One Field (Fieldname)
 	    CALL Clear Variable Box
	  ENDIF
	ELSE IF (Check .EQ. 2) THEN
          CALL Build Prefix Filename (Filename)
          INQUIRE (FILE = Prefile, EXIST = Exist)
	  IF (Exist) THEN
     	    Yes = Get Yes No (20, 2,
     +	     ' Output file exists, do you wish to overwrite?...<Y/N>')
	    IF (.NOT. Yes) THEN
              Error = .TRUE.
              CALL Set Character Value (Fieldname, Blank)
	      CALL Display One Field(Fieldname)
	    ENDIF
	  ENDIF
 	ENDIF
      ENDIF

      RETURN
      END
c**********************************************************************

