$LARGE
c**********************************************************************
c*** File name: TRANSD.FOR (Source code for the program TRANS)
c***********************************************************************
c        P r o c e s s                 (subroutine)
c**********************************************************************
      SUBROUTINE Process

c*** This routine processes the unary and binary operations.

c*** GLOBAL VARIABLES:
c*** Calcomp - .TRUE. when the calculation is complete or when a fatal
c***           error occurs during computation.
c*** LOCAL VARIABLES:
c*** Rconstant	- temporary real variable.
c*** Constant1 	- temporary real variable.
c*** Constant2	- temporary real variable.
c*** Nerrors    - counter used to tally number of missing values in a
c***              variable.
c*** All other variables are fairly obvious in their usage.


c*** FUNCTIONS:
c*** Vmiss - logical function that is .true. if the argument is equivalent
c***         to a missing value.

$INCLUDE:'Trans.dec'
$INCLUDE:'Screen.str'

      REAL*4        Rconstant, Constant1, Constant2, Get Element
      LOGICAL*1     Yes, Error, Exit, Get Yes No, Vmiss, Passover
      CHARACTER*10  O1choice, O2choice, Fldchoice, Choice
      CHARACTER     Bnchoice*8, Ch*35, Readmess*18
      INTEGER*4     Opchoice, I, K, J, V1, V2, Nerrors

c***  Set Missing variable to desired value.
      Missing = 1.E+31
      Error = .FALSE.
      Nerrors = 0
      Readmess = 'Processing Data...'
      CALL Put Str (Readmess, 18, 20, 2, 2)
c***  Check if a variable to store results has been chosen ***
      IF (Newvar .OR. Oldvar) THEN
c***  Perform a unary operation ***
	IF (Uoperator) THEN
	  Choice = 'UNARY     '
	  CALL Get Toggle Value (Choice, Opchoice, Bnchoice)
c***      Check if operand is a constant ***
	  IF (Uocflag) THEN
            Fldchoice = 'UOPERCONST'
	    CALL Get 4 Floating Point Value(Fldchoice, Constant1)
c***        Determine the Sqrt of the constant operand ***
	    IF (Vmiss(Constant1)) THEN
	       Constant1 = Missing
	       CALL Flash(20,2,
     +         'Constant is a missing value...(press any key)')
	       CALL Clear Operation Box
	       Calcomp = .TRUE.
	       RETURN
	    ELSE
 	      IF (Opchoice .EQ. 0) THEN
c***          If operand of Sqrt is negative then issue error ***
	        IF (Constant1 .LT. 0) Then
		  CALL Flash (20,2,
     +		  'Sqrt of a negative is undefined...(press any key)')     
		  CALL Clear Operation Box
		  Calcomp = .TRUE.
		  RETURN
	        ELSE
c***              If operand of Sqrt is positive then perform operation ***
		  Constant1 = SQRT (Constant1)
	        ENDIF
c***          Perform Log operation ***
	      ELSE IF (Opchoice .EQ. 1) Then
c***          If operand of Common Log function is <= 0 then issue warning ***
	        IF (Constant1 .LE. 0) THEN
 		  CALL Flash(20, 2,
     +          'Log of a negative or 0 is undefined...(press any key)')
		  CALL Clear Operation Box
		  Calcomp = .TRUE.
		  RETURN
	        ELSE
		  Constant1 = ALOG10 (Constant1)
	        ENDIF
c***          Perform Ln operation ***
	      ELSE IF (Opchoice .EQ. 2) THEN
c***          If operand of Ln function is <= to 0 then issue warning ***
	        IF (Constant1 .LE. 0) THEN
		  CALL Flash(20,2,
     +	       'Ln of a negative or 0 is undefined...(press any key)')
		  CALL Clear Operation Box
		  Calcomp = .TRUE.
		  RETURN
	        ELSE 
		  Constant1 = ALOG (Constant1)
	        ENDIF
c***          Truncate the constant operand ***
	      ELSE IF (Opchoice .EQ. 3) THEN
                Constant1 = AINT (Constant1)
c***          Calculate the Exp of the constant ***
	      ELSE IF (Opchoice .EQ. 4) THEN
		 Constant1 = EXP(Constant1)
c***          Issue warning message for performing rank on a constant.
	      ELSE IF (Opchoice .EQ. 5) THEN
	         CALL Flash(20,2,
     +	        'Rank of a constant is one...(press any key)')
	         CALL Clear Operation Box
	         Calcomp = .TRUE.
                 RETURN
c***          Assign value of the constant
	      ELSE IF (Opchoice .EQ. 6) THEN
                Constant1 = Constant1
c***          Take absolute value of the constant operand ***
	      ELSE IF (Opchoice .EQ. 7) THEN
                Constant1 = ABS (Constant1)
c***          Assign -1 to < 0, 0 to = 0, or 1 to > 0 ***
	      ELSE IF (Opchoice .EQ. 8) THEN
                IF (Constant1 .LT. 0) THEN
                   Constant1 = -1.0
                ELSEIF (Constant1 .EQ. 0) THEN
                   Constant1 =  0.0
                ELSEIF (Constant1 .GT. 0) THEN
                   Constant1 =  1.0
                ENDIF
              ENDIF
	    ENDIF
c***        Assign results of chosen operation to temporary array ***
	    DO 40 I = 1, Ndata
	      Tmpdata(I) = Constant1
  40        CONTINUE
c***      Check if Unary operand is a variable ***
	  ELSE IF (Uovflag) Then
c***      Get the user's choice of the result variable ***
            Fldchoice = 'U OPERAND '
	    CALL Get Toggle Value (Fldchoice, K, O1choice)
c***        J identifies the chosen variable ie., the column number.
	    J = K + 1
c***        Calculate Sqrt of the operand ***
            IF (Opchoice .EQ. 0) THEN
c***          Check for a negative or missing sample, is so then 
c***          assign a missing value to that sample, else perform the 
c***          computation.
	      DO 50 I=1, Ndata
 		IF ((Get Element(1,J,I,MAXVAR) .LT. 0) .OR. 
     +               Vmiss(Get Element(1,J,I,MAXVAR))) THEN
		  Tmpdata(I) = Missing
		ELSE
        	  Tmpdata(I) = SQRT(Get Element(1,J,I,MAXVAR))
		ENDIF
  50          CONTINUE
c***        Calculate the Common Log of the variable ***
	    ELSEIF (Opchoice .EQ. 1) THEN
C***        If sample is a negative/zero/missing value, then assign
c***	    a missing value to that sample, else perform the computation.
	      DO 52 I=1,Ndata
 		IF ((Get Element(1,J,I,MAXVAR) .LE. 0) .OR. 
     +               Vmiss(Get Element(1,J,I,MAXVAR))) THEN
		  Tmpdata(I) = Missing
		ELSE
		  Tmpdata(I) = ALOG10(Get Element(1,J,I,MAXVAR))
		ENDIF
  52          CONTINUE
C***        Calculate Ln of a variable ***
	    ELSE IF (Opchoice .EQ. 2) THEN
	      DO 54 I=1,Ndata
C***          If sample is a negative/zero/missing value, then assign
c***	      a missing value to that sample, else perform the computation.
 		IF ((Get Element(1,J,I,MAXVAR) .LE. 0) .OR. 
     +               Vmiss(Get Element(1,J,I,MAXVAR))) THEN
		  Tmpdata(I) = Missing
		ELSE
		  Tmpdata(I) = ALOG(Get Element(1,J,I,MAXVAR))
		ENDIF
  54          CONTINUE
c***        Truncate the variable items ***
	    ELSE IF (Opchoice .EQ. 3) THEN
              DO 56 I=1, Ndata
c***            If sample is not missing then perform computation,
c***            else assign a missing value.
		IF (Vmiss(Get Element(1,J,I,MAXVAR))) THEN
		  Tmpdata(I) = Missing
		ELSE
		  Tmpdata(I) = AINT(Get Element(1,J,I,MAXVAR))
		ENDIF
  56          CONTINUE
c***        Calculate the Exp function ***
	    ELSE IF (Opchoice .EQ. 4) THEN
	      DO 99 I = 1, Ndata
		IF (Vmiss(Get Element(1,J,I,MAXVAR))) THEN
		  Tmpdata(I) = Missing
		ELSE
		  Tmpdata(I) = EXP(Get Element(1,J,I,MAXVAR))
		ENDIF
  99	      CONTINUE
c***        Rank the data from lowest to highest ***
	    ELSE IF (Opchoice .EQ. 5) THEN
	      CALL Rank Var(J)
c***          Assign value of the variable
	    ELSE IF (Opchoice .EQ. 6) THEN
              DO 100 I=1, Ndata
c***            If sample is not missing then perform computation,
c***            else assign a missing value.
		IF (Vmiss(Get Element(1,J,I,MAXVAR))) THEN
		  Tmpdata(I) = Missing
		ELSE
		  Tmpdata(I) = Get Element(1,J,I,MAXVAR)
		ENDIF
 100          CONTINUE
c***          Take absoulte value of the variable ***
	    ELSE IF (Opchoice .EQ. 7) THEN
              DO 101 I=1, Ndata
c***            If sample is not missing then perform computation,
c***            else assign a missing value.
		IF (Vmiss(Get Element(1,J,I,MAXVAR))) THEN
		  Tmpdata(I) = Missing
		ELSE
		  Tmpdata(I) = ABS(Get Element(1,J,I,MAXVAR))
		ENDIF
 101          CONTINUE
c***          Assign -1 to < 0, 0 to = 0, or 1 to > 0 ***
	    ELSE IF (Opchoice .EQ. 8) THEN
              DO 102 I=1, Ndata
c***            If sample is not missing then perform computation,
c***            else assign a missing value.
		IF (Vmiss(Get Element(1,J,I,MAXVAR))) THEN
		  Tmpdata(I) = Missing
		ELSE
                  TmpData (I) =  Get Element(1,J,I,MAXVAR)
                  IF (TmpData (I) .LT. 0) THEN
                     Tmpdata (I) = -1.0
                  ELSEIF (Tmpdata (I) .EQ. 0) THEN
                     Tmpdata (I) =  0.0
                  ELSEIF (Tmpdata (I) .GT. 0) THEN
                     Tmpdata (I) =  1.0
                  ENDIF
	        ENDIF
 102          CONTINUE
	    ENDIF
C***      Issue error message if operand has not been specified ***
	  ELSE
	    CALL Flash (20, 2,
     +	   'Unary operand has not been specified...(press any key)')
	    CALL Clear Operation Box
	    Calcomp = .TRUE.
	    RETURN
	  ENDIF
C***    Perform Binary operations ***
	ELSE IF (Boperator) THEN
c***      Get the user selected binary operation.
	  Choice = 'BINARY    '
	  Call Get Toggle Value (Choice, Opchoice, Bnchoice)
c***      Binary operations using two constants ***
	  IF (B1cflag .AND. B2cflag) THEN
C***      Get floating point values for each constant ***
	    Fldchoice = 'BOP1CONST '
	    CALL Get 4 Floating Point Value(Fldchoice, Constant1)
	    Fldchoice = 'BOP2CONST '
	    CALL Get 4 Floating Point Value(Fldchoice, Constant2)
c***        Perform addition of two constants and assign to temp constant ***
	    IF (Vmiss(Constant1) .OR. Vmiss(Constant2)) THEN
	      Constant1 = Missing
	    ELSE
	      IF (Opchoice .EQ. 0) THEN
	        Constant1 = Constant1 + Constant2
c***          Perform division of two constants ***
	      ELSE IF (Opchoice .EQ. 1) THEN
c***          Check for negative denominator ***
	        IF (Constant2 .NE. 0) THEN
		  Constant1 = Constant1 / Constant2
	        ELSE
		  CALL Flash(20, 2,
     +		  'Division by zero is undefined...(press any key)')
		  CALL Clear Operation Box
		  Calcomp = .TRUE.
		  RETURN
	        ENDIF
c***          Perform multiplication ***
	      ELSE IF (Opchoice .EQ. 2) THEN
	        Constant1 = Constant1 * Constant2
c***          Perform subtraction ***
	      ELSE IF (Opchoice .EQ. 3) THEN
	        Constant1 = Constant1 - Constant2
c***          Perform exponentiation ***
	      ELSE IF (Opchoice .EQ. 4) THEN
	        Rconstant = AINT(Constant2)
c***            check for zero operand raised to a negative power ***
                IF ((Constant1 .EQ. 0) .AND.
     +                   (Constant2 .LT. 0)) THEN
		  CALL Flash(20, 2,
     +	  'Attempt to raise zero to a negative power...(press any key)')
		  CALL Clear Operation Box
		  Calcomp = .TRUE.
		  RETURN
c***            check for a neg. operand and a real power ***
	        ELSE IF ((Constant1 .LT. 0) .AND.
     +		        (Constant2 .GT. Rconstant)) THEN
		  CALL Flash (20, 2,
     +	  'Attempt to raise a negative to a real...(press any key)')
		  CALL Clear Operation Box
		  Calcomp = .TRUE.
		  RETURN
	        ELSE
                  Constant1 = Constant1 ** Constant2
	        ENDIF
c***          Determine maximum value ***
	      ELSE IF (Opchoice .EQ. 5) THEN
	        Constant1 = AMAX1 (Constant1, Constant2)
c***          Determine minimum value ***
	      ELSE IF (Opchoice .EQ. 6) THEN
	        Constant1 = AMIN1 (Constant1, Constant2)
c***          Calculate Mean ***
	      ELSE IF (Opchoice .EQ. 7) THEN
	        Constant1 = (Constant1 + Constant2) / 2.0
	      ENDIF
	    ENDIF
C***        Assign result to temp array ***
	    DO 57 I=1, Ndata
	      Tmpdata(I) = Constant1
  57	    CONTINUE
c***      Perform operations on a constant and a variable ***
          ELSE IF (B1cflag .AND. B2vflag) THEN
  	    Fldchoice = 'BOP1CONST '
	    CALL Get 4 Floating Point Value(Fldchoice, Constant1)
	    Fldchoice = 'B OPERAND2'
c***        Get position of variable chosen by user ***
	    CALL Get Toggle Value(Fldchoice, K, O2choice)
	    J = K + 1
c***        add the constant and the variable ***
	    IF (Vmiss(Constant1)) THEN
	      DO 58 I = 1, Ndata
		Tmpdata(I) = Missing
  58	      CONTINUE
	    ELSE
	      IF (Opchoice .EQ. 0) THEN
	        DO 59 I=1, Ndata
		  IF (Vmiss(Get Element(1,J,I,MAXVAR))) THEN
		    Tmpdata(I) = Missing
		  ELSE
		    Tmpdata(I) = Constant1 + Get Element(1,J,I,MAXVAR)
		  ENDIF
  59	        CONTINUE
C***          Perform division  ***
	      ELSE IF (Opchoice .EQ. 1) THEN
C***          Check for a zero divisor and issue error if so ***
	        DO 60 I=1, Ndata
		  IF ((Get Element(1,J,I,MAXVAR) .EQ. 0) .OR. 
     +                  Vmiss(Get Element(1,J,I,MAXVAR))) THEN
		    Tmpdata(I) = Missing
		  ELSE
		    Tmpdata(I) = Constant1 / Get Element(1,J,I,MAXVAR)
		  ENDIF
  60            CONTINUE
c***          Perform multiplication of the constant and variable ***
	      ELSE IF (Opchoice .EQ. 2) THEN
	        DO 62 I=1, Ndata
		  IF (Vmiss(Get Element(1,J,I,MAXVAR))) THEN
		    Tmpdata(I) = Missing
		  ELSE
		    Tmpdata(I) = Constant1 * Get Element(1,J,I,MAXVAR)
		  ENDIF
  62            CONTINUE
C***          Perform subtraction of the variable data from the constant ***
              ELSE IF (Opchoice .EQ. 3) THEN
                DO 63 I=1, Ndata
		  IF (Vmiss(Get Element(1,J,I,MAXVAR))) THEN
		    Tmpdata(I) = Missing
		  ELSE
		    Tmpdata(I) = Constant1 - Get Element(1,J,I,MAXVAR)
		  ENDIF
  63            CONTINUE
C***          Perform exponentiation ***
	      ELSE IF (Opchoice .EQ. 4) THEN
	        DO 64 I=1, Ndata
                  Rconstant = AINT(Get Element(1,J,I,MAXVAR))
		  IF (Vmiss(Get Element(1,J,I,MAXVAR))) THEN
c***                If exponent is a missing value then assign
c***                a missing value.
		    Tmpdata(I) = Missing
	          ELSE IF (.NOT.(Constant1 .NE. 0).AND.
     +                (Get Element(1,J,I,MAXVAR).LT.0)) THEN
c***                   If there is an attempt to raise a 0-value operand 
c***                   to a negative power then assign a missing value.
		       Tmpdata(I) = Missing
	          ELSE IF ((Constant1 .LT. 0).AND.
     +	               (Get Element(1,J,I,MAXVAR) .GT. Rconstant)) THEN
c***                   If there is an attempt to raise a negative-value 
c***                   operand to a nonintegral real power then assign a
c***                   missing value.
		       Tmpdata(I) = Missing
	          ELSE
c***                Perform the computation.
                    Tmpdata(I) = Constant1 ** Get Element(1,J,I,MAXVAR)
                  ENDIF
  64		CONTINUE
c***          Determine maximum value of the constant and variable  ***
	      ELSE IF (Opchoice .EQ. 5) THEN
	        DO 105 I=1, Ndata
		  IF (Vmiss(Get Element(1,J,I,MAXVAR))) THEN
		    Tmpdata(I) = Missing
		  ELSE
		  Tmpdata(I)=AMAX1(Constant1,Get Element(1,J,I,MAXVAR))
		  ENDIF
 105            CONTINUE
c***          Determine minimum value of the constant and variable  ***
	      ELSE IF (Opchoice .EQ. 6) THEN
	        DO 106 I=1, Ndata
		  IF (Vmiss(Get Element(1,J,I,MAXVAR))) THEN
		    Tmpdata(I) = Missing
		  ELSE
		  Tmpdata(I)=AMIN1(Constant1,Get Element(1,J,I,MAXVAR))
		  ENDIF
 106            CONTINUE
c***          Calculate Mean value of the constant and variable***
	      ELSE IF (Opchoice .EQ. 7) THEN
	        DO 107 I=1, Ndata
		  IF (Vmiss(Get Element(1,J,I,MAXVAR))) THEN
		    Tmpdata(I) = Missing
		  ELSE
		  Tmpdata(I)= (Constant1+Get Element(1,J,I,MAXVAR))/2.0
		  ENDIF
 107            CONTINUE
	      ENDIF
	    ENDIF
C***        Perform operations for a variable and a constant ***
            ELSE IF (B1vflag .AND. B2cflag) THEN
              Fldchoice = 'B OPERAND1'
	      CALL Get Toggle Value (Fldchoice, K, O1choice)
              J = K + 1
              Fldchoice = 'BOP2CONST '
              CALL Get 4 Floating Point Value(Fldchoice, Constant2)
c***          If the Operand 2 constant is a missing value then assign
c***          a missing value to the all the samples and do not perform
c***          any further computations. 
	      IF (Vmiss(Constant2)) THEN
		DO 65 I = 1, Ndata
		  Tmpdata(I) = Missing
  65		CONTINUE
c***          Perform addition on the variable and the constant ***
	      ELSE
	        IF (Opchoice .EQ. 0) THEN 
	          DO 66 I=1, Ndata
		    IF (Vmiss(Get Element(1,J,I,MAXVAR))) THEN
		      Tmpdata(I) = Missing
		    ELSE
		      Tmpdata(I) = Get Element(1,J,I,MAXVAR) + Constant2
		    ENDIF
  66              CONTINUE
c***            Perform division ***
	        ELSE IF (Opchoice .EQ. 1) THEN
c***              Denominator cannot be a zero.
	          IF (Constant2 .EQ. 0) THEN
		    CALL Flash(20, 2,
     +              'Division by zero is undefined...(press any key)')
		    CALL Clear Operation Box
		    Calcomp = .TRUE.
  		    RETURN
                  ELSE
c***                Perform the computation.
	            DO 67 I=1,Ndata
		      IF (Vmiss(Get Element(1,J,I,MAXVAR))) THEN
			Tmpdata(I) = Missing
		      ELSE
			Tmpdata(I) = Get Element(1,J,I,MAXVAR)/Constant2
		      ENDIF
  67                CONTINUE
                  ENDIF
C***            Perform multiplication ***
                ELSE IF(Opchoice .EQ. 2) THEN
                  DO 68 I=1, Ndata
		    IF (Vmiss(Get Element(1,J,I,MAXVAR))) THEN
		      Tmpdata(I) = Missing
		    ELSE
		      Tmpdata(I) = Get Element(1,J,I,MAXVAR) * Constant2
		    ENDIF
  68              CONTINUE
c***            Subtract the constant from the variable ***
	        ELSE IF (Opchoice .EQ. 3) THEN
                  DO 69 I=1, Ndata
		    IF (Vmiss(Get Element(1,J,I,MAXVAR))) THEN
		      Tmpdata(I) = Missing
		    ELSE
		      Tmpdata(I) = Get Element(1,J,I,MAXVAR) - Constant2
		    ENDIF
  69	          CONTINUE
C***            Perform exponentiation ***
	        ELSE IF (Opchoice .EQ. 4) THEN
	          Rconstant = AINT(Constant2)
                  DO 70 I=1, Ndata
		    IF (Vmiss(Get Element(1,J,I,MAXVAR))) THEN
		      Tmpdata(I) = Missing
                    ELSE IF ((Get Element(1,J,I,MAXVAR) .EQ. 0) .AND.
     +                      (Constant2 .LT. 0)) THEN
c***                  If there is an attempt to raise a 0-value operand 
c***                  to a negative power then assign a missing value.
		      Tmpdata(I) = Missing
                    ELSE IF((Get Element(1,J,I,MAXVAR) .LT. 0) .AND.
     +			  (Constant2 .GT. Rconstant)) THEN
c***                  If there is an attempt to raise a negative-value 
c***                  operand to a nonintegral real power then assign a
c***                  missing value.
		      Tmpdata(I) = Missing
                    ELSE
	              Tmpdata(I) = Get Element(1,J,I,MAXVAR)**Constant2
	            ENDIF
  70	          CONTINUE
c***            Determine maximum value of the variable and constant  ***
	        ELSE IF (Opchoice .EQ. 5) THEN
	          DO 108 I=1, Ndata
		   IF (Vmiss(Get Element(1,J,I,MAXVAR))) THEN
		     Tmpdata(I) = Missing
		   ELSE
		   Tmpdata(I)=AMAX1(Constant2,Get Element(1,J,I,MAXVAR))
		   ENDIF
 108              CONTINUE
c***            Determine minimum value of the variable and constant ***
	        ELSE IF (Opchoice .EQ. 6) THEN
	          DO 109 I=1, Ndata
		   IF (Vmiss(Get Element(1,J,I,MAXVAR))) THEN
		     Tmpdata(I) = Missing
		   ELSE
		   Tmpdata(I)=AMIN1(Constant2,Get Element(1,J,I,MAXVAR))
		   ENDIF
 109              CONTINUE
c***            Calculate Mean value of the variable and constant ***
	        ELSE IF (Opchoice .EQ. 7) THEN
	          DO 110 I=1, Ndata
		   IF (Vmiss(Get Element(1,J,I,MAXVAR))) THEN
		     Tmpdata(I) = Missing
		   ELSE
		   Tmpdata(I)= (Constant2+Get Element(1,J,I,MAXVAR))/2.0
		   ENDIF
 110              CONTINUE
                ENDIF
              ENDIF 
c***        Perform operations on two variables ***
            ELSE IF (B1vflag .AND. B2vflag) THEN
              Fldchoice = 'B OPERAND1'
	      CALL Get Toggle Value (Fldchoice, K, O1choice)
c***          V1 identifies the user selected variable for Operand 1.
              V1 = K + 1
              Fldchoice = 'B OPERAND2'
	      CALL Get Toggle Value (Fldchoice, K, O2choice)
c***          V2 identifies the user selected variable for Operand 2.
              V2 = K + 1
              IF (Opchoice .EQ. 0) THEN
                DO 73 I=1, Ndata
		  IF (Vmiss(Get Element(1,V1,I,MAXVAR))
     +               .OR. Vmiss(Get Element(1,V2,I,MAXVAR))) THEN
c***                If the sample of Operand 2 or Operand 2 is missing 
c***                then assign a missing value.
		    Tmpdata(I) = Missing
		  ELSE
c***              Perform addition on two variables ***
		    Tmpdata(I) = Get Element(1,V1,I,MAXVAR) + 
     +                           Get Element(1,V2,I,MAXVAR)
		  ENDIF
  73            CONTINUE
c***          Perform division of variable 1 by variable 2 ***
              ELSE IF (Opchoice .EQ. 1) THEN
                DO 74 I = 1, Ndata
		  IF (Vmiss(Get Element(1,V1,I,MAXVAR)) .OR. 
     +                Vmiss(Get Element(1,V2,I,MAXVAR)) .OR.
     +		      (Get Element(1,V2,I,MAXVAR) .EQ. 0)) THEN
		    Tmpdata(I) = Missing
		  ELSE
		    Tmpdata(I) = Get Element(1,V1,I,MAXVAR) / 
     +                           Get Element(1,V2,I,MAXVAR)
		  ENDIF
  74            CONTINUE 
C***          Perform multiplication of two variables ***
              ELSE IF (Opchoice .EQ. 2) THEN
              DO 75 I=1, Ndata
		IF (Vmiss(Get Element(1,V1,I,MAXVAR)) .OR. 
     +                    Vmiss(Get Element(1,V2,I,MAXVAR))) THEN
		  Tmpdata(I) = Missing
		ELSE
		  Tmpdata(I) = Get Element(1,V1,I,MAXVAR) * 
     +                         Get Element(1,V2,I,MAXVAR)
		ENDIF
  75          CONTINUE
C***          Subtract variable 2 from variable 1 ***
              ELSE IF (Opchoice .EQ. 3) THEN
                DO 76 I=1, Ndata
		  IF (Vmiss(Get Element(1,V1,I,MAXVAR)) .OR. 
     +                    Vmiss(Get Element(1,V2,I,MAXVAR))) THEN
		    Tmpdata(I) = Missing
		  ELSE
		    Tmpdata(I) = Get Element(1,V1,I,MAXVAR) - 
     +                           Get Element(1,V2,I,MAXVAR)
		  ENDIF
  76            CONTINUE
c***          Raise variable 1 to the power of variable 2 ***
              ELSE IF ( Opchoice .EQ. 4) THEN
                DO 77 I=1, Ndata
                  Rconstant =  AINT(Get Element(1,V2,I,MAXVAR))
c***              check for a zero operand and a negative exponent ***
		  IF (Vmiss(Get Element(1,V1,I,MAXVAR)) .OR. 
     +                    Vmiss(Get Element(1,V2,I,MAXVAR))) THEN
		    Tmpdata(I) = Missing
	          ELSE IF ((Get Element(1,V1,I,MAXVAR) .EQ. 0) .AND.
     +                     (Get Element(1,V2,I,MAXVAR) .LT. 0)) THEN
		    Tmpdata(I) = Missing
c***              check for a negative raised to a nonintegral real 
c***              exponent ***
                  ELSE IF ((Get Element(1,V1,I,MAXVAR) .LT. 0) .AND.
     +               (Get Element(1,V2,I,MAXVAR) .GT. Rconstant)) THEN
		    Tmpdata(I) = Missing
	          ELSE
	            Tmpdata(I) = Get Element(1,V1,I,MAXVAR)
     +                           ** Get Element(1,V2,I,MAXVAR)
	          ENDIF
  77	        CONTINUE
C***          Determine maximum value of variable 2 and variable 1 ***
              ELSE IF (Opchoice .EQ. 5) THEN
                DO 111 I=1, Ndata
		  IF (Vmiss(Get Element(1,V1,I,MAXVAR)) .AND. 
     +                    Vmiss(Get Element(1,V2,I,MAXVAR))) THEN
		    Tmpdata(I) = Missing
		  ELSEIF (Vmiss(Get Element(1,V1,I,MAXVAR))) THEN 
		     Tmpdata(I) = Get Element(1,V2,I,MAXVAR)
		  ELSEIF (Vmiss(Get Element(1,V2,I,MAXVAR))) THEN 
		     Tmpdata(I) = Get Element(1,V1,I,MAXVAR)
                  ELSE
		     Tmpdata(I) = AMAX1(Get Element(1,V1,I,MAXVAR),
     +                                  Get Element(1,V2,I,MAXVAR))         
		  ENDIF
  111           CONTINUE
C***          Determine maximum value of variable 2 and variable 1 ***
              ELSE IF (Opchoice .EQ. 6) THEN
                DO 112 I=1, Ndata
		  IF (Vmiss(Get Element(1,V1,I,MAXVAR)) .AND. 
     +                    Vmiss(Get Element(1,V2,I,MAXVAR))) THEN
		    Tmpdata(I) = Missing
		  ELSEIF (Vmiss(Get Element(1,V1,I,MAXVAR))) THEN 
		     Tmpdata(I) = Get Element(1,V2,I,MAXVAR)
		  ELSEIF (Vmiss(Get Element(1,V2,I,MAXVAR))) THEN 
		     Tmpdata(I) = Get Element(1,V1,I,MAXVAR)
                  ELSE
		     Tmpdata(I) = AMIN1(Get Element(1,V1,I,MAXVAR),
     +                                  Get Element(1,V2,I,MAXVAR))         
		  ENDIF
 112            CONTINUE
C***          Determine mean value of variable 2 and variable 1 ***
              ELSE IF (Opchoice .EQ. 7) THEN
                DO 113 I=1, Ndata
		  IF (Vmiss(Get Element(1,V1,I,MAXVAR)) .AND. 
     +                    Vmiss(Get Element(1,V2,I,MAXVAR))) THEN
		    Tmpdata(I) = Missing
		  ELSEIF (Vmiss(Get Element(1,V1,I,MAXVAR))) THEN 
		     Tmpdata(I) = Get Element(1,V2,I,MAXVAR)
		  ELSEIF (Vmiss(Get Element(1,V2,I,MAXVAR))) THEN  
		     Tmpdata(I) = Get Element(1,V1,I,MAXVAR) 
                  ELSE
		     Tmpdata(I) = (Get Element(1,V1,I,MAXVAR) +
     +                             Get Element(1,V2,I,MAXVAR)) / 2.0         
		  ENDIF
 113            CONTINUE
	      ENDIF
	  ELSE
            CALL Flash(20,2,
     +		     'Binary operand(s) not specified (press any key)')
	    CALL Clear Operation Box
	    Calcomp = .TRUE.
	    RETURN
	  ENDIF
	ELSE
	  CALL Flash(20,2,
     +	   'Variable to contain results not specified (press any key)')
	  CALL Clear Operation Box
	  Calcomp = .TRUE.
	  RETURN
	ENDIF
      ENDIF

	IF ((Newvar .OR. Oldvar).AND.(Uoperator.OR.Boperator))THEN
C***    Assign results to chosen variable ***
	  IF (Newvar) THEN
c***      Check for variable overload ***
            IF((Nvar + 1) .GT. MAXVAR) THEN
	      CALL Flash(20,2,
     +	       'Exceeding max number of variables (press any key)')
	      CALL Clear Operation Box
	      Calcomp = .TRUE.
	      RETURN
	    ELSE
c***        Assign additional position for new variable ***
	      Nvar = Nvar + 1
              J = Nvar
C***          Assign data results to the Data Matrix ***
	      DO 78 I=1, Ndata
	        CALL Put Element (1,Nvar,I,MAXVAR,Tmpdata(I),Error)
		IF (Error) GOTO 799
  78	      CONTINUE
	    ENDIF
	  ELSE
	    Yes = Get Yes No
     +		    (20, 2,
     +	       ' Do you want new results to replace old (Y/N)')
	    IF (Yes) THEN
c***          Find which variable the Tmpvnam matches.  The 
c***          identifier for the variable is assigne to J.
	      DO 90 I = 1, Nvar
		IF (Tmpvnam .EQ. Varnam(I)) THEN
		  J = I
		  GOTO 95
		ENDIF
  90	      CONTINUE
  95          CONTINUE
C***          Assign data results to the Data Matrix ***
	      DO 79 I=1, Ndata
	        CALL Put Element (1,J,I,MAXVAR,Tmpdata(I),Error)
		IF (Error) GOTO 799
  79	      CONTINUE
	    ELSE
	      CALL Clear Operation Box
	      Calcomp = .TRUE.
	      RETURN
	    ENDIF
	  ENDIF
c***      Set global flag to indicate that new data has been generated ***
	  Newdata = .TRUE.
c***      Assign New Name and New Units to correct array ***
	  IF (Newvar) THEN
	    Varnam(J) = Tmpvnam
	    Units(J)  = Tunits
c***        Load the revised array into the different toggle fields ***
	    CALL Load New Toggle Values
	    CALL Display Variables
          ENDIF
c***      Issue message regarding number of missing values if applicable.	  
	  DO 85 I=1, Ndata
cccc	    IF (Tmpdata(I) .EQ. Missing) THEN
	    IF (Vmiss(Tmpdata(I))) Nerrors = Nerrors + 1
  85	  CONTINUE
c***      Calculate and print min max results. J is the variable index ***
	  CALL Minmax(J, Nerrors)
	  IF (Nerrors .GT. 0) THEN
	    Fldchoice = 'NERROR    '
	    CALL Set Integer Value(Fldchoice, Nerrors)
	    CALL Display One Field (Fldchoice)
	    Ch = ' missing  value(s)  generated  in '''
	    CALL Put Str(Ch, 35, 20, 9, 2)
	    CALL Put Str(Varnam(J), 10, 20, 44, 2)
	    CALL Flash(20, 54,
     +	       ''' (press any key )')
	    CALL Erase(20,2,70)
	  ENDIF
        ENDIF

  799 CALL Clear Operation Box
      Calcomp = .TRUE.
      RETURN
  

      END
C**********************************************************************
