$LARGE
c**********************************************************************
c*** File name: TRANSF.FOR (Source code for the program TRANS)
c**********************************************************************
c	 G e t   H e a d e r  D a t a             (subroutine)
c**********************************************************************
      SUBROUTINE Get Header Data (File Name, Error, Nvar temp)
 
c*** This routine reads the following data from Geo-EAS data input file:
c***     1. The title(description) of the file.
c***     2. The number of variables in an input file.
c***     3. The variable names.
c***
c*** VARIABLES:
c*** File Name - This character string stores the name of the Geo-EAS
c***             data input file.
c*** Error     - Returns .TRUE. if the file could not be opened or if 
c***             an error occurs while reading the data.
c*** Nvar temp - Returns the number of variables in the input file. 

$INCLUDE:'Trans.dec'
$INCLUDE:'Screen.str'

      CHARACTER  File Name*(*)
      CHARACTER  Ch*1, Header*80, Readmess*16
      INTEGER*4	 J, I, Nvar temp
      LOGICAL*1  Error

      Error   = .False.
      Readmess = 'Reading Data ...'

c*** Concatenate prefix to filename ***
      CALL Put Str (Readmess, 16, 20, 2, 2)
      CALL Build Prefix Filename ( File Name)
      OPEN (UNIT = 10, FILE = Prefile, Err = 999)
	
c***  Read title(description) of Geo-EAS data file.
      READ (10,'(A)',ERR=999 ) Header
      Title (1:66) =  Header

c***  Get number of variables in file
      READ (10, *, ERR = 999) Nvar temp

c***  If the number of variables is outside the range of (1...48) then...
      IF (Nvar temp .LT. 1  .OR.  Nvar temp .GT. MAXVAR) GOTO 999

c***  Set format array 'Formats' to blanks
      DO 15 I = 1, Nvar temp
	 Formats(I) = '          '
  15  CONTINUE

c***  Get variable names for each variable in file
      DO 20 I = 1, Nvar temp
        READ (10, 11, Err = 999) Varnam(I), Units(I), Formats(I)
  11	FORMAT(2A10,A6)
	CALL Input Format Check(I)
  20  CONTINUE
      CALL Erase (20, 1, 78)
      CLOSE (10)
      RETURN

c***  Error has been found in data of file - return to calling subroutine.

 999  CALL Beep
      CLOSE (10)
      CALL Erase (20, 1, 78)
      CALL Flash (20, 2, 
     +     'ERROR - reading data file...(press any key)')
      Error = .TRUE.

      RETURN
      END
c**********************************************************************
c        R e a d   N u m e r i c a l   D a t a       (subroutine)
c**********************************************************************
      SUBROUTINE Read Numerical Data (Unitnmbr,Filename,Nvar temp,
     +                                Nvar file,Error,Recstart)

c***  This routine reads only the samples of the Geo-EAS data input file.
c***
c***  VARIABLES:
c***  Unitnmbr  - unit number represents the 'data' matrix simulated by
c***              scratch file, Unit number = 1 for 'data1'.
c***  Filename  - Name of file from which to read data.
c***  Nvar temp - number of variables in input file.
c***  Nvar file - number of variables in output file.
c***  Error     - Returns .TRUE. if error occurred whilst opening or 
c***              reading file, else true.
c***  Recstart  - global variable - indicates the first record with
c***              sample data and is provided by the calling subroutine.
c***  Prefile   - global variable, stores the filename preceded by a path
c***              if the path was specified.
c***  Ndata     - global variable, stores the number of samples in the file.
c***  Infil     - global variable, stores the filename only, no path.
c***  Vector48  - global variable, declared as 'real*4 Vector48(48)'.
c***              Temporary array which stores one record of sample data.
 
$INCLUDE:'Trans.dec'
$INCLUDE:'Screen.str'

      INTEGER*4   Nvar temp, I, N, J, Unitnmbr, Nvar file
      LOGICAL*1   Error
      CHARACTER   Filename*(*), Ch*1, Readmess*16

      Error = .FALSE.
      Readmess = 'Reading Data ...'
      CALL Put Str (Readmess, 16, 20, 2, 2)
c***  Open scratch file to contain 'Data' 'structure'. ***
      CALL Open Scratch Files
c***  Attach the prefix(if any) to the file name.  The global variable
c***  'Prefile' stores the prefix + file name.
      CALL Build Prefix Filename (Filename)
      OPEN (UNIT = 10, FILE = Prefile, ERR = 899)
c***  Skip over records until the first record containing sample data 
c***  has been reached.
      DO 15 I = 1, Recstart - 1
        READ (10, '(A)', ERR = 999, END = 999) Ch
  15  CONTINUE

c***  Get data
      N = 1
c***  Store one record of sample data into 'Vector48'.
  30  READ (10,*,END=40,Err=999) (Vector48(I),I=1,Nvar temp) 
c***  Write out each sample in 'Vector48' to the scratch file.
      DO 35 J = 1, Nvar temp
	I = J + (Nvar file * (N-1)) 
	WRITE(Unitnmbr,REC=I,ERR=799) Vector48(J)
  35  CONTINUE
      N = N + 1
      IF (N .GT. MAXDATA) THEN
	CALL Beep
	CALL Flash (20, 2, 
     +     'Warning - Too many samples in data file...(press any key)')
	GOTO 40
      ENDIF
      GOTO 30

  40  CONTINUE
      Ndata = N - 1
      CALL Erase (20, 1, 78)
      CLOSE (10)
      RETURN

c*** Error has been found in data of file - return to calling subroutine.

  799 CALL Beep
      Error = .TRUE.
      CLOSE (10)
      CALL Flash (20, 2,
     + 'ERROR-not enough space to write scratch file...(press any key)')
      RETURN

  899 CALL Beep
      Error = .TRUE.
      CLOSE (10)
      CALL Flash (20, 2,
     +     'ERROR - opening data file...(press any key)')
      RETURN

 999  CALL Beep
      Error = .TRUE.
      CLOSE (10)
      CALL Erase (20, 1, 78)
      CALL Flash (20, 2, 
     +      'ERROR - reading data file...(press any key)')

      RETURN
      END
c**********************************************************************
c	 B u i l d  P r e f i x  F i l e n a m e     (subroutine)
c**********************************************************************
      SUBROUTINE Build Prefix Filename (File Name)
 
c*** This routine concatenates a prefix to a filename.	 
c***
c*** VARIABLES:
c*** File Name -  Name of file to which the prefix is affixed.
c*** Prefix    -  Global variable which stores the prefix character string.
c*** Prefile   -  Global variable which stores the Prefix + File Name.

$INCLUDE:'Trans.dec'
$INCLUDE:'Screen.str'

      CHARACTER  File Name*(*)
      INTEGER*2  I, J, K

c***  Initialize Prefile to blanks.
      DO 15 I = 1, 77
        Prefile(I:I) = ' '
  15  CONTINUE
c***  Affix Prefix to the File Name and assign to Prefile.
      J = LENSTR(Prefix)
      Prefile = Prefix 
      DO 10 I = 1, 14
	K = J + I
c*** Prefile   -
	Prefile(K:K) = File Name(I:I)
  10  CONTINUE

      RETURN
      END

c**********************************************************************
c	 D e l e t e   O p t i o n     (subroutine)
c**********************************************************************
      SUBROUTINE Delete Option (Ivar, Error)
 
c***  This routine deletes a variable by advancing all succeeding variables
c***  one positon.
c***
c***  VARIABLES:
c***  Ivar    - This integer identifies the column number of the variable
c***            to be deleted.
c***  Error   - Return .TRUE. if the data could not be written out to the
c***            scratch file.
c***  MAXVAR  - Global variable that stores the maximum number of 
c***            variables permitted in Trans.
c***  Nvar    -	Global variable containing the number of varibles.
c***  Varnam  -	Global array of names for each variable.
c***  Units   -	Global array of measurement description s for each variable.
c***  Formats -	Global array of output formats for each variable.

$INCLUDE:'Trans.dec'
$INCLUDE:'Screen.str'

      CHARACTER*10  Uoperand,Boperand1,Boperand2,Dvariable,Svariable
      CHARACTER*10  Ioperand, Writemess*19
      INTEGER*4     Ivar
      REAL*4        Value, Get Element
      LOGICAL*1     Passover, Error

c***  Assign screen field names.
      Uoperand    = 'U OPERAND '
      Boperand1   = 'B OPERAND1'
      Boperand2   = 'B OPERAND2'
      Ioperand    = 'I OPERAND '
      Dvariable   = 'DVARIABLE '
      Svariable   = 'SVARIABLE '
      Error       = .FALSE.

c***  Display processing message.
      Writemess	  = ' Processing data...'
      CALL Put Str (Writemess, 19, 20, 2, 2)

c***  Advance the sample data of all variables succeeding the variable to be
c***  deleted by one position.
      DO 20 I = Ivar, Nvar - 1
        DO 30 J = 1, Ndata
          Value = Get Element(1,I+1,J,MAXVAR)
	  CALL Put Element(1,I,J,MAXVAR,Value,Error)
	  IF (Error) GOTO 799
  30  	CONTINUE
  20  CONTINUE

c***  Advance the succeeding variable names by one position.
      Nvar = Nvar - 1
      DO 40 I = Ivar, Nvar
	Varnam(I)  = Varnam(I+1)
	Units(I)   = Units(I+1)
	Formats(I) = Formats(I+1)
  40  CONTINUE

c***  Reload the toggle fields. 
      CALL Load New Toggle Values
      CALL Display Variables
      Newdata = .TRUE.
      CALL Flash (20, 2,
     +      'Variable has been deleted...(press any key)')

c***  Erase the operation name and the name of the variable to be
c***  deleted.
  799 CALL Erase (8,  2, 13)
      CALL Erase (8, 16, 53)

      RETURN
      END	

c**********************************************************************
c	 Load New Toggle Values     (subroutine)
c**********************************************************************
      SUBROUTINE Load New Toggle Values 

c***  This routine loads all variable names into the appropriate 
c***  toggle fields.
c***
c***  VARIABLE:
c***  Fldchoice - Name of screen label field.

$INCLUDE:'Trans.dec'
$INCLUDE:'Screen.str'

      CHARACTER*10  Fldchoice

c***  Load the revised variable name array into the different 
c***  toggle fields.
      Fldchoice = 'DVARIABLE '
      CALL Load Toggle Values(Fldchoice, 10, Nvar, Varnam, 0)
      Fldchoice = 'SVARIABLE '
      CALL Load Toggle Values(Fldchoice, 10, Nvar, Varnam, 0)
      Fldchoice = 'U OPERAND '
      CALL Load Toggle Values(Fldchoice, 10, Nvar, Varnam, 0)
      Fldchoice = 'B OPERAND1'
      CALL Load Toggle Values(FLdchoice, 10, Nvar, Varnam, 0)
      Fldchoice = 'B OPERAND2'
      CALL Load Toggle Values(Fldchoice, 10, Nvar, Varnam, 0)
      Fldchoice = 'I OPERAND '
      CALL Load Toggle Values(Fldchoice, 10, Nvar, Varnam, 0)
 
      RETURN
      END	
c**********************************************************************
c	 I n p u t  F o r m a t  C h e c k     (subroutine)
c**********************************************************************
      SUBROUTINE Input Format Check(Col)

c***  This routine checks if the format listed in the input file is
c***  acceptable.  The acceptable formats are 'F' or 'G' where the
c***  width field is not greater than 16 and the decimal positions is
c***  not greater than 9.  The width field cannot fall below (decimal  
c***  positions + 1) and the decimal positions cannot fall below 0.  
c***  If they do then the default format of G16.9 is used.
c***
c***  VARIABLES:
c***  Col	- contains an integer value that indicates the
c***              variable (column) index. 
c***  Fwidth	- global contains the width field of the output format.
c***  Fdecimals - global that contains the decimal field of the output 
c***              format.
c***  Formats	- global array that contains the output formats for
c***              each variable.
c***  C1,C2,C3	- temporary variables that contain one character each.


$INCLUDE:'Trans.dec'

      CHARACTER*1  C1, C2, C3
      INTEGER*4    Col

      Fwidth = 0
      Fdecimals = 0
c***  Assign either a 'G' or an 'F' for the format.
      IF ((Formats(Col)(1:1) .EQ. 'F').OR.(Formats(Col)(1:1).EQ. 'f'))
     +   THEN 
            Formats(Col)(1:1) = 'F'
      ELSEIF((Formats(Col)(1:1).EQ.'G').OR.(Formats(Col)(1:1).EQ.'g'))
     +   THEN
            Formats(Col)(1:1) = 'G'
      ENDIF

c***  If the first character of the format if not a 'G' or 'F' then
c***  assign the default format of 'G16.9' for the variable in question
c***  and return to the calling subroutine.
      IF(.NOT.((Formats(Col)(1:1).EQ. 'F').OR.(Formats(Col)(1:1).EQ.'f')
     +   .OR.(Formats(Col)(1:1).EQ.'G').OR.(Formats(Col)(1:1).EQ.'g')))
     +   THEN
            Formats(Col) = 'G16.9     '
	    RETURN
      ELSE
	 IF (Formats(Col)(3:3).EQ.'.') THEN
c***        Process format with a one digit width value, eg., G9.5.
     	    C1 = Formats(Col)(2:2)
	    C2 = Formats(Col)(4:4)
c***        Make sure that the numeric characters are in fact numeric and
c***        not another type character else assign the default format
c***        and return to the calling subroutine.
	    IF ((C1.LT.'0').OR.(C1.GT.'9').OR.
     +          (C2.LT.'0').OR.(C2.GT.'9'))THEN
	        Formats(Col) = 'G16.9     '
		RETURN
	    ELSE
c***           Convert character values to integer values.
 	       Fwidth =  ICHAR(Formats(Col)(2:2)) - ICHAR('0')
	       Fdecimals = ICHAR(Formats(Col)(4:4)) - ICHAR('0')
	    ENDIF
	 ELSE IF (Formats(Col)(4:4).EQ. '.') THEN
c***        Process format with a two digit width value, eg., G15.7.
	    C1 = Formats(Col)(2:2)
	    C2 = Formats(Col)(3:3)
	    C3 = Formats(Col)(5:5)
c***        Make sure that the numeric characters are in fact numeric and
c***        not another type character else assign default format and
c***        return to calling subroutine..
	    IF ((C1.LT.'0').OR.(C1.GT.'9').OR.
     +	        (C2.LT.'0').OR.(C2.GT.'9').OR.
     +	        (C3.LT.'0').OR.(C3.GT.'9'))THEN
	        Formats(Col) = 'G16.9     '
		RETURN
	    ELSE
c***           Convert character values to integer values.
	       Fwidth = ICHAR(Formats(Col)(2:2)) - ICHAR('0')
	       Fwidth = ICHAR(Formats(Col)(3:3)) - ICHAR('0') +
     +               (10 * Fwidth)
	       Fdecimals = ICHAR(Formats(Col)(5:5)) - ICHAR('0')
	    ENDIF
         ELSE
c***        If the format has neither a one or two digit width field
c***        then assign the default value of 'G16.9' to the 'Formats'
c***        array and return to the calling subroutine.
	    Formats(Col) = 'G16.9     '
            RETURN
	 ENDIF
	 D1 = Fdecimals
	 W1 = Fwidth
c***     If the 'F' formats are used then check if formats are 
c***     in correct ranges.  F2.0 will produce asterisks.
	 IF ((Formats(Col)(1:1).EQ.'F').OR.(Formats(Col)(1:1).EQ.'f'))
     +      THEN
	      IF ((W1.GT.16).OR.(D1.GT.9).OR.(W1.LT.D1+1).OR.(W1.LT.2)
     +           .OR.(D1.LT.0))THEN
		   Formats(Col) = 'G16.9     '
	      ELSE
c***             Convert integers to character strings and replace the
c***             the 'G' inserted during the conversion with an 'F' since
c***             this was originally and 'F' format.
		 CALL Write Format from Int to Char(Col)
		 Formats(Col)(1:1) = 'F'
	      ENDIF
c***     If the 'G' formats are used then check if formats are 
c***     in correct ranges.  G5.0 will produce asterisks.
	 ELSE IF((Formats(Col)(1:1).EQ.'G')
     +               .OR.(Formats(Col)(1:1).EQ.'g')) THEN
	      IF ((W1.GT.16).OR.(D1.GT.9).OR.(W1.LT.D1+1).OR.
     +            (W1.LT.6).OR.(D1.LT.0))THEN
		  Formats(Col) = 'G16.9     '
	      ELSE
c***              Convert integers to character strings.
c***		  CALL Convert Int to Char Format(Fwidth,Fdecimals,Col)
	          CALL Write Format from Int to Char(Col)
	      ENDIF
	 ENDIF
      ENDIF
      RETURN
      END

c**********************************************************************
c     W r i t e  F o r m a t  f r o m  I n t  t o  C h a r (subroutine)
c**********************************************************************
      SUBROUTINE Write Format from Int to Char (Col)

c***  This routine converts the integer values in the variables 
c***  Fwidth and Fdecimals into a FORTRAN output format.  This format
c***  is then inserted into the global 'Formats' array element 
c***  corresponding to the variable indicated by 'Col'. 
c***    
c***  VARIABLES:
c***  Fwidth	- global width of the FORTRAN output format.
c***  Fdecimals - global number of decimals to be included in the 
c***              output format. 
c***  Col	- variable (column) identifier.
c***  W1	- temporary variable equivalent to Fdecimals.
c***  F1	- temporary variable equivalent to Fwidth.
c***  Formats	- global array containing the output formats for each
c***              variable.

$INCLUDE:'Trans.dec'

      INTEGER*4 Col, I

c***  Convert the format output width and decimal variables(integer) to
c***  character type and insert into the array 'Formats'.  The 
c***  formats always use the G format.

      Formats(Col) = '          '
      Formats(Col)(1:1) = 'G'
c***  If width field is < than 9, process. (Eg. G9.6)
      IF (Fwidth .LT. 10) THEN
	Formats(Col)(2:2) = CHAR(ICHAR('0') + Fwidth)
	Formats(Col)(3:3) = '.'
	Formats(Col)(4:4) = CHAR(ICHAR('0') + Fdecimals)
c***  If width field is > than 9, process. (Eg. G15.6)
      ELSE
	I = INT(Fwidth/10)
	Formats(Col)(2:2) = CHAR(ICHAR('0') + I)
	I = MOD (Fwidth, 10)
	Formats(Col)(3:3) = CHAR (ICHAR('0') + I)
	Formats(Col)(4:4) = '.'
	Formats(Col)(5:5) = CHAR(ICHAR('0') + Fdecimals)
      ENDIF
      RETURN
      END
c**********************************************************************

