$LARGE
C*********************************************************************
C*** File Name: REGRESS.FOR (Source code for the program XYGRAPH)
C*********************************************************************
C SUBROUTINE REGRESS (XDATA, YDATA)
C
C Formal Arguements:
C  XDATA   -  Array of X data values.
C  YDATA   -  Array of Y data values.
C
C Local Variables:
C  N              - Number of data values.
C  Xbar, Ybar     - Mean of the X and Y data values.
C  Sumx, Sumy     - Sum of the X and Y data value. 	
C  SumXsq,SumYsq  - Sum of the X and Y data values squared.  	
C  SumXY   	  - Sum of the X and Y data values multiplied.
C  Correlation 	  - Stores the correlation coefficient.
C  Slope	  - Slope of the regression line.	
C  Intercept	  - Y-intercept of the regression line.
C
C This routine calculates linear regression then plots the regression
C line and the regression coefficients on the graph. 

        SUBROUTINE Regress (Xdata, Ydata)

        IMPLICIT INTEGER*4 (A-Z)

$INCLUDE: 'XYGRAPH.CMN'

        CHARACTER       Corr*10, Rsq*35
        REAL*4          Xdata (1), Ydata (1)
	REAL*4		X, Y, Xbar, Ybar, Missing , X1, X2, Y1, Y2
        REAL*4          Slope, Intercept, Correlation
	REAL*8		Sxx, Syy, Sxy
        REAL*8          SumX, SumY, SumXsq, SumYsq, Sumxy, Divisor
	LOGICAL		Error

	DATA 		Missing /1.E31/

	Error = .TRUE.       
	IF (NData .EQ. 0) RETURN

C *** Initialize variables
        N		= 0
        Xbar    	= 0.0
        Ybar    	= 0.0
        Sumx    	= 0.0
        Sumy    	= 0.0
        SumXsq  	= 0.0
        SumYsq  	= 0.0
        SumXY   	= 0.0
        Sxx     	= 0.0
        Syy     	= 0.0
        Sxy     	= 0.0
        Correlation 	= 0.0
        Slope	 	= 0.0
	Intercept	= 0.0

C ***  Get data values
	DO 10 I = 1, Ndata
           IF (Xdata (I) .GE. Missing .OR.
     +         Ydata (I) .GE. Missing) GOTO 10
              N      = N + 1
              X      = Xdata (I)
              Y      = Ydata (I) 
              SumX   = SumX + X
              SumY   = SumY + Y
              SumXsq = SumXsq +  X * X
              SumYsq = SumYsq +  Y * Y
              SumXY  = SumXY  +  X * Y
 10     CONTINUE 

C *** Calculate regression statistics        
        IF (N .GT. 0) THEN
	   Error   = .FALSE.
           Xbar         = SumX / N
           Ybar         = SumY / N
           Sxx          = SumXsq - (SumX * SumX) / N
           Syy          = SumYsq - (SumY * SumY) / N
           Sxy          = SumXY  - (SumX * SumY) / N

           Divisor      = ((N*SumXsq) - (SumX * SumX))
           IF (Divisor .NE. 0)
     +        Slope     = ((N*SumXY)  - (SumX*SumY)) / Divisor
     
           Intercept    = Ybar - Slope * Xbar

            Divisor       = SQRT (Sxx * Syy)
            IF (Divisor .NE. 0.) 
     +          Correlation      = Sxy / Divisor
        ENDIF         

C ***   Calculate regression line
        y = slope*Xmax+Intercept
        X1 = Xmin
        Y1 = Slope*Xmin+Intercept
        X2 = Xmax
        Y2 = Slope*Xmax+Intercept
      
        CALL OPTN ('COLOR','BLACK')
        CALL Clip(X1,Y1,X2,Y2,Xmin,Xmax, Ymin, Ymax)


C ***  Plot correlation coefficient

        WRITE (Corr, '(F10.3)') Correlation
        CALL LeftJustify (Corr)
	la = Lenstr(Corr)
        
	Rsq = 'Correlation Coefficient: '//Corr(1:la)
	IF (Legend Position .EQ. 'Bottom') THEN
	   CALL SET (.15,.95,.0,.17,0.,6.,0.,6.,1)
	   CALL PWRIT (3.,4.,Rsq,35,2,-1,1)
	ELSE IF (Legend Position .EQ. 'Right ') THEN
	   CALL SET (.80,1.,.60,.85,0.,6.,0.,6.,1)
	   CALL PWRIT (6.,6.,Rsq,35,2,-1,1)
	ELSE
	   CALL SET (.15,.95,0.,.13,0.,6.,0.,6.,1)
	   CALL PWRIT (3.,2.,Rsq,35,2,-1,1)
	ENDIF
	
        RETURN
        END

C*************************************************************************
C SUBROUTINE CLIP (X1, Y1, X2, Y2, XMIN, XMAX, YMIN, YMAX)
C
C Formal Arguments:
C  X1,Y1       -  First point on the line.
C  X2,Y2       -  Second point on the line.
C  XMIN,YMIN   -  Minimum X and Y coordinates for the boundary area.
C  XMAX,YMAX   -  Maximum X and Y coordinates for the boundary area.

C This routine is for clipping lines.  It was taken from pages 133-134 of
C "COMPUTER GRAPHICS", by  Hearn and Baker.

        SUBROUTINE Clip (X1, Y1, X2, Y2, Xmin, Xmax, Ymin, Ymax)

        REAL*4  X1, Y1, X2, Y2, Xmin, Xmax, Ymin, Ymax

        REAL*4  u1, u2, dx, dy

        LOGICAL Cliptest

        u1 = 0.0
        u2 = 1.0
        dx = X2 - X1
        IF (Cliptest(-dx, X1-Xmin, u1, u2)) THEN
           IF (Cliptest(dx, Xmax-X1, u1, u2)) THEN
              dy = Y2 - Y1
              IF (Cliptest(-dy, Y1-Ymin, u1, u2)) THEN
                 IF (Cliptest(dy, Ymax-Y1, u1, u2)) THEN

C *** If u1, and u2 are within range of 0 to 1, use to calculate
C *** new line endpoints.
                    IF (u2 .LT. 1.0) THEN
                       X2 = X1 + u2 * dx
                       Y2 = Y1 + u2 * dy
                    ENDIF
                    IF (u1 .GT. 0.0) THEN
                       X1 = X1 + u1 * dx
                       Y1 = Y1 + u1 * dy
                    ENDIF
                 ENDIF
              ENDIF
           ENDIF
        ENDIF


        CALL Line(X1, Y1, X2, Y2)

        RETURN
        END
             
C**************************************************************************
C LOGICAL FUNCTION CLIPTEST (P,Q,U1,U2)
C
C Formal Arguments:
C  P,Q   - Coordinates of the line.
C  U1,U2 - The part of the line that lies within the boundary.
C
C This routine determines whether the line can be rejected or the
C intersection parameters are to be adjusted.  Taken from  pages
C 133-134 of "COMPUTER GRAPHICS", by Hearn and Baker.
C
        LOGICAL FUNCTION Cliptest (p,q,u1,u2)
         
        REAL*4  r,p,q,u1,u2

        LOGICAL Result

        Result = .TRUE.

C *** Line from outside to inside of this boundary
        IF (p .LT. 0.0) THEN
           r = q/p

C *** Reject line or update u1 if appropriate
           IF (r .GT. u2) THEN
              Result = .FALSE.
           ELSE IF (r .GT. u1) THEN
               u1 = r
           ENDIF
 
C *** Line from outside to inside of this boundary
       ELSE IF (p .GT. 0.0) THEN
           r = q/p

C *** Reject line or update u2 if appropriate
           IF (r .LT. u1) THEN
              Result = .FALSE.
           ELSE IF (r .LT. u2) THEN
              u2 = r
           ENDIF

C *** p = 0; line parallel to boundary
        ELSE
           IF (q .LT. 0.0) Result = .FALSE.
        ENDIF

        Cliptest = Result

        RETURN
        END


