$LARGE
C*********************************************************************
C*** File Name: RPARAM.FOR (Source code for the program XYGRAPH)
C*********************************************************************
C SUBROUTINE READ PARAM FILE (ERROR)
C
C Formal Arguments:
C  ERROR      - Logical variable set to true if an error occurs.
C
C Local Variables:
C  Signature  - Character string to identify the parameter file.
C
C This routine processes the input parameter file and sets the screen
C fields. It calls Get Curve Parms and Get Plot Parms to processes the 
C curve and plot parameters.  If an error occurs a message will be
C displayed in the message area.
C  
	SUBROUTINE Read Param File (Error) 
 
        IMPLICIT INTEGER*4 (A-Z)

$INCLUDE: 'SCREEN.STR'
$INCLUDE: 'XYGRAPH.CMN'

        CHARACTER*50   User DName, Blanks, Signature*15

	LOGICAL*1      ERROR

        DATA   Signature  /' XYGRAPH '/

        CALL Put Str ('Reading parameter file    ',25,20,2,2)

C *** Get identification signature
        READ (11,'(A15)',ERR=999,END=999) Text
        IF (Text(1:15) .NE. Signature) GOTO 999

C *** Get filenames and Independent Variable; set as defaults.

c        READ (11,'(A50)',ERR=999,END=999) User Dname
c        IF (File Prefix .EQ. Blanks) File Prefix = User Dname 

        READ (11,20,ERR=999,END=999) Param File,User Pfile,Data File,
     +  	                           Metacode File
   20   FORMAT (4(A14,4X))
        READ (11,'(A10)',ERR=999,END=999) Ind Var

        IF (Data File .EQ. Blanks(1:14)) GOTO 999
        IF (User Pfile .EQ. Blanks(1:14) .OR. 
     +      User Pfile .EQ. '              ')  User Pfile = Param File

c	CALL Set Character Value ('PREFIX    ',File Prefix)
        CALL Set Character Value ('PARAMETER ',Param File)
        CALL Set Character Value ('METACODE  ',Metacode File)
        CALL Set Character Value ('DATA FILE ',Data File)
        CALL Set Character Value ('OPARAMETER',User Pfile)
 
C *** Get curve parameters from parameter file; set defaults.
        CALL Get Curve Parms(ERROR)
	IF (ERROR) GOTO 999

C *** Get plot parameters from parameter file; set defaults.
        CALL Get Plot Parms(ERROR)
 	IF (ERROR)  GOTO 999
 
        CALL Display Flds
        CLOSE (11)
        RETURN

999     CLOSE (11) 
        Error = .TRUE.
   	CALL Beep
	CALL Putstr ('ERROR reading parameter file ',29,10,43,2)
	CALL Putstr ('An ERROR was encountered in  ',29,11,43,0)
	CALL Putstr ('reading the input parameter  ',29,12,43,0)
	CALL Putstr ('file.   Check the file name, ',29,13,43,0)
	CALL Putstr ('and that the named file has  ',29,14,43,0)
	CALL Putstr ('the standard format  for the ',29,15,43,0)
	CALL Putstr ('parameter file.  Consult the ',29,16,43,0)
        CALL Putstr ('Users Manual for more        ',29,17,43,0)
	CALL Putstr ('information about file format.',30,18,43,0)
	CALL Flash (20,2,'Press any key to exit ')
	CALL Erase Field ('PARAMETER ')
        CALL Set Character Value ('PARAMETER ', Blanks(1:14))
        CALL Erase (20,2,75)
        CALL Erase Message (10,42,31)	  
	
        RETURN
        END

C******************************************************************
C SUBROUTINE GET CURVE PARAMS (ERROR)
C
C Formal Arguments:
C  ERROR   - Logical variable set to true if an error occurs.
C
C This routine reads the records 5-10, from the parameter file
C repetitively as specified by record 4 (Num curves). The curve 
C parameters are checked and screen fields are set.
C  
	SUBROUTINE Get Curve Parms (ERROR)
 
	IMPLICIT INTEGER*4 (A-Z)
 
$INCLUDE: 'SCREEN.STR'
$INCLUDE: 'XYGRAPH.CMN'
 
        LOGICAL*1    ERROR

C ***  Get number of curves
 
	READ (11,'(I1)',ERR=999,END=999) Num Curves
 
C ***  Get parameters for each curve

	DO 10  i = 1, Num Curves
	   READ (11,'(A10)',ERR=999,END=999) Text
	   Var Args(i,1) = Text(1:35)

C ***  Line  type        
	   READ (11,'(A40)',ERR=999,END=999) Text
	   Var args(i,2) = Text(1:35)
           WRITE(Lccod,'(A7,I1)') 'LTYPE C', i
           READ(Text(27:27),'(I1)') pos
           CALL Set Toggle Position (Lccod,pos)
 
	   READ (11,'(A40)',ERR=999,END=999) Text
	   Var Args(i,3) = Text(1:35)
           READ (11,'(A40)',ERR=999,END=999) Text
	   Var Args(i,4) = Text(1:35)
 
C *** Symbol type
	   READ (11,'(A40)',ERR=999,END=999) Text
	   Var Args(i,5) = Text(1:35)
	   CALL Set Stype (Text(19:22), nt)
           WRITE(Sccod,'(A7,I1)') 'STYPE C', i
           CALL Set Toggle Position (Sccod, nt)

C *** Color	     
	   READ (11,'(A40)',ERR=999,END=999) Text
	   Var Args(i,6) = Text(1:35)
	   CALL Set Color (Text(12:19), Ccode, nt)
           WRITE(Cccod,'(A7,I1)') 'COLOR C', i
	   CALL Set Toggle Position (Cccod, nt)
	   
  10    CONTINUE

 
C ***  Dummy params in upper array

        IF (Num Curves .lt. Maxvar) THEN
	   DO 20  i = Num Curves+1,Maxvar
              WRITE(Yvarcod,'(A7,I1)') 'Y VAR C', i
	      CALL Set Toggle Value (Yvarcod,1,Var args(i,1))
              CALL Set Toggle Position(Yvarcod, 0) 	
           
              WRITE(Lccod,'(A7,I1)') 'LTYPE C', i
	      CALL Set Toggle Position (Lccod, 0)
            
           DO 25  j = 1,Maxvar
              Var Args(i,j) = Var Args(1,j)
  25       CONTINUE
 20        CONTINUE
	ENDIF

        RETURN

999     CLOSE (11) 
        ERROR = .TRUE.
  
        RETURN
        END
 

C************************************************************************
C SUBROUTINE GET PLOT PARMS (ERROR)
C
C Formal Arguments:
C  ERROR    - Logical variable set to true if an error occurs.
C
C This routine reads the parameter file for the basic plot
C parameters and sets appropriate screen field defaults.
C  
	SUBROUTINE Get Plot Parms (ERROR)
 
	IMPLICIT INTEGER*4 (A-Z)
 
$INCLUDE: 'SCREEN.STR'
$INCLUDE: 'XYGRAPH.CMN'
 
        LOGICAL*1   ERROR

C ****** Read all records into appropriate arrays for later processing

C *** AXES argument
	READ (11,'(A25)',ERR=999,END=999) Gen Args(1)
 
C *** X-axis arguments
	DO 10 i = 1,8
	   READ (11,'(A45)',ERR=999,END=999) Xaxis Args(i)
   10   CONTINUE
 
C *** Y-axis arguments
	DO 20 i = 1,8
	   READ (11,'(A45)',ERR=999,END=999) Yaxis Args(i)
   20   CONTINUE
 
C *** Title/Label Maximum and Label/Control arguments
	READ (11,'(A25)',ERR=999,END=999) Gen Args(2)
        READ (11,'(A25)',ERR=999,END=999) Gen Args(3)

C *** TITLE and Axis LABEL arguments
	DO 30  i = 1,5
	   DO 35  j = 1,3
	      READ (11,'(A60)',ERR=999,END=999) LabLin Args(i,j)
   35      CONTINUE
  30    CONTINUE
 
C *** Legend argument
        READ (11, '(A6)',ERR=999,END=999) Legend Position

C *** Regression argument 
	READ (11, '(A3)',ERR=999,END=999) Regression

C *** User arguments 
c       DO 40 j = 1,Maxuser
c          READ (11, '(A80)', ERR=999, END=900) User Args(j)
c  40   CONTINUE

c 900   CLOSE (11)

C ****** Extract argument values; check and set defaults ************
C *** Check AXES code; set toggle position default
 
	READ(Gen Args(1)(18:18),'(I1)') axcod
	IF (axcod .EQ. 3) THEN
	   nt = 0
	ELSE IF (axcod .EQ. 2) THEN
           nt = 2
        ELSE
	   nt = 1
	ENDIF

	CALL Set Toggle Position ('AXES      ', nt)

C *** Check X-axis parameters; set defaults
	CALL Xaxis Parms
 
C *** Check Y-axis parameters; set defaults
	CALL Yaxis Parms

C *** Extract and set TITLE and LABELs defaults
	ltitl = Lenstr(LabLin Args(1,3)) - 3
	CALL Set Character Value ('TITLE',LabLin Args(1,3)(16:ltitl))

	lstit1 = Lenstr(LabLin Args(2,3)) - 3
	CALL Set Character Value('SUBTITLE1',LabLin Args(2,3)(16:lstit1))
	
        lstit2 = Lenstr(LabLin Args(3,3)) - 3
	CALL Set Character Value('SUBTITLE2',LabLin Args(3,3)(16:lstit2))
 
	lxlab = Lenstr(LabLin Args(4,3)) - 3
	CALL Set Character Value('X LABEL',LabLin Args(4,3)(16:lxlab))
 
	lylab = Lenstr(LabLin Args(5,3)) - 3
	CALL Set Character Value('Y LABEL',LabLin Args(5,3)(16:lylab))
 
C *** Extract and set Legend defaults
        IF (Legend Position .EQ. 'Bottom') THEN
           nt=0
        ELSE IF (Legend Position .EQ. 'Right ') THEN
           nt=1
	ELSE
	   nt=2
        ENDIF               

        CALL Set Toggle Position ('LEGEND    ',nt)

C *** Extract and set Regression option
        IF (Regression .EQ. 'Yes') THEN
           nt=1
        ELSE 
           nt=0
        ENDIF

	CALL Set Toggle Position ('REGRESS',nt)

	RETURN

999     CLOSE (11) 
        ERROR = .TRUE.
 	
        RETURN
	END

C************************************************************************
C SUBROUTINE XAXIS PARMS
C
C This routine extracts values from X-axis parameter arguments and 
C sets the appropriate screen fields.
C  
	SUBROUTINE Xaxis Parms
 
	IMPLICIT INTEGER*4 (A-Z)
 
$INCLUDE: 'SCREEN.STR'
$INCLUDE: 'XYGRAPH.CMN'
 
	CHARACTER*10   xttyp, xtbas, xntyp, xnexp, xnfrac
        CHARACTER*13   cxmin, cxmax

C ***  xmin, xmax
	lxmin = Lenstr(Xaxis Args(1)) - 1
	lxmax = Lenstr(Xaxis Args(2)) - 1
	cxmin = Xaxis Args(1)(17:lxmin)
	cxmax = Xaxis Args(2)(17:lxmax)

	READ (cxmax, '(G10.3)') Xmax

        IF (cxmin(1:1) .NE. 'A') CALL Right Justify (cxmin,13)
        IF (cxmax(1:1) .NE. 'A') CALL Right Justify (cxmax,13)
	CALL Set Character Value('MIN X',cxmin)
	CALL Set Character Value('MAX X',cxmax)
      
C ***  Tick spacing, type & base
	lttyp = Lenstr(Xaxis Args(3)) - 1
	ltbas = Lenstr(Xaxis Args(4)) - 1
	xttyp = Xaxis args(3)(30:lttyp)
	xtbas = Xaxis Args(4)(30:ltbas)
        IF (xtbas .EQ. '1.E36') xtbas = '   Auto   '

        IF (xttyp .EQ. '1.E36' .OR. xttyp .EQ. 'Auto') THEN
	   nt = 4
        ELSE
	   READ(Xaxis Args(3)(30:30),'(I1)') nt
	   nt = nt 
        ENDIF

        CALL Set Toggle Position ('TIC TYP X ', nt) 

        IF (xtbas(4:4) .NE. 'A') CALL Right Justify (xtbas,10)
        CALL Set Character Value('TIC BAS X',xtbas)

C ***  Tick label type
	lntyp = Lenstr(Xaxis Args(5)) - 1

	xntyp = Xaxis Args(5)(32:lntyp)

	IF (xntyp .EQ. '1.E36' .OR. xntyp .EQ. 'Auto') THEN
	   nt = 0
	ELSE
	   READ(Xaxis Args(5)(32:32),'(I1)') nt
	   nt = nt + 1
	ENDIF

	CALL Set Toggle Position('TIC LAB X ', nt)

C *** Tick exponent and fraction
        lnexp = Lenstr(Xaxis Args(6)) - 1
        lnfrac = Lenstr(Xaxis Args(7)) - 1

        xnexp = Xaxis Args(6)(36:lnexp)
        xnfrac= Xaxis Args(7)(36:lnfrac)

        IF (xnexp .EQ. '1.E36') xnexp = '   Auto   '
        IF (xnfrac .EQ. '1.E36') xnfrac = '   Auto   '
 
        IF (xnexp(4:4) .NE. 'A') CALL Right Justify (xnexp,10)
        CALL Set Character Value('TIC EXP X',xnexp)
      
        IF (xnfrac(4:4) .NE. 'A') CALL Right Justify (xnfrac,10)
        CALL Set Character Value('TIC FRC X',xnfrac)

C ***  Xaxis scale
	READ (Xaxis Args(8)(21:21),'(I1)') nt
	CALL Set Toggle Position ('SCALE X   ', nt)

	RETURN
	END

C********************************************************************
C SUBROUTINE YAXIS PARMS
C
C This routine extracts values from Y-axis parameter arguments and 
C sets the appropriate screen fields.
C  
	SUBROUTINE Yaxis Parms
 
	IMPLICIT INTEGER*4 (A-Z)
 
$INCLUDE: 'SCREEN.STR'
$INCLUDE: 'XYGRAPH.CMN'
 
	CHARACTER*10   yttyp, ytbas, yntyp, ynexp, ynfrac
        CHARACTER*13   cymin, cymax

C ***  ymin, ymax
	lymin = Lenstr(Yaxis Args(1)) - 1
	lymax = Lenstr(Yaxis Args(2)) - 1
	cymin = Yaxis Args(1)(17:lymin)
	cymax = Yaxis Args(2)(17:lymax)

	READ (cymax, '(G10.3)') Ymax

        IF (cymin(1:1) .NE. 'A') CALL Right Justify (cymin,13)
	IF (cymax(1:1) .NE. 'A') CALL Right Justify (cymax,13)
	CALL Set Character Value('MIN Y',cymin)
	CALL Set Character Value('MAX Y',cymax)

C ***  Tick spacing, type & base
	lttyp = Lenstr(Yaxis Args(3)) - 1
	ltbas = Lenstr(Yaxis Args(4)) - 1

        yttyp = Yaxis args(3)(28:lttyp)
	ytbas = Yaxis Args(4)(28:ltbas)

        IF (ytbas .EQ. '1.E36') ytbas = '   Auto   '

	IF (yttyp .EQ. '1.E36' .OR. yttyp .EQ. 'Auto') THEN
	   nt = 4
	ELSE
	   READ(Yaxis Args(3)(28:28),'(I1)') nt
	   nt = nt 
	ENDIF

	CALL Set Toggle Position ('TIC TYP Y ', nt)

        IF (ytbas(4:4) .NE. 'A') CALL Right Justify (ytbas,10)
	CALL Set Character Value('TIC BAS Y',ytbas)

C ***  Tick label type
	lntyp = Lenstr(Yaxis Args(5)) - 1
	yntyp = Yaxis Args(5)(30:lntyp)

	IF (yntyp .EQ. '1.E36' .OR. yntyp .EQ. 'Auto') THEN
	   nt = 0
	ELSE
	   READ(Yaxis Args(5)(30:30),'(I1)') nt
	   nt = nt + 1
	ENDIF

	CALL Set Toggle Position ('TIC LAB Y ', nt)

C ***  Tick exponent  and fraction
        lnexp = Lenstr(Yaxis Args(6)) - 1
        lnfrac = Lenstr(Yaxis Args(7)) - 1

        ynexp = Yaxis Args(6)(34:lnexp)
        ynfrac= Yaxis Args(7)(34:lnfrac)

        IF (ynexp .EQ. '1.E36') ynexp = '   Auto   '
        IF (ynfrac .EQ. '1.E36') ynfrac = '   Auto   '
 
        IF (ynexp(4:4) .NE. 'A') CALL Right Justify (ynexp,10)
        CALL Set Character Value('TIC EXP Y',ynexp)
      
        IF (ynfrac(4:4) .NE. 'A') CALL Right Justify (ynfrac,10)
        CALL Set Character Value('TIC FRC Y',ynfrac)

C ***  Yaxis scale
	READ (Yaxis Args(8)(21:21),'(I1)') nt
	CALL Set Toggle Position ('SCALE Y   ', nt)

	RETURN
	END

C*********************************************************************
C SUBROUTINE SET STYPE (SCODE, NT)
C
C Formal Arguments:
C  SCODE    -  Stores the Hershey character code.
C  NT       -  Position to set toggle field
C
C This routine checks the Parameter File specification for the symbol
C type against the HERSHEY character code and returns a code for  
C setting the toggle position.
C 
	SUBROUTINE Set Stype (Scode, nt)
 
	IMPLICIT INTEGER*4 (A-Z)
       
	INTEGER*4   Nt
	CHARACTER*4 Scode
 
	IF (Scode .eq. '032J') nt = 0
	IF (Scode .eq. '032&') nt = 1
	IF (Scode .eq. '032:') nt = 2
	IF (Scode .eq. '032''') nt = 3
	IF (Scode .eq. '032?') nt = 4
	IF (Scode .eq. '032B') nt = 5
	IF (Scode .eq. '0322') nt = 6
  
	RETURN 
	END
 
C*********************************************************************
C SUBROUTINE SET COLOR (LCODE, CCODE, NT)
C
C Formal Arguments:
C  LCODE    - Color code from parameter file.
C  CCODE    - Stores the color code.
C  NT       - Position to set toggle field.
C
C This routine reads the parameter file specification of color (for  
C CALL to NCAR subroutine OPTN), and returns a code for setting the 
C toggle position.
C  
	SUBROUTINE Set Color (Lcode, Ccode, nt)
 
	IMPLICIT INTEGER*4 (A-Z)
 
	CHARACTER*8 Lcode, Ccode
 
	IF (Lcode(2:2) .eq. 'B') Then
	   IF (Lcode(3:3) .eq. 'L') Then
	      IF (Lcode(4:4) .eq. 'A') Then
 	         Ccode = '''BLACK'''
	         nt = 0
	         RETURN
 	      ELSE
 	         Ccode = '''BLUE'''
	         nt = 1
	         RETURN
 	      ENDIF
	   ELSE
 	      Ccode = '''BROWN'''
	      nt = 5
	      RETURN
 	   ENDIF
	ELSE
 	   IF (Lcode(2:2) .eq. 'R') Then
 	      Ccode = '''RED'''
	      nt = 2
	      RETURN
	   ELSE
	      IF (Lcode(2:2) .eq. 'Y') Then
	         Ccode = '''YELLOW'''
	         nt = 3
	         RETURN
	      ELSE
	         Ccode = '''GREEN'''
	         nt = 4
	         RETURN
	      ENDIF
	   ENDIF
	ENDIF
  
	END
 

