cdebug
clarge
C*******************************************************
C     
      SUBROUTINE DISCRT
C     
C     DISCRT breaks up a rectangular block into a user
C     defined grid pattern that will be used to compute
C     approximate point to block average variograms and
C     block variances.  
C     
C*******************************************************
C     
C     Variable declaration block.
C     
            include'cokrig.inc'      
      REAL XO,YO,ZO,DSX,DSY,DSZ
C     
C     Generate the local coordinates.
C     
      NDP = NPX*NPY*NPZ
      XO  = XLBL/FLOAT(NPX*2)
      YO  = YLBL/FLOAT(NPY*2)
      ZO  = -1.0*ZLBL/FLOAT(NPZ*2)
      DSX = XLBL/FLOAT(NPX)
      DSY = YLBL/FLOAT(NPY)
      DSZ = ZLBL/FLOAT(NPZ)
      ICNT = 0
C     
      DO 10 I = 1,NPZ
      DO 10 J = 1,NPY
      DO 10 K = 1,NPX
        ICNT = ICNT + 1
        XDB(ICNT) = XO + FLOAT(K-1)*DSX
        YDB(ICNT) = YO + FLOAT(J-1)*DSY
        ZDB(ICNT) = ZO - FLOAT(I-1)*DSZ
 10   CONTINUE
C     
      RETURN
      END
C***********************************************************
C     
      REAL FUNCTION BKVAR(N)
C     
C     Function BKVAR computes the block variance for a 
C     block via the moving stick algorithm.  It is assumed
C     that the number of discrete points that make up the
C     block and their general spatial locations have been
C     previously generated.  Utilizes function GAMAVG which
C     determines the average variogram between a discrete
C     point in the block and all the discrete points in the
C     block.
C     
C***********************************************************
C     
C     Variable declaration.
C     
            include'cokrig.inc'
      REAL P,GAMH
C     
      P = 0.0
      GAMH = 0.0
      DO 10 I = 1,NDP
        GAMH = GAMH + GAMAVG(XDB(I),YDB(I),ZDB(I),P,P,P,N)
 10   CONTINUE
      BKVAR =  GAMH/FLOAT(NDP)
C     
      RETURN
      END
C*********************************************************
C     
      REAL FUNCTION GAMMA(X1,Y1,Z1,X2,Y2,Z2,NVAR)
C     
C     Function GAMMA determines the value of a nested
C     variogram function between points (X1,Y1,Z1) and
C     (X2,Y2,Z2).
C     
C     Input Parameters.
C     
C     X1        X coordinate of first point.
C     Y1        Y coordinate of first point.
C     Z1        Z coordinate of first point.
C     X2        X coordinate of second point.
C     Y2        Y coordinate of second point.
C     Z2        Z coordinate of second point.
C     NVAR      Variable number J.
C     
C*********************************************************
C     
            include'cokrig.inc'
      INTEGER NVAR,I,J
      REAL D,DX,DY,DZ,H1,HX,HY,HZ,RATIO,BT,GAMI,X1,Y1,Z1,
     1     X2,Y2,Z2
cclong
       real ln20
       parameter(ln20 = 2.9957323)
cclong
C     
      GAMMA = 0.0
      IF((X1.EQ.X2).AND.(Y1.EQ.Y2).AND.(Z1.EQ.Z2))THEN
      ELSE
C*******Calculate gamma value if distance > zero.
        DO 50 J = 1,NEST(NVAR)
          I = NESTP(NVAR) + J
          IF(NVTYPE(I).EQ.1)THEN
C*********Nugget-effect model.
            GAMI  = C(I)
          ELSE
C*********Calculate model's equivalent scalar distance.
            DX = X1 - X2
            DY = Y1 - Y2
            DZ = Z1 - Z2
            H1 = DX*CTHETA(I) + DY*STHETA(I)
            HX = CBETA(I)*H1 + DZ*SBETA(I)
            HY = (DY*CTHETA(I) - DX*STHETA(I))*AFAOB(I)
            HZ = AFAOC(I)*(DZ*CBETA(I) - SBETA(I)*H1)
            D  = SQRT(HX*HX + HY*HY + HZ*HZ)
C***********Calculate contribution for type of nested model.
            IF(NVTYPE(I).EQ.2)THEN
C***********Spherical model.
              IF(D.GE.AAXIS(I))THEN
                GAMI  = C(I)
              ELSE
                RATIO = D/AAXIS(I)
                GAMI  = C(I)*(1.5*RATIO - 0.5*RATIO**3)
              ENDIF
            ELSEIF(NVTYPE(I).EQ.3)THEN
C***********Exponential model.
ccc               RATIO = -D/AAXIS(I)
               RATIO = -D/AAXIS(I) * ln20
ccc
               GAMI  = C(I)*(1.0 - EXP(RATIO))
            ELSEIF(NVTYPE(I).EQ.4)THEN
C***********Gaussian model.
ccc               RATIO = -(D/AAXIS(I))**2
               RATIO = -(D/AAXIS(I))**2 * ln20
ccc
               GAMI  = C(I)*(1.0 - EXP(RATIO))
            ELSEIF(NVTYPE(I).EQ.5)THEN
C***********Cubic model.
              IF(D.GE.AAXIS(I))THEN
                GAMI  = C(I)
              ELSE
                RATIO = D/AAXIS(I)
                GAMI  = C(I)*(7.0*RATIO**2 
     1                              - 8.75*RATIO**3
     2                              + 3.50*RATIO**5 
     3                              - 0.75*RATIO**7)
              ENDIF
            ELSEIF(NVTYPE(I).EQ.6)THEN
C***********Quadratic model.
              IF(D.GE.AAXIS(I))THEN
                GAMI  = C(I)
              ELSE
                RATIO = D/AAXIS(I)
                GAMI  = C(I)*(2.0*RATIO - RATIO**2)
              ENDIF
            ELSEIF(NVTYPE(I).EQ.7)THEN
C***********Hole-effect model.
              RATIO = AAXIS(I)*D
              GAMI  = C(I)*(1.0 - (SIN(RATIO)/(RATIO)))
            ELSEIF(NVTYPE(I).EQ.8)THEN
C***********General multiquadratic model.
              BT    = (1.0/C(I))**(1.0/AAXIS(I))
              GAMI  = C(I) - (D*D + BT)**(-AAXIS(I))
            ELSEIF(NVTYPE(I).EQ.9)THEN
C***********Uniform mean cosine model.
              BT     = C(I) + AAXIS(I)
              GAMI  = C(I) 
     1              - ( (SIN(BT*D) - SIN(AAXIS(I)*D)) / D )
            ELSEIF(NVTYPE(I).EQ.10)THEN
C***********Uniform mean spherical model.
              BT = C(I) + AAXIS(I)
              IF(D.GE.BT)THEN
                GAMI  = C(I)
              ELSEIF(D.LE.AAXIS(I))THEN
                RATIO = ALOG(BT/AAXIS(I))
                GAMI  = 1.5*D*RATIO 
     1          - ((D**3)/4.0)*(1.0/(AAXIS(I)*AAXIS(I)) - 1.0/(BT*BT))
              ELSEIF(D.LT.BT)THEN
                RATIO = ALOG(BT/D)
                GAMI  = D - AAXIS(I) + 1.5*D*RATIO
     1                - (D**3/4.0)*(1.0/(D*D) - 1.0/(BT*BT))
              ENDIF
            ELSEIF(NVTYPE(I).EQ.11)THEN
C***********Uniform mean gaussian model.
              BT = C(I) + AAXIS(I)
              GAMI  = C(I) 
     1              - ( (EXP(-AAXIS(I)*D*D) - EXP(-BT*D*D)) / (D*D) )
            ELSEIF(NVTYPE(I).EQ.12)THEN
C***********Uniform mean power model.
              IF(D.EQ.1.0)THEN
                GAMI  = C(I) - AAXIS(I)
              ELSE
                BT    = ALOG(D)
                GAMI  = ( (D**C(I) - D**AAXIS(I))/BT )
              ENDIF
            ELSEIF(NVTYPE(I).EQ.13)THEN
C***********Linear mean power model.
              IF(D.EQ.1.0)THEN
                GAMI  = (C(I)*C(I) - AAXIS(I)*AAXIS(I))/2.0
              ELSE
                BT    = ALOG(D)
                GAMI  = ( (C(I)*D**C(I) - AAXIS(I)*D**AAXIS(I))*BT
     1                - (D**C(I) - D**AAXIS(I)) ) / ( BT*BT )
              ENDIF
            ELSEIF(NVTYPE(I).EQ.14)THEN
C***********Exponential mean power model.
              BT     = D*EXP(-1.0)
              GAMI  = ( BT**C(I) - BT**AAXIS(I) ) / ( ALOG(D) - 1.0 )
            ELSEIF(NVTYPE(I).EQ.15)THEN
C***********Power model.
              GAMI  = C(I)*(D**AAXIS(I))
            ELSEIF(NVTYPE(I).EQ.16)THEN
C***********Gamma mean gaussian.
              GAMI  = (D*D + C(I))**AAXIS(I) - C(I)**AAXIS(I)
            ELSEIF(NVTYPE(I).EQ.17)THEN
C***********De-Wijsian model.
              IF(D.GT.1.0)THEN
                GAMI = C(I)*ALOG(D)
              ENDIF
            ENDIF
          ENDIF
          GAMMA = GAMMA + GAMI*NSIGN(I)
 50     CONTINUE
      ENDIF
C*****Return with computed value of gamma.
      RETURN
      END
C***********************************************************
C     
      FUNCTION GAMAVG(X1,Y1,Z1,XCB,YCB,ZCB,NVAR)
C     
C     GAMAVG computes the average variogram between a point,
C     located at (X1,Y1,Z1) and a block whose coordinates
C     for its SW corner-crest are (XCB,YCB,ZCB).  Utilizes
C     function GAMMA which computes the value of a nested
C     variogram between two points.  The number of discrete
C     points that make up the block and their general spatial
C     locations are assumed to be previously generated.  
C     The variogram function chosen is model NVAR.
C     
C***********************************************************
C     
C     Variable declarations.
C     
            include'cokrig.inc'
      DIMENSION D(3)
C     
      GAMH = 0.0
      DO 10 I = 1,NDP
        D(1) = XDB(I) + XCB
        D(2) = YDB(I) + YCB
        D(3) = ZDB(I) + ZCB
        GAMH = GAMH + GAMMA(X1,Y1,Z1,D(1),D(2),D(3),NVAR)
 10   CONTINUE
      GAMAVG = GAMH/FLOAT(NDP)
C     
      RETURN
      END
C***********************************************************
C     
      REAL FUNCTION CROSS(X1,Y1,Z1,X2,Y2,Z2,I,J)
C     
C     Function CROSS determines the cross covariance function
C     between point (X1,Y1,Z1) of attribute type I and point
C     (X2,Y2,Z2) of attribute type J with a sum, difference,
C     nested model, or sum - difference nested model.
C     
C******************************************************************
C     
C     Variable declaration block.
C     
            include'cokrig.inc'
      INTEGER I,J,II,JJ,IJ
      REAL CI,CJ,CK
C*****Compute pointers.
      II = NDG(I) + I
      JJ = NDG(J) + J
      IF(I.GE.J)THEN
        IJ = NDG(I) + J
      ELSE
        IJ = NDG(J) + I
      ENDIF
C*****Compute respective cross covariances.
      IF((NTYPCS(IJ).EQ.1).OR.(NTYPCS(IJ).EQ.2))THEN
        CI = SILL(II) - GAMMA(X1,Y1,Z1,X2,Y2,Z2,II)
        CJ = SILL(JJ) - GAMMA(X1,Y1,Z1,X2,Y2,Z2,JJ)
      ENDIF
      CK = SILL(IJ) - GAMMA(X1,Y1,Z1,X2,Y2,Z2,IJ)
      IF(NTYPCS(IJ).EQ.1)THEN
C*******Sum variable cross covariance.
        CROSS = 0.5*(CK - CI - CJ)
      ELSEIF(NTYPCS(IJ).EQ.2)THEN
C*******Difference variable cross covariance.
        CROSS = 0.5*(CI + CJ - CK)
      ELSEIF(NTYPCS(IJ).EQ.3)THEN
C*******Signed nested model cross covariance.
        CROSS = CK
      ELSEIF(NTYPCS(IJ).EQ.4)THEN
C*******Sum - difference variable cross covariance,
C*******which are stored in one nested model.
        CROSS = 0.25*CK
      ENDIF
C*****Return.
      RETURN
      END
C***********************************************************
C     
      REAL FUNCTION CRSAVG(X1,Y1,Z1,XCB,YCB,ZCB,I,J)
C     
C     FUNCTION CRSAVG COMPUTES THE AVERAGE CROSS COVARIANCE 
C     BETWEEN A POINT LOCATED AT (X1,Y1,Z1) AND A BLOCK, WHOSE
C     COORDINATES FOR ITS SW CORNER-CREST ARE (XCB,YCB,ZCB).
C     USES FUNCTION CROSS WHICH DETERMINES THE VALUE OF THE
C     CROSS COVARIANCE BETWEEN TWO POINTS.  THE NUMBER OF 
C     DISCRETE POINTS THAT MADE UP THE BLOCK AND THEIR GENERAL
C     SPATIAL LOCATIONS ARE ASSUMED TO BE GENERATED.
C     
C***********************************************************
C     
C     VARIABLE DECLARE BLOCK.
C     
            include'cokrig.inc'
      REAL D(3),GAMH,X1,Y1,Z1,XCB,YCB,ZCB
      INTEGER K,I,J
C     
      GAMH = 0.0
      DO 10 K = 1,NDP
        D(1) = XDB(K) + XCB
        D(2) = YDB(K) + YCB
        D(3) = ZDB(K) + ZCB
        GAMH = GAMH + CROSS(X1,Y1,Z1,D(1),D(2),D(3),I,J)
 10   CONTINUE
      CRSAVG = GAMH/FLOAT(NDP)
C     
      RETURN
      END
C***********************************************************
C     
      REAL FUNCTION TERM(X,Y,Z,EX,EY,EZ)
C     
C     Function TERM computes the basis function value at
C     a point located at (X,Y,Z).  The basis function is the
C     product of polynomials in X, Y, and Z with powers on
C     each of the X,Y,Z coordinates being EX, EY, and EZ.
C     
C***********************************************************
C     
C     Variable declarations.
C     
      REAL X,Y,Z,EX,EY,EZ,TX,TY,TZ
C     
      IF(EX.EQ.0.0)THEN
        TX = 1.0
      ELSE
        TX = X**EX
      ENDIF
      IF(EY.EQ.0.0)THEN
        TY = 1.0
      ELSE
        TY = Y**EY
      ENDIF
      IF(EZ.EQ.0.0)THEN
        TZ = 1.0
      ELSE
        TZ = Z**EZ
      ENDIF
C     
      TERM = TX*TY*TZ
C     
      RETURN
      END
C***********************************************************
C     
      REAL FUNCTION TERMAV(XCB,YCB,ZCB,XE,YE,ZE)
C     
C     Function TERMAV computes the average basis function
C     with power function exponents of XE, YE, ZE on the
C     X, Y, Z coordinates of the discrete points that make
C     up a block.
C     
C***********************************************************
C     
C     Variable declarations.
C     
            include'cokrig.inc'
      INTEGER I
      REAL TEMPT,XE,YE,ZE,XCB,YCB,ZCB,D(3)
C     
      TEMPT = 0.0
      DO 10 I = 1,NDP
        D(1) = XDB(I) + XCB
        D(2) = YDB(I) + YCB
        D(3) = ZDB(I) + ZCB
        TEMPT = TEMPT + TERM(D(1),D(2),D(3),XE,YE,ZE)
 10   CONTINUE
      TERMAV = TEMPT/FLOAT(NDP)
C     
      RETURN
      END
C*******************************************************
C     
      REAL FUNCTION DELH(X1,Y1,Z1,X2,Y2,Z2,AF1,AF2,
     1                   STHETA,CTHETA,SBETA,CBETA)
C     
C     Function DELH determines the elliptical distance
C     between points (X1,Y1,Z1) and (X2,Y2,Z2).
C     
C*******************************************************
C     
      REAL X1,Y1,Z1,X2,Y2,Z2,AF1,AF2,STHETA,CTHETA,
     1     SBETA,CBETA,D,DX,DY,DZ,H1,HX,HY,HZ,RATIO,B
C     
      IF((X1.EQ.X2).AND.(Y1.EQ.Y2).AND.(Z1.EQ.Z2))THEN
        DELH = 0.0
      ELSE
        DX = X1 - X2
        DY = Y1 - Y2
        DZ = Z1 - Z2
        H1 = DX*CTHETA + DY*STHETA
        HX = CBETA*H1 + DZ*SBETA
        HY = (DY*CTHETA - DX*STHETA)*AF1
        HZ = AF2*(DZ*CBETA - SBETA*H1)
        DELH = SQRT(HX*HX + HY*HY + HZ*HZ)
      ENDIF
C*****Return with computed value of delh.
      RETURN
      END
C**********************************************************
C     
      SUBROUTINE DIAG(NDG,N)
C     
C     Subroutine DIAG determines the diagonal term for
C     locating entries in a symmetric matrix that has
C     been stored in packed form.
C     
C**********************************************************
C     
      INTEGER NDG(1),N
C     
      DO 10 I = 1,N
        NDG(I) = (I*(I-1))/2
 10   CONTINUE
C     
      RETURN
      END
C**********************************************************
C     
      SUBROUTINE UPPER(A,NDG,N,TESPIV,ZERO,IERROR)
C     
C     SUBROUTINE UPPER FACTORS A REAL SYMMETRIC POSITIVE
C     DEFINITE MATRIX STORED IN PACKED FORM INTO UPPER
C     TRIANGULAR FORM BY NAIVE GAUSSIAN ELIMINATION.
C     
C     ON ENTRY
C       
C       A          REAL (N*(N+1)/2)
C                  THE PACKED FORM OF THE SYMMETRIC MATRIX A.
C                  THE COLUMNS OF THE UPPER TRIANGULAR MATRIX
C                  ARE STORED SEQUENTIALLY IN A ONE-DIMENSIONAL
C                  ARRAY OF LENGTH  N*(N+1)/2.
C       
C       NDG        INTEGER (N)
C                  POINTER ARRAY TO LOCATE ENTRIES IN A
C                  PACKED SYMMETRIC MATRIX A.
C       
C       N          INTEGER
C                  THE ORDER OF THE MATRIX A.
C       
C       TESPIV     INTEGER
C                  IF = 0, NO TEST OF ZERO PIVOT ELEMENT.
C                  IF > 0, TEST FOR ZERO PIVOT ELEMENT.
C       
C       ZERO       REAL
C                  INDICATOR OF ZERO PIVOT ELEMENT.
C     
C     ON RETURN
C     
C       A          A UPPER TRIANGULAR MATRIX, STORED IN PACKED
C                  FORM.  IF IERROR IS .NE. 0, THEN THE INPUTTED
C                  MATRIX HAS NOT BEEN COMPLETELY FACTORIZED.
C      
C**********************************************************
C      
C     VARIABLE DECLARATIONS.
C      
      REAL A(1),PIV,ZERO
      INTEGER K,I,N,NDG(1),IERROR,TESPIV
      IERROR = 0
C     
C     CONDUCT THE UPPER TRIANGULARIZATION.
C     
      IF(TESPIV.EQ.0)THEN
        DO 30 K = 1,N-1
          DO 20 I = K+1,N
            PIV = A(NDG(I)+K)/A(NDG(K)+K)
            DO 10 J = I,N
              A(NDG(J)+I) = A(NDG(J)+I) - PIV*A(NDG(J)+K)
 10         CONTINUE
 20       CONTINUE
 30     CONTINUE
      ELSE
        DO 60 K = 1,N-1
          DO 50 I = K+1,N
            IF(ABS(A(NDG(K)+K)).LE.ZERO)THEN
              IERROR = K
              RETURN
            ENDIF
            PIV = A(NDG(I)+K)/A(NDG(K)+K)
            DO 40 J = I,N
              A(NDG(J)+I) = A(NDG(J)+I) - PIV*A(NDG(J)+K)
 40         CONTINUE
 50       CONTINUE
 60     CONTINUE
      ENDIF
C     
      RETURN
      END
C**********************************************************
C     
      SUBROUTINE SOLVE(A,NDG,N,B,X,NRHS,SUM,WGT)
C     
C     SUBROUTINE SOLVE SOLVES THE REAL SYMMETRIC POSITIVE
C     DEFINITE SYSTEM   " A * X = B " FOR MULTIPLE RIGHT
C     HAND SIDE VECTORS B, BY THE FORWARD DECOMPOSITION 
C     AND THE BACK SUBSTITUTION PHASES OF NAIVE GAUSSIAN
C     ELIMINATION.  IT IS ASSUMED THE MATRIX A HAS BEEN
C     STORED IN PACKED FORM AND HAS BEEN FACTORIZED BY
C     SUBROUTINE UPPER.
C     
C     ON ENTRY
C     
C       A        REAL  (N*(N+1)/2).
C                THE FACTORIZED MATRIX FROM THE OUTPUT OF
C                SUBROUTINE UPPER.
C       
C       NDG      INTEGER (N).
C                POINTER ARRAY TO LOCATE ENTRIES IN
C                A PACKED SYMMETRIC MATRIX A.
C       
C       N        INTEGER
C                THE ORDER OF THE MATRIX A.
C       
C       NRHS     INTEGER
C                THE NUMBER OF RIGHT HAND SIDE VECTORS WHOSE
C                SOLUTION IS DESIRED.
C       
C       B        REAL   (NRHS*N)
C                THE RIGHT HAND SIDED VECTORS STORED IN COLUMN
C                FORM.  THE FIRST N ENTRIES ARE FOR THE FIRST
C                RIGHT HAND SIDE, THE NEXT N+1 TO 2N ENTRIES 
C                ARE FOR THE SECOND RIGHT HAND SIDE AND SO ON.
C       
C       WGT      REAL   (NRHS)
C                INDICATOR ARRAY TO INDICATE IF SOLUTION TO 
C                THE RHS "I" IS DESIRED.
C                WGT(I) = 1.0  :  FIND SOLUTION TO RHS "I".
C                WGT(I) = 0.0  :  NO SOLUTION TO RHS "I". 
C       
C     ON RETURN
C       
C       X        REAL   (NRHS*N)
C                THE SOLUTION TO THE RIGHT HAND SIDED VECTORS
C                STORED IN COLUMN FORM.  THE FIRST N ENTRIES 
C                ARE FOR THE SOLUTION FOR THE FIRST RIGHT HAND
C                SIDE, THE NEXT N+1 TO 2N ENTRIES ARE FOR THE
C                SOLUTION TO THE SECOND RIGHT HAND SIDE, ECT.
C       
C       SUM      REAL   (NRHS)
C                A WORKING ARRAY TO BE USED IN THE BACK 
C                SUBSTITUTION PROCESS OF NAIVE GAUSSIAN
C                ELIMINATION.
C       
C**********************************************************
C     
C     VARIABLE DECLATIONS.
C     
      REAL A(1),B(1),X(1),SUM(1),WGT(1),PIV
      INTEGER NDG(1),N,K,I
C     
C     CONDUCT THE FORWARD DECOMPOSITION PROCESS ON THE
C     MULTIPLE RIGHT HAND SIDE VECTORS.
C     
      DO 30 K = 1,N-1
        DO 20 I = K+1,N
          PIV = A(NDG(I)+K)/A(NDG(K)+K)
          DO 10 J = 1,NRHS
            IF(WGT(J).NE.0.0)THEN
              B((I+(J-1)*N)) = B((I+(J-1)*N))
     1                      - PIV*B((K+(J-1)*N))
            ENDIF
 10       CONTINUE
 20     CONTINUE
 30   CONTINUE
C     
C     NOW CONDUCT THE BACK SUBSTITUTION PHASE ON THE
C     MULTIPLE RIGHT HAND SIDED VECTORS TO OBTAIN THE
C     MULTIPLE SOLUTIONS TO THE SYSTEM OF EQUATIONS.
C     
      DO 40 J = 1,NRHS
        IF(WGT(J).NE.0.0)THEN
          X(N*J) = B(N*J)/A(NDG(N)+N)
        ENDIF
 40   CONTINUE
      DO 90 K = N-1,1,-1
        DO 50 I = 1,NRHS
          IF(WGT(I).NE.0.0)THEN
            SUM(I) = 0.0
          ENDIF
 50     CONTINUE
        DO 70 J = K+1,N
          DO 60 I = 1,NRHS
            IF(WGT(I).NE.0.0)THEN
              SUM(I) = SUM(I) + A(NDG(J)+K)*X((J+(I-1)*N))
            ENDIF
 60       CONTINUE
 70     CONTINUE
        DO 80 I = 1,NRHS
          IF(WGT(I).NE.0.0)THEN
            X((K+(I-1)*N)) = (B((K+(I-1)*N)) - SUM(I))/A(NDG(K)+K)
          ENDIF
 80     CONTINUE     
 90   CONTINUE
C     
      RETURN
      END
C*********************************************************
C     
      SUBROUTINE IDSORT(N,A,INDEX)
C     
C     SUBROUTINE IDSORT SORTS AN INPUT ARRAY "A" OF ORDER
C     N ELEMENTS VIA THE USEAGE OF AN INDEX ARRAY "INDEX"
C     UTILIZING THE HEAPSORT SORTING ALGORITHM OF KNUTH,
C     DONALD E., 1968, FUNDAMENTAL ALGORITHMS, VOL. 1 OF
C     THE ART OF COMPUTER PROGRAMMING, S2.3.3
C     
C     INPUT VARIABLES
C     
C     N         SIZE OF INPUT ARRAY "A".
C     A         INPUT ARRAY TO BE SORTED.
C     
C     OUTPUT VARIABLES.
C     
C     INDEX     INDEX ARRAY THE CONTAINS THE INDEXES OF
C               SORTED ELEMENTS OF INPUT ARRAY "A", IN
C               ASCENDING ORDER.
C     
C     NOTE:  IDSORT DOES NOT CHANGE THE ORDER OF THE 
C            ELEMENTS OF INPUT ARRAY "A" UPON RETURN.
C     
C**********************************************************
C     
C     VARIABLE DECLARATIONS.
C     
      DIMENSION A(1),INDEX(1)
C     
C     INITIALIZE THE INDEX ARRAY.
C     
      DO 5 J = 1,N
        INDEX(J) = J
 5    CONTINUE
C     
C     NOW CONDUCT THE HEAPSORT WITH INDIRECT INDEXING.
C     
      L  = N/2 + 1
      IR = N
 10   CONTINUE
        IF(L.GT.1)THEN
          L = L -1
          INDXT = INDEX(L)
          Q = A(INDXT)
        ELSE
          INDXT = INDEX(IR)
          Q = A(INDXT)
          INDEX(IR) = INDEX(1)
          IR = IR - 1
          IF(IR.EQ.1)THEN
            INDEX(1) = INDXT
            GO TO 30
          ENDIF
        ENDIF
        I = L
        J = L + L
 20     IF(J.LE.IR)THEN
          IF(J.LT.IR)THEN
            IF(A(INDEX(J)).LT.A(INDEX(J+1)))THEN
              J = J + 1
            ENDIF
          ENDIF
          IF(Q.LT.A(INDEX(J)))THEN
            INDEX(I) = INDEX(J)
            I = J
            J = J + J
          ELSE
            J = IR + 1
          ENDIF
          GO TO 20
        ENDIF
        INDEX(I) = INDXT
      GO TO 10
C     
 30   RETURN
      END
C**********************************************************
C     
      SUBROUTINE CVTYPE(N,MODEL)
C     
C     Subroutine CVTYPE returns the proper variogram
C     model name on input of its model parameters.
C     
C     Input parameters.
C     
C     N        Model type code.
C     
C     Output parameters.
C     
C     MODEL    Model type name.
C     
C********************************************************** 
C     
C     Variable declarations.
C     
      CHARACTER*30 MODEL
      INTEGER N
C     
      IF(N.EQ.1)THEN
        MODEL = 'Nugget-effect model'
      ELSEIF(N.EQ.2)THEN
        MODEL = 'Spherical model'
      ELSEIF(N.EQ.3)THEN
        MODEL = 'Exponential model'
      ELSEIF(N.EQ.4)THEN
        MODEL = 'Gaussian model'
      ELSEIF(N.EQ.5)THEN
        MODEL = 'Cubic model'
      ELSEIF(N.EQ.6)THEN
        MODEL = 'Quadratic model'
      ELSEIF(N.EQ.7)THEN
        MODEL = 'Hole-effect model'
      ELSEIF(N.EQ.8)THEN
        MODEL = 'General multiquadratic model'
      ELSEIF(N.EQ.9)THEN
        MODEL = 'Uniform mean cosine model'
      ELSEIF(N.EQ.10)THEN
        MODEL = 'Uniform mean spherical model'
      ELSEIF(N.EQ.11)THEN
        MODEL = 'Uniform mean gaussian model'
      ELSEIF(N.EQ.12)THEN
        MODEL = 'Uniform mean power model'
      ELSEIF(N.EQ.13)THEN
        MODEL = 'Linear mean power model'
      ELSEIF(N.EQ.14)THEN
        MODEL = 'Exponential mean power model'
      ELSEIF(N.EQ.15)THEN
        MODEL = 'Power model'
      ELSEIF(N.EQ.16)THEN
        MODEL = 'Gamma mean gaussian model'
      ELSEIF(N.EQ.17)THEN
        MODEL = 'De-Wijsian model'
      ENDIF
C     
      RETURN
      END
C**********************************************************
C     
      SUBROUTINE LEFT(IERROR)
C     
C     Subroutine LEFT sequentially calls the necessary
C     subroutines to conduct the left hand side portion
C     of cokriging via a partitioned matrix approach.
C     
C     Note:  If successful, IERROR returned as 0,
C                     else, IERROR returned as >0.
C            If IERROR > 0, then the columns in
C            the matrix B are not independent.
C     
C**********************************************************
C     
C     Variable declarations.
C     
            include'cokrig.inc'
C*****Start by generating covariance matrix.
      DO 10 J = 1,NSAMP
        DO 5 I = 1,J
          K = NDG(J) + I
          IF(NATR(I).EQ.NATR(J))THEN
C***********Covariance between points.
            L = NDG(NATR(I)) + NATR(J)
            A(K) = SILL(L) - GAMMA(X(NHOL(I)),Y(NHOL(I)),
     1        Z(NHOL(I)),X(NHOL(J)),Y(NHOL(J)),Z(NHOL(J)),L)
          ELSE
C***********Cross-covariance between points.
            A(K) = CROSS(X(NHOL(I)),Y(NHOL(I)),Z(NHOL(I)),
     1        X(NHOL(J)),Y(NHOL(J)),Z(NHOL(J)),NATR(I),NATR(J))
          ENDIF
  5     CONTINUE
 10   CONTINUE
C*****Debug output if desired.
      IF(IDBUG.EQ.1)THEN
        WRITE(IOUT,25)
 25     FORMAT(//,' Echo of packed covariance matrix.',//)
        DO 35 I = 1,NSAMP
          WRITE(IOUT,30) (A(NDG(I)+J),J=1,I)
 30       FORMAT(1X,8E10.4)
 35     CONTINUE
      ENDIF
C*****Factor A into L U, not checking for zero pivot.
      NTEST = 0
      CALL UPPER(A,NDG,NSAMP,NTEST,ZERO,IERROR)
C*****Generate basis matrix partition of LHS cokriging 
C*****system and solve for that part of the system.
      N = 0
      NXCOL = 0
      DO 55 J = 1,MVAR
        NTERMP(J) = NXCOL
        DO 50 K = 1,NTERM(J)
          NXCOL = NXCOL + 1
          NP = NTERMP(J) + K
          DO 49 I = 1,NSAMP
            N = N + 1
            IF(NATR(I).EQ.J)THEN
              B(N)  = TERM(X(NHOL(I)),Y(NHOL(I)),Z(NHOL(I)),
     1                     EX(NP),EY(NP),EZ(NP))
              BB(N) = B(N)
            ELSE
              B(N)  = 0.0
              BB(N) = 0.0
            ENDIF
 49       CONTINUE
 50     CONTINUE
 55   CONTINUE
C*****Debug output if desired.
      IF(IDBUG.EQ.1)THEN
        WRITE(IOUT,60)
 60     FORMAT(//,' Echo of cokriging design matrix.',//)
        DO 65 I = 1,NSAMP
          WRITE(IOUT,30) (B(I+(J-1)*NSAMP),J=1,NXCOL)
 65     CONTINUE
      ENDIF
C*****Compute solution to [K] [XL] = [B].
      DO 70 J = 1,NXCOL
        XLAG(J) = 1.0
 70   CONTINUE
      CALL SOLVE(A,NDG,NSAMP,BB,XL,NXCOL,SUM,XLAG)
C*****Debug output if desired.     
      IF(IDBUG.EQ.1)THEN
        WRITE(IOUT,75) 
 75     FORMAT(//,' Echo solution to cokriging design matrix.',//)
        DO 90 I = 1,NSAMP
          WRITE(IOUT,30) (XL(I+(J-1)*NSAMP),J=1,NXCOL)
 90     CONTINUE
      ENDIF
C                      T  -1
C*****Now generate  [X] [A] [X] MATRIX.
      IN = 0
      DO 120 K = 1,NXCOL
        DO 115 I = 1,K
          IN = IN + 1
          DD(IN) = 0.0
          DO 110 J = 1,NSAMP
            DD(IN) =  DD(IN) + B(J+(K-1)*NSAMP)*XL(J+(I-1)*NSAMP)
 110      CONTINUE
 115    CONTINUE
 120  CONTINUE
C*****Debug output if desired.
      IF(IDBUG.EQ.1)THEN
        WRITE(IOUT,125)
 125    FORMAT(//,
     1  ' Echo of X transpose * Sigma Inverse * X matrix.',//)
        DO 130 I = 1,NXCOL
          WRITE(IOUT,30) (DD(NDG(I)+J),J=1,I)
 130    CONTINUE
      ENDIF
C               t   -1           
C*****Factor [X]  [K]  [X] into L U, checking for zero pivot.
      IF(NXCOL.GT.1)THEN
        NTEST = 1
        CALL UPPER(DD,NDG,NXCOL,NTEST,ZERO,IERROR)
      ENDIF
C*****Done with left portion, so return.
 200  RETURN
      END
C**********************************************************
C     
      SUBROUTINE RIGHT(TCOKVA,XCB,YCB,ZCB)
C     
C     Subroutine RIGHT computes the vector of cokriging
C     estimators, the cokriging variances, and the total
C     cokriging variance for a point located at the point
C     XCB, YCB, ZCB or for a block whose south-west
C     corner's crest is located at these coordinates.
C     Utilizes previous matrix factorizations computed
C     by subroutine LEFT.
C     
C**********************************************************
C     
C     Variable declarations.
C     
            include'cokrig.inc'
C*******************************************************
C*****Conduct the RHS part of the decomposition.********
C*******************************************************
      TCOKVA = 0.0
      DO 200 JVAR = 1,MVAR
C*******Generate the RHS vector associated with variable JVAR.
        IF(WGT(JVAR).EQ.0.0)THEN
          EST(JVAR)    = 0.0
          COKVAR(JVAR) = 0.0
          GO TO 200
        ENDIF
        DO 10 I = 1,NSAMP
          IF(NATR(I).EQ.JVAR)THEN
            L = NDG(JVAR) + JVAR
            IF(IPKRIG.EQ.0)THEN
C*************Point kriging covariance.
              COV(I) = SILL(L) - GAMMA(X(NHOL(I)),Y(NHOL(I)),
     1                  Z(NHOL(I)),XCB,YCB,ZCB,L)
            ELSE
C*************Block kriging average covariance.
              COV(I) = SILL(L) - GAMAVG(X(NHOL(I)),Y(NHOL(I)),
     1                  Z(NHOL(I)),XCB,YCB,ZCB,L)
            ENDIF
          ELSE
            IF(IPKRIG.EQ.0)THEN
C*************Point kriging cross-covariance.
              COV(I) = CROSS(X(NHOL(I)),Y(NHOL(I)),Z(NHOL(I)),
     1                  XCB,YCB,ZCB,NATR(I),JVAR)
            ELSE
C*************Block kriging average cross-covariance.
              COV(I) = CRSAVG(X(NHOL(I)),Y(NHOL(I)),Z(NHOL(I)),
     1                  XCB,YCB,ZCB,NATR(I),JVAR)
            ENDIF
          ENDIF
          BB(I) = COV(I)
 10     CONTINUE
C*******Find solutions to rhs covariances.
        NRHS = 1
        XLAG(1) = 1.0
        CALL SOLVE(A,NDG,NSAMP,BB,XR,NRHS,SUM,XLAG)
C*******Echo solution if desired.
        IF(IDBUG.EQ.1)THEN
          WRITE(IOUT,40) JVAR
 40       FORMAT(//,' Echo of cov. rhs and solution to rhs.',I5,//)
          DO 50 I = 1,NSAMP
            WRITE(IOUT,45) COV(I),XR(I)
 45         FORMAT(1X,8E10.4)
 50       CONTINUE
        ENDIF
C*******Generate solution to lagrange column.
C                            T   -1
C*******Generate  {BBAR} - [X] [K] {SIGBAR}.
        DO 70 J = 1,MVAR
          DO 65 K = 1,NTERM(J)
C***********{BBAR} portion.
            NP = NTERMP(J) + K
            IF(JVAR.EQ.J)THEN
              IF(IPKRIG.EQ.0)THEN
                G(NP) = TERM(XCB,YCB,ZCB,EX(NP),EY(NP),EZ(NP))
              ELSE
                G(NP) = TERMAV(XCB,YCB,ZCB,EX(NP),EY(NP),EZ(NP))
              ENDIF
            ELSE
              G(NP) = 0.0
            ENDIF
            BBAR(NP) = G(NP)
C             t   -1
C***********[X] [K] {sigbar} portion.
            DO 60 I = 1,NSAMP
              G(NP) = G(NP) - B(I+(NP-1)*NSAMP)*XR(I)
 60         CONTINUE
            GG(NP) = G(NP)
 65       CONTINUE
 70     CONTINUE
C*******Calculate lag. multiplier solution columns.
        IF(NXCOL.EQ.1)THEN
          XLAG(1) = G(1)/DD(1)
        ELSE
          NRHS = 1
          TEMP = BBAR(1)
          BBAR(1) = 1.0
          CALL SOLVE(DD,NDG,NXCOL,GG,XLAG,NRHS,SUM,BBAR)
          BBAR(1) = TEMP
        ENDIF
C*******Echo solution if desired.
        IF(IDBUG.EQ.1)THEN
          WRITE(IOUT,75) JVAR
 75       FORMAT(//,
     1    ' Echo of lagrangean rhs and lagrangean solution for',I5,//)
          DO 80 I = 1,NXCOL
            WRITE(IOUT,45) G(I),XLAG(I)
 80       CONTINUE
        ENDIF
C*******Complete partitioned system solution.
C*******Start with solving for the cokriging weights.
        DO 90 I = 1,NSAMP
          BB(I) = XR(I)
          DO 85 J = 1,NXCOL
            BB(I) = BB(I) + XL(I+(J-1)*NSAMP)*XLAG(J)
 85       CONTINUE
 90     CONTINUE
C*******Debug output if desired.
        IF(IDBUG.EQ.1)THEN
          WRITE(IOUT,95) JVAR
 95       FORMAT(//,
     1    ' Echo of kriging wts and grades for estimating ',I5,//)
          DO 100 I = 1,NSAMP
            LOC = NATR(I) + (NHOL(I)-1)*MVAR
            WRITE(IOUT,45) BB(I),GRADE(LOC)
 100      CONTINUE
          WRITE(IOUT,105)
 105      FORMAT(1X)
        ENDIF
C*******Calculate estimator and variance.
        EST(JVAR) = 0.0
        AG = 0.0
        DO 110 I = 1,NSAMP
          LOC = NATR(I)  + (NHOL(I)-1)*MVAR
          EST(JVAR) = EST(JVAR) + BB(I)*GRADE(LOC)
          AG = AG + BB(I)*COV(I)
 110    CONTINUE
        BU = 0.0
        DO 115 I = 1,NXCOL
          BU = BU + BBAR(I)*XLAG(I)
 115    CONTINUE
        IF(IPKRIG.EQ.0)THEN
C*********Point kriging variance.
          COKVAR(JVAR) = SILL(NDG(JVAR)+JVAR) - AG + BU
          COKVAR(JVAR) = SQRT(ABS(COKVAR(JVAR)))
        ELSE
C*********Block kriging variance.
          COKVAR(JVAR) = SILL(NDG(JVAR)+JVAR) - CBKVAR(JVAR) - AG + BU
          COKVAR(JVAR) = SQRT(ABS(COKVAR(JVAR)))
        ENDIF
        IF(COKVAR(JVAR).LT.0.0) COKVAR(JVAR) = 0.0
C*******Compute contribution to total cokriging variance.
        TCOKVA = TCOKVA + WGT(JVAR)*COKVAR(JVAR)
 200  CONTINUE
C*****Finished cokriging so return.
      RETURN
      END
C**********************************************************
C     
      SUBROUTINE DUAL(TCOKVA,XCB,YCB,ZCB)
C     
C     Subroutine DUAL computes the vector of dual cokriging
C     estimators and the dual objective function value for
C     a point located at the point XCB, YCB, ZCB or for a 
C     block whose south-west corner's crest is located at
C     these coordinates.  Utilizes the previous matrix 
C     factorizations computed by subroutine LEFT.
C     
C**********************************************************
C     
C     Variable declarations.
C     
            include'cokrig.inc'
C*******************************************************
C*****Conduct the RHS part of the decomposition.********
C*******************************************************
      TCOKVA = 0.0
C*****Generate the subset vector of observations in kriging nbhd.
      DO 10 I = 1,NSAMP
        LOC = NATR(I) + (NHOL(I)-1)*MVAR
        BB(I) = GRADE(LOC)
 10   CONTINUE
C*****Find solution to rhs observations.
      NRHS = 1
      XLAG(1) = 1.0
      CALL SOLVE(A,NDG,NSAMP,BB,XR,NRHS,SUM,XLAG)
C*****Echo solution if desired.
      IF(IDBUG.EQ.1)THEN
        WRITE(IOUT,20)
 20     FORMAT(//,' Echo of observation rhs and its solution.'//)
        DO 50 I = 1,NSAMP
          LOC = NATR(I) + (NHOL(I)-1)*MVAR
          WRITE(IOUT,45) GRADE(LOC),XR(I)
 45       FORMAT(1X,8E10.4)
 50     CONTINUE
      ENDIF
C                T   -1
C*****Generate [X] [K] {Y}.
      DO 70 J = 1,MVAR
        DO 65 K = 1,NTERM(J)
          NP = NTERMP(J) + K
          G(NP) = 0.0
          DO 60 I = 1,NSAMP
            G(NP) = G(NP) + B(I+(NP-1)*NSAMP)*XR(I)
 60       CONTINUE
          GG(NP) = G(NP)
 65     CONTINUE
 70   CONTINUE
C*****Calculate dual mean basis function coefficients.
      IF(NXCOL.EQ.1)THEN
        XLAG(1) = G(1)/DD(1)
      ELSE
        NRHS = 1
        BBAR(1) = 1.0
        CALL SOLVE(DD,NDG,NXCOL,GG,XLAG,NRHS,SUM,BBAR)
      ENDIF
C*****Echo solution if desired.
      IF(IDBUG.EQ.1)THEN
        WRITE(IOUT,75)
 75     FORMAT(//,
     1  ' Echo of rhs and solution to mean function coefficients.'//)
        DO 80 I = 1,NXCOL
          WRITE(IOUT,45) G(I),XLAG(I)
 80     CONTINUE
      ENDIF
C*****Complete partitioned system solution.
C*****Start with solving for the dual cokriging weights.
      DO 90 I = 1,NSAMP
        BB(I) = XR(I)
        DO 85 J = 1,NXCOL
          BB(I) = BB(I) - XL(I+(J-1)*NSAMP)*XLAG(J)
 85     CONTINUE
 90   CONTINUE
C*****Debug output if desired.
      IF(IDBUG.EQ.1)THEN
        WRITE(IOUT,95)
 95     FORMAT(//,
     1  ' Echo of dual kriging wts',//)
        DO 100 I = 1,NSAMP
          WRITE(IOUT,45) BB(I)
 100    CONTINUE
        WRITE(IOUT,105)
 105    FORMAT(1X)
      ENDIF
C*****Calculate value of dual objective function.
      DOBJ = 0.0
      DO 110 I = 1,NSAMP
        LOC = NATR(I) + (NHOL(I)-1)*MVAR
        DOBJ = DOBJ + BB(I)*GRADE(LOC)
 110  CONTINUE
C*****Calculate dual estimators.
      DO 200 JVAR = 1,MVAR
        IF(WGT(JVAR).EQ.0.0)THEN
          EST(JVAR)    = 0.0
          COKVAR(JVAR) = 0.0
          GO TO 200
        ENDIF
C*******Portion of estimator due to residual.
        AG = 0.0
        DO 120 I = 1,NSAMP
          IF(NATR(I).EQ.JVAR)THEN
            L = NDG(JVAR) + JVAR
            IF(IPKRIG.EQ.0)THEN
C*************Point kriging covariance.
              COV(I) = SILL(L) - GAMMA(X(NHOL(I)),Y(NHOL(I)),
     1                  Z(NHOL(I)),XCB,YCB,ZCB,L)
            ELSE
C*************Block kriging average covariance.
              COV(I) = SILL(L) - GAMAVG(X(NHOL(I)),Y(NHOL(I)),
     1                  Z(NHOL(I)),XCB,YCB,ZCB,L)
            ENDIF
          ELSE
            IF(IPKRIG.EQ.0)THEN
C*************Point kriging cross-covariance.
              COV(I) = CROSS(X(NHOL(I)),Y(NHOL(I)),Z(NHOL(I)),
     1                  XCB,YCB,ZCB,NATR(I),JVAR)
            ELSE
C*************Block kriging average cross-covariance.
              COV(I) = CRSAVG(X(NHOL(I)),Y(NHOL(I)),Z(NHOL(I)),
     1                  XCB,YCB,ZCB,NATR(I),JVAR)
            ENDIF
          ENDIF
          AG = AG + COV(I)*BB(I)
 120    CONTINUE
C*******Portion of estimator due to the mean.
        BU = 0.0
        DO 140 J = 1,MVAR
          DO 130 K = 1,NTERM(J)
            NP = NTERMP(J) + K
            IF(JVAR.EQ.J)THEN
              IF(IPKRIG.EQ.0)THEN
                BBAR(NP) = TERM(XCB,YCB,ZCB,EX(NP),EY(NP),EZ(NP))
              ELSE
                BBAR(NP) = TERMAV(XCB,YCB,ZCB,EX(NP),EY(NP),EZ(NP))
              ENDIF
            ELSE
              BBAR(NP) = 0.0
            ENDIF
            BU = BU + BBAR(NP)*XLAG(NP)
 130      CONTINUE
 140    CONTINUE
        EST(JVAR) = AG + BU
        COKVAR(JVAR) = DOBJ
        TCOKVA = TCOKVA + WGT(JVAR)*COKVAR(JVAR)
 200  CONTINUE
C*****Finished dual cokriging so return.
      RETURN
      END
