C*******************************************************************
C  File CORRES.DEC  - Global variable declaration to be $INCLUDED
C                       in subroutines of program CORRES.
C
C  Description for common blocks and parameter values
C  MaxVar      - Maximum number of variables used in program
C  MaxFileVar  - Maximum number of variables in the data file
C  MaxSamples  - Maximum number of data records in the data file
C  MaxSuppVar  - Maximum number of supplementary variables 
C  MaxFactors  - Maximum number of factors used in program


cclong These are also given in anerr.for, so if they are to be 
cc changed then they must be changed there too!
        PARAMETER (MaxVar = 38)
        PARAMETER (MaxFileVar = 48)
        PARAMETER (MaxSamples = 10000)
        PARAMETER (MaxSuppVar = 10)
        PARAMETER (MaxFactors = 9)

C********************************************************************
C /FILES/     -  Common block for file names
C 
C FilePrefix     - File prefix for file names
C DataFile       - Data file name
c Outfile        - output file name
C SampFile       - Samples file name
C VarFile        - Variables file name
C ReconFile      - Reconstructed data file name
C ResidFile      - Residuals file name
C FileName       - File name including the file prefix

        COMMON /FILES/ DataFile, FilePrefix, FileName, Sampfile,
     +                 VarFile, ReconFile, ResidFile, outfile

	CHARACTER*14  DataFile, SampFile, VarFile, ReconFile, ResidFile
     &	              , outfile
        CHARACTER*50  FilePrefix
	CHARACTER*64  FileName
 
C*********************************************************************
C /DATFIL/     - Common block for the data 
C
C Header           - Title from the data file
C FileNVar         - Number of variables in the data file
C FileVarNam       - Array of variable names in data file
C FileUnits        - Array of measurement units for each variable
C FileFormats      - Array of format specifications for each variable
C NVar             - Number of variables selected
C VarNam           - Array of variable names selected
C PIJ              - Array of variable values
C NSuppVar         - Number of supplementary variables
C SVarNam          - Array of supplementary variable names selected
C PSIJ             - Array of supplementary variables values
C NSamples         - Number of samples

        COMMON /DATFIL/ Header, FileNVar, FileVarNam, FileUnits,
     +                  FileFormats, NVar, VarNam, PIJ, NSuppVar,
     +                  SVarNam, PSIJ, NSamples 

        CHARACTER*10  FileVarNam (MaxFileVar),  FileUnits (MaxVar), 
     +                FileFormats (MaxFileVar), 
     +                VarNam (MaxVar), SVarNam (MaxSuppVar)
        CHARACTER*80  Header
        INTEGER*4     NVar, FileNVar, NSamples, NSuppVar
        REAL*8        PIJ (MaxSamples,MaxVar), 
     +                PSIJ (MaxSamples,MaxSuppVar)

C*********************************************************************
C /EXESCR/     - Common block for Execute Screen
C
C Valpr      - Array of Non trival eigenvalues
C Vecpr      - Array of eigenvectors 
C Pourc      - Array to store the variation (percent) of the factors
C Cumm       - Array to store the cummulative variation of the factors

        COMMON /EXESCR/  Valpr, Vecpr, Pourc, Cumm
 
        REAL*8  Pourc (MaxVar), Cumm (MaxVar)
        REAL*8  Valpr (MaxVar), Vecpr (MaxVar, MaxVar)

C*********************************************************************
C /RSTSCR/     - Common block for the Results screen
C
C PEJ    - Array to store the error of reconstruction for variables
C PEI    - Array to store the error of reconstruction for samples
C CCRJ   - Array to store the error of representation for variables
C CCRI   - Array to store the error of representation for samples


        COMMON /RSTSCR/ PEJ, CCRJ, PEI, CCRI

        REAL*8  PEJ (MaxVar), CCRJ (MaxVar)
        REAL*8  PEI (MaxSamples), CCRI (MaxSamples)

C*********************************************************************
C /FACTS/     - Common block for the factors
C
C FACJ   - Array of J Factors (variables)
C FACI   - Array of I Factors (samples)

        COMMON /FACTS/ FACJ, FACI

        REAL*8  FACJ (MaxVar, MaxVar), FACI (MaxSamples, MaxVar) 
 
C*********************************************************************
C /COORDS/     - Common block for the coordinates 
C
C COORJ   - Array of variable coordinates
C COORI   - Array of sample coordinates
C COORJS  - Array of supplementray variable coordinates 

        COMMON  /COORDS/ COORJ, COORI, COORJS

        REAL*8  COORJ (MaxVar, MaxVar), COORI (MaxSamples, MaxVar)
        REAL*8  COORJS (MaxSuppVar, MaxFactors)

C*********************************************************************
C /ERRPRF/     - Common block for the error profiles
C
C PJ    - Array to store the sum over samples of PIJ's
C CAJ   - Array to store the absolute contributions for variables
C CRJ   - Array to store the relative contributions for variables
C PI    - Array to store the sum over columns of PIJ's
C CAI   - Array to store the absolute contributions for samples
C CRI   - Array to store the relative contributions for samples
C CRJS  - Array to store the relative contributions for
C         supplementary variables

        COMMON /ERRPRF/ PJ, CAJ, CRJ, PI, CAI, CRI, CRJS

        REAL*8  PJ (MaxVar), PI (MaxSamples) 
        REAL*8  CAJ (MaxVar, MaxVar), CRJ (MaxVar, MaxVar)
        REAL*8  CAI (MaxSamples, MaxVar), CRI (MaxSamples, MaxVar)
        REAL*8  CRJS (MaxSuppVar, MaxVar) 

C*********************************************************************
C /WORK/     - Common block for work variables
C
C NFactors   - Number of factors
C SJJ        - Cross product matrix
C SomIJ      - Sum of the array SumI
C RTable     - Reconstructed data table

       COMMON  /WORK/ SomIJ, SJJ, RTable, NFactors

       INTEGER*4  NFactors
       REAL*8     SomIJ, RTable (MaxSamples, MaxVar)
       REAL*8     SJJ (MaxVar, MaxVar)

C*********************************************************************
C /MISC/     - Common block for miscellaneous variables
C
C Intro      - Flag to suppress Intro Screen
C MenuChoice - Menu option selected
C Message    - Character string for error messages
C RFlag      - Flag to compute reconstructed and residual data

        COMMON /MISC/  Intro, RFlag, outfile_on, Message, MenuChoice

        CHARACTER*12  MenuChoice
        CHARACTER*75  Message
        LOGICAL       Intro, RFlag, outfile_on
