clarge
c**********************************************************************
c*** File name: PREP1.FOR (Source code for the program DATAPREP)
c**********************************************************************
                      PROGRAM DATA PREPARATION

c***  This program provides utilities for Geo-EAS data files.  Dataprep
c***  has two divisions, the DOS Utilities and the File Operations.
c***  The DOS Utilities allows access to commonly used DOS commands.
c***  The File Operations include utilities to manipulate Geo-EAS data
c***  files.
c***  The DOS Utilities include the commands Directory, Print, List(same
c***  as More), Copy, Rename, And Delete.  In addition the user can access
c***  DOS via the option 'DOS Command'.
c***  The File Operations include Append, Column Extract, Row Extract,
c***  Compress, ID Variable, Merge, Report, and Sort.
c***  The Dataprep File Operations use temporary files called Scratch
c***  Files(Temporary Files) for storing data read from a Geo-EAS data
c***  file and for processing.  The Append and Merge operations require
c***  an additional temporary file for processing.  The temporary files
c***  are ZZSCTCH1.FIL and ZZSCTCH2.FIL.  Each time an operation is 
c***  executed the temporary files are generated and then deleted after
c***  processing is complete.

c***  Although the program displays many different screens, only two
c***  screens are defined in the 'Dataprep.scr' file - an intro screen
c***  and a main screen(from which menus are displayed).  It is the
c***  main screen that changes to give the appearance of many screens.
c***  The code reflects the screen changes with 'Put Str' calls and 'Erase'
c***  calls.  These calls are placed at specific points in the code so
c***  that the screen changes accordingly.  Caution must therefore be used
c***  when moving or changing these calls.

c***  The main program performs the following:
c***	   Displays the intro screen.  Displays the main menu and 
c***       executes the user's requests by calling the appropriate 
c***       subroutine.
c***
c***  VARIABLES:
c***  Main Menu	  - Variable containing the screen field name of the
c***                main menu.
c***  Screen File - Variable containing the name of Dataprep's screen file
c***                which is DATAPREP.SCR.
c***  Menu Choice - Variable containing the user's menu option selection.
c***  Heading	  - Variable containing a screen label.
c***  Exit	  - Logical indicating if it is okay to exit the program.
c***  Get Yes No  - A function which displays a question and returns a
c***                .true. or .false. depending on the user's response to
c***                the displayed question.

       include 'dataprep.dec'
       include '../common/screen.str'

      CHARACTER   Main Menu*10, Screen File*12, Menu Choice*15
      CHARACTER   Heading*12, geoeas_home*80
      LOGICAL   Exit, Get Yes No

ccclong
      call get_geoeas_dir(10,geoeas_home)
      CALL Read Structure  (geoeas_home(1:lenstr(geoeas_home))
     &     //'/dataprep/dataprep.scr')
      call set up screen ()
ccclong

      Heading     = 'DESCRIPTION:'
c***  Screen field name of main menu.
      Main Menu   = 'ME MAIN   '
      Screen File = 'Dataprep.scr'
c***  Set global variable names to blanks.  Refer to the file called
c***  DATAPREP.DEC for explanantions of global variables.
      CALL Init Prefix
      CALL Init Tfprefix
      Infil       = '              '
      Outfil      = '              '
      Outfile1    = '              '

c***  Initialize global missing value.
      Missing = 1.E+31
c***  Read input file name and prefix (if any) and assign to 
c***  global variables.

      CALL Read System Defaults
c***  Display Intro Screen.
      IF (Intro) CALL Intro Screen
c***  Erase Intro Screen.
      CALL Clear Screen
c***  Display main screen along with frame and labels.
      CALL Set Screen(2)
      CALL Display Frame
      CALL Display Labels
c***  Display 'DESCRIPTION:' heading.
      CALL Put Str (Heading, 12, 5, 33, 0)
      CALL Display Prefix Labels
c***  Display the main menu.
      CALL Put Menu (Main Menu)
c***  The following if then else if statement is in a loop that 
c***  provides the main menu options and calls the necessary routines
c***  relating to the user's selection.
  10  CALL Get Menu Choice (Main Menu, Menu Choice)
      IF (Menu Choice .EQ. 'Quit           ') THEN
         Exit = Get Yes No 
     +	     (20, 2, 'Do you REALLY want to exit the program? <Y/N>')
         IF (Exit) THEN
           CALL Clear Screen
           CALL reset Screen
c***       Write the output file name and data file prefix (if any) to
c***       the designated file(which should be geoeas.def). 
	   CALL Write System Defaults
	   GOTO	20
         ENDIF
	 GOTO 10
      ELSE IF (Menu Choice .EQ. 'Prefix         ') THEN
	CALL Prefix Operation
        CALL Put Menu (Main Menu)
      ELSE IF (Menu Choice .EQ. 'UNIX Utilities ') THEN
c***    Erase 'DESCRIPTION:' heading.
	CALL Erase( 5, 33, 12)
	CALL DOS Utilities
	CALL Display Prefix Labels
        CALL Put Str (Heading, 12, 5, 33, 0)
        CALL Put Menu (Main Menu)
      ELSE IF (Menu Choice .EQ. 'File Operations') THEN
c***    Erase 'DESCRIPTION:' heading.
	CALL Erase( 5, 33, 12)
	CALL File Operations
	CALL Display Prefix Labels
        CALL Put Menu (Main Menu)
        CALL Put Str (Heading, 12, 5, 33, 0)
      ENDIF
      GOTO 10

  20  CONTINUE
      END
C**********************************************************************
C        D O S   U t i l i t i e s     (subroutine)
C**********************************************************************
      SUBROUTINE DOS Utilities

c***  This routine displays the DOS Utilities (vertical) menu and
c***  calls the necessary routines related to the user's menu 
c***  option selection.
c***
c***  VARIABLES:
c***  Menu Choice - Variable containing the user's option selection. 
c***  Menu        - Screen field name of menu.
c***  Label1	  - Variable containing a screen label.

       include 'dataprep.dec'
       include '../common/screen.str'

      CHARACTER   Menu Choice*15, Menu*10, Label1*15

      Label1 = 'UNIX UTILITIES:'
      Menu = 'DOS MENU  '

c***  Redraw the DOS Utilities screen and display the menu.
  10  CALL Display Prefix Labels
c***  Draw the vertical message divider.
      CALL Draw Message Divider
c***  Display 'DOS UTILITIES:' heading.
      CALL Put Str (Label1, 15, 7, 2, 0)
c***  Display the 'DOS UTILITIES' menu.
      CALL Put Menu (Menu)
 
c***  The following if then else if statement is in a loop that 
c***  provides the DOS UTILITIES menu options and calls the necessary 
c***  routines relating to the user's selection.
      CALL Get Menu Choice (Menu, Menu Choice)
      IF (Menu Choice      .EQ. 'Quit           ') THEN
c***     Erase DOS Utilities screen.
	 CALL Erase Message Divider
         CALL Clear Rows(1,18)
	 RETURN
      ELSE IF (Menu Choice .EQ. 'Copy           ') THEN
c***     Erase DOS Utilities screen.
	 CALL Erase Message Divider
         CALL ClearRows(1,18)
	 CALL Copy Prompts
      ELSE IF (Menu Choice .EQ. 'Delete         ') THEN
c***     Erase DOS Utilities screen.
	 CALL Erase Message Divider
         CALL ClearRows(1,18)
	 CALL Delete Prompts
      ELSE IF (Menu Choice .EQ. 'Directory      ') THEN
c***     Erase DOS Utilities screen.
	 CALL Erase Message Divider
         CALL ClearRows(1,18)
	 CALL Directory Prompts
       ELSE IF (Menu Choice .EQ. 'UNIX Command   ') THEN
c***     The screen including the frame is cleared for DOS 
c***     prompts and messages.
         CALL Clear Screen
	 CALL System Command
         CALL Clear Screen
c***     Redraw the DOS Utilities screen.
         CALL Set Screen(2)
         CALL Display Frame
         CALL Display Labels
      ELSE IF (Menu Choice .EQ. 'Print          ') THEN
c***     Erase DOS Utilities screen.
	 CALL Erase Message Divider
         CALL ClearRows(1,18)
	 CALL Print Prompts
      ELSE IF (Menu Choice .EQ. 'Rename         ') THEN
c***     Erase DOS Utilities screen.
	 CALL Erase Message Divider
         CALL ClearRows(1,18)
	 CALL Rename Prompts
      ELSE IF (Menu Choice .EQ. 'List           ') THEN
c***     Erase DOS Utilities screen.
	 CALL Erase Message Divider
         CALL ClearRows(1,18)
	 CALL List Prompts
       ENDIF

      GOTO 10
      RETURN
      END
C**********************************************************************
C	F i l e   O p e r a t i o n s         (subroutine)
C**********************************************************************
      SUBROUTINE File Operations

c***  This routine displays the File Operations (vertical) menu and
c***  calls the necessary routines related to the user's menu 
c***  option selection.
c***
c***  VARIABLES:
c***  Menu Choice - Variable containing the user's option selection. 
c***  Menu        - Screen field name of menu.
c***  Label1	  - Variable containing a screen label.

       include 'dataprep.dec'
       include '../common/screen.str'

      CHARACTER    Menu Choice*15, Menu*10, Label1*13

      Label1 = 'FILE OPTIONS:'
      Menu = 'FILE MENU '
  10  CALL Display Prefix Labels
c***  Draw the vertical message divider.
      CALL Draw Message Divider
c***  Display 'FILE OPERATIONS:' heading.
      CALL Put Str (Label1, 13, 7, 2, 0)
c***  Display the 'FILE OPERATIONS' menu.
      CALL Put Menu (Menu)
c***  The following if then else if statement is in a loop that 
c***  provides the File Operations menu options and calls the necessary 
c***  routines relating to the user's selection.
      CALL Get Menu Choice (Menu, Menu Choice)
      IF (Menu Choice      .EQ. 'Append         ') THEN
	 CALL Append Prompts
c***     Erase the message line of any remaining messages.
         CALL Erase(20,  2, 76)
      ELSE IF (Menu Choice .EQ. 'Column Extract ') THEN
	 CALL Column Extract Prompts
c***     Erase the message line of any remaining messages.
         CALL Erase(20,  2, 76)
      ELSE IF (Menu Choice .EQ. 'Row Extract    ') THEN
	 CALL Row Extract Prompts
c***     Erase the message line of any remaining messages.
         CALL Erase(20,  2, 76)
      ELSE IF (Menu Choice .EQ. 'Compress       ') THEN
	 CALL Compress Prompts
c***     Erase the message line of any remaining messages.
         CALL Erase(20,  2, 76)
      ELSE IF (Menu Choice .EQ. 'ID Variable    ') THEN
	 CALL Create ID Prompts
c***     Erase the message line of any remaining messages.
         CALL Erase(20,  2, 76)
      ELSE IF (Menu Choice .EQ. 'Merge          ') THEN
	 CALL Merge Prompts
c***     Erase the message line of any remaining messages.
         CALL Erase(20,  2, 76)
      ELSE IF (Menu Choice .EQ. 'Report         ') THEN
	 CALL Report Prompts
c***     Erase the message line of any remaining messages.
         CALL Erase(20,  2, 76)
      ELSE IF (Menu Choice .EQ. 'Sort           ') THEN
         CALL Sort Prompts
c***     Erase the message line of any remaining messages.
         CALL Erase(20,  2, 76)
      ELSE IF (Menu Choice .EQ. 'Quit           ') THEN
c***     Erase the File Options screen.
	 CALL Clear Rows(1,18)
	 CALL Erase Message Divider
	 RETURN
      ENDIF
      GOTO 10

      RETURN
      END
c**********************************************************************
c        O p e n   S c r a t c h   F i l e s   (subroutine)
c**********************************************************************
      SUBROUTINE Open Scratch Files

c***  This routine opens two scratch files called,
c***       ZZSCTCH1.FIL  and
c***       ZZSCTCH2.FIL.
c***  The routine 'Build Prefix Scratch Filename' affixes the scratch file
c***  prefix (if any) to a scratch file name and assigns this new
c***  name to the global variable 'Prefile'.  The prefix is contained
c***  in the global variable 'Tfprefix' and does not appear in this 
c***  routine.
c***
c***  VARIABLE:
c***  File Name - contains a scratch file name.

       include 'dataprep.dec'

      CHARACTER*14 File Name

c***  Open scratch file to contain 'Data1' structure. 
      File Name = 'ZZSCTCH1.FIL  '
      CALL Build Prefix Scratch Filename(File Name)
ccc      OPEN (UNIT=1,FILE=Prefile,ACCESS='DIRECT',RECL=4,FORM='BINARY')
      OPEN (UNIT=1,FILE=Prefile,
     &     ACCESS='DIRECT',RECL=4,FORM='unformatted')

c***  Open scratch file to contain 'Data2' structure. 
      File Name = 'ZZSCTCH2.FIL  '
      CALL Build Prefix Scratch Filename(File Name)
ccc      OPEN (UNIT=2,FILE=Prefile,ACCESS='DIRECT',RECL=4,FORM='BINARY')
      OPEN (UNIT=2,FILE=Prefile,
     &     ACCESS='DIRECT',RECL=4,FORM='unformatted')
		
      RETURN
      END
C**********************************************************************
c        C l o s e   S c r a t c h   F i l e s   (subroutine)
C**********************************************************************
      SUBROUTINE Close Scratch Files

c***  This routine closes the scratch files called
c***       ZZSCTCH1.FIL  and
c***       ZZSCTCH2.FIL.

c*** Close scratch file that contains 'Data1' 'structure'. ***
      CLOSE(1, STATUS='DELETE')
c*** Close scratch file that contains 'Data2' 'structure'. ***
      CLOSE(2, STATUS='DELETE')
	
      RETURN
      END
C**********************************************************************



