clarge
c**********************************************************************
c*** File name: PREP10.FOR (Source code for the program DATAPREP)
c**********************************************************************
c        C o m p r e s s   P r o m p t s     (subroutine)
c**********************************************************************
      SUBROUTINE Compress Prompts

c***  This routine constructs the Compress screen and displays
c***  the Compress menu.  The user is prompted for a menu 
c***  selection which is then processed.  
c***
c***  VARIABLES:
c***  Menu        - contains the field name of the Compress menu. 
c***  Menu Choice - contains the user's menu selection.
c***  Infil       - global variable that contains the name of the Geo-EAS
c***                input data file.
c***  Outfil	  - global variable that contains the name of the Geo-EAS
c***                output data file.
c***  Error	  - logical variable set to .true. if a fatal error occurs 
c***                during processing.
c***                
c***  Yes	  - logical variable is always set to true.  'Yes' is used
c***                as a parameter for the 'Compress' subroutine but due
c***                to revisions in the Compress operations is set 
c***                permanently true. 
  
       include 'dataprep.dec'
       include '../common/screen.str'
		   
      CHARACTER   Menu Choice*20, Menu*10, Blank*14, File*8
      CHARACTER   Toggle Field*8,Readmess*17,Label3*17
      CHARACTER   Label5*22
      CHARACTER   Const Field*9, Label1*11,Label2*12,Label4*17
      INTEGER*4   FLD Index, I, J, K, L
      LOGICAL   Yes, Error

c***  Erase the File Operations screen.
      CALL Erase Message Divider
      CALL ClearRows(1,18)
      CALL Draw Middle Horizontal Line(4)
      CALL Draw Middle Horizontal Line(6)
      CALL Draw Middle Horizontal Line(9)
      Blank = '              '
      Menu  = 'CS MENU   '
      Error = .TRUE.

c***  Initialize and display labels upon the screen.
      Readmess = ' Processing data '
      Label1 = 'Input File:'
      Label2 = 'Output File:'
      Label3 = 'COMPRESS FUNCTION'
      Label4 = 'Data File Prefix:'
      Label5 = 'Input\Output Variables'
      CALL Put Str (Label1, 11,  5,  2, 0)
      CALL Put Str (Label2, 12,  5, 39, 0)
      CALL Put Str (Label3, 17,  2, 32, 0)
      CALL Put Str (Label4, 17,  3,  2, 0)
      CALL Put Str (Prefix, 55,  3, 20, 0)
      CALL Put Str (Label5, 22, 10, 28, 0)
c***  Initialize and display file names.
      CALL Set Character Value ('INFILE1 ', Infil)
      CALL Set Character Value ('OUTFILE1 ', Outfil)
cccc      CALL Display One Field('INFILE1 ')
cccc      CALL Display One Field('OUTFILE1')
c***  Display Compress menu and prompt user for choice ***
      CALL Put Menu (Menu)
  10  CALL Get Menu Choice (Menu, Menu Choice)
      IF (Menu Choice      .EQ. 'Quit                ') THEN
c***    Erase the Compress screen.
	CALL Erase Middle Horizontal Line(4)
	CALL Erase Middle Horizontal Line(6)
	CALL Erase Middle Horizontal Line(9)
	CALL Clear Rows(1,18)
c***    Erase the variables heading.
  	RETURN
      ELSE IF (Menu Choice .EQ. 'Files               ') THEN
	CALL Erase Variable List
c***    Prompt user for input file.
  	File = 'INFILE1 '
	CALL Get Name of File(File,Infil,Error,1)
c***    If error return to menu.
	IF (Error) GOTO 10
c***    Prompt user for outfile.
	File = 'OUTFILE1'
	CALL Get Name of File(File, Outfil, Error,2)
c***    If error return to menu.
	IF (Error) GOTO 10
        CALL File Set Up (Error, Nvar)
c***    Clear the message line of any text.
        CALL Erase(20,  1, 45)
c***    Set the global variable 'Recskip' to the record number at
c***    which sample data begins.
	Recskip = Nvar + 3
c***    If not error then display variables of input file.
	IF (.NOT. Error) CALL Display Variables(Nvar)
      ELSE IF (Menu Choice .EQ. 'Execute             ') THEN
c***    If error in reading file or input not specified then return 
c***    to the menu else call the subroutine Compress which eliminates
c***    duplicate records in the input file .
	IF (Error) THEN
	  CALL Beep
          CALL Flash (20, 2,
     +     'ERROR - filenames have not been entered...(Press any key)')
	  GOTO 10
	ELSE
          CALL Put Str (Readmess, 17, 20, 2, 2)
	  CALL Open Scratch Files
c***      Read in sample data from input file and store in Scratch File 1.
	  IF (.NOT. Error)
     +     CALL Read Numerical Data(1,Infil,Nvar,Nvar,Error,Recskip)
	  Yes = .FALSE.
	  IF (.NOT. Error) THEN
            CALL Put Str (Readmess, 17, 20, 2, 2)
     	    CALL Compress (Outfil,Yes)
	  ENDIF
	  CALL Close Scratch Files
	ENDIF
      ENDIF

      GOTO 10
      RETURN
      END
c**********************************************************************
c        C o m p r e s s	       (subroutine)
c**********************************************************************
      SUBROUTINE Compress (Outfile, Yes)

c*** This subroutine eliminates any duplicate records in an input file.
c*** the resulting data is then written to the file whose name is stored
c*** in the variable 'Outfile'.
c*** to use this subroutine the following requirements must be satisfied:
c***                
c*** 1.) the input file has been read in and the data structures
c***     containing the varnames, formats and related samples have
c***     been initialized with the input file data.
c*** 2.) the 'Outfile' name has been entered and is not blank.
c***
c***  VARIABLES:
c***  Tmpndata - temporary variable used to maintain the number of records
c***             of the file being compressed.
c***  Nvar     - global variable that contains the number of variables in 
c***             the file being compressed.
c***  Ndata    - global variable that contains the number of records 
c***             of first the input file and then the compressed file 
c***             at the time it is written to the output file. 

       include 'dataprep.dec'
       include '../common/screen.str'

      CHARACTER*14  Outfile
      INTEGER*4     K, J, I, Tmpndata, Count
      REAL*4        Value, Get Element
      LOGICAL     Yes, Equal

c***  Set temporary data variable to 'Ndata' as it will change during
c***  data processing in this subroutine.
      Tmpndata = Ndata
      I = 1
c***  For every record of data, determine if duplicate records exist.
  20  IF (I .LE. Tmpndata) THEN
c***    Compare current record with all succeeding records.
        Count = -1
c***    Increment row counter J by increasing 'Count' ***
  30	Count = Count + 1
        J = I + 1 + Count
c***    Check that next record to be processed does not exceed the number
c***    of existing rows.
	IF (J .LE. Tmpndata) THEN
	  Equal = .FALSE.
c***      If the current record equals another record then set equal 
c***      to true, else exist loop and continue with next record.
	  DO 35 K = 1, Nvar
	    IF (Get Element(1,K,J,Nvar) .EQ.
     +                 Get Element(1,K,I,Nvar)) THEN
	      Equal = .TRUE.
	    ELSE
	      Equal = .FALSE.
	      GOTO 30
	    ENDIF
  35      CONTINUE
c***      If a duplicate record exists then delete it by moving all 
c***      succeeding records up one line.
	  IF (Equal) THEN
c***        Decrement the row count to check for a duplicate record.
	    Count = Count - 1
	    DO 40 L = J, Tmpndata-1
	      DO 45 K = 1, Nvar
		Value = Get Element(1,K,L+1,Nvar)
		CALL Put Element(1,K,L,Nvar,Value)
  45	      CONTINUE
  40	    CONTINUE
	    Tmpndata = Tmpndata-1
	  ENDIF
c***      Go to next record and repeat above process.
	  GOTO 30
	ENDIF
c***    Increment counter to next row and repeat the above process.
	I = I + 1
	GOTO 20
      ENDIF
      Ndata = Tmpndata
c***  If yes=.true. then assign results to Data 2 structure else write
c***  data to specified output file.
      IF (Yes) THEN
        Nvar2 = Nvar
c***    Copy file 2 data over to file 2 data structures.
	DO 50 I = 1,Nvar2
	  Varnam2(I) = Varnam(I)
	  Units2(I)  = Units(I)
	  Formats2(I) = Formats(I)
  50    CONTINUE
	Ndata2 = Ndata
	DO 55 I = 1,Nvar2
	  DO 60 J = 1,Ndata2
	    Value = Get Element(1,I,J,Nvar)
	    CALL Put Element(2,I,J,Nvar,Value)
  60	  CONTINUE
  55	CONTINUE
      ELSE
c***    Set global variable 'Outfil' to name of output file. 
        Outfil = Outfile
c***    Write compressed data to the output file.

        CALL Write to Output File 
      ENDIF

      RETURN
      END
c**********************************************************************
c        Read System Defaults   (subroutine)
c**********************************************************************
      SUBROUTINE Read System Defaults

c***  This routine reads in a Prefix name, an Input File name and a
c***  the logical 'Intro' (if they exist) from a 'geoeas.def' file 
c***  (if it exists) and assigns these names to the corresponding 
c***  variables(character strings).
c***
c***  VARIABLES:
c***  Prefix - global variable that contains the name of the Geo-EAS
c***           data file prefix.
c***  Infil  - global variable that contains the name of the Geo-EAS
c***           input data file.
c***  String - variable that contains the character string 
c***           'INTRO=whatever'.

       include 'dataprep.dec'
       include '../common/screen.str'

      CHARACTER String*80

      OPEN (UNIT = 10, FILE = 'geoeas.def', STATUS = 'UNKNOWN',
     +		       ERR = 999)
      READ (10,'(A)',ERR = 999,End = 999) Prefix
      READ (10,'(A)',ERR = 999,End = 999) Infil
      READ (10,'(A)',ERR = 999,End = 999) String

c***  Intro flag embedded as INTRO=ON, INTRO=anything else.
c***  Intro is a global logical.
      Intro = ((String (7:8) .EQ. 'ON') .OR.
     +         (String (7:8) .EQ. 'on')) 
      CALL Set Character Value ('PREFIX    ', Prefix)    
      CALL Set Character Value ('INFILE    ', Infil)    
      CLOSE (10)
      RETURN

  999 CLOSE (10)
      Intro = .TRUE.
      RETURN
      END
C**********************************************************************
c        Write System Defaults   (subroutine)
C**********************************************************************
      SUBROUTINE Write System Defaults

c***  This routine writes out a Prefix name, Output File name 
c***  and the logical 'Intro' to the 'geoeas.def' file.

c***
c***  VARIABLES:
c***  Prefix - global variable that contains the name of the Geo-EAS
c***           data file prefix.
c***  Outfil - global variable that contains the name of the Geo-EAS
c***           output data file.

       include 'dataprep.dec'

      OPEN (UNIT = 10, FILE = 'geoeas.def', STATUS = 'UNKNOWN',
     +		       ERR = 999)
      WRITE (10,'(A)',ERR = 999) Prefix
      WRITE (10,'(A)',ERR = 999) Outfil
      IF (Intro) THEN
        WRITE (10,'(A)',ERR = 999) 'INTRO=ON'
      ELSE
        WRITE (10,'(A)',ERR = 999) 'INTRO=OFF'
      ENDIF

  999 CLOSE (10)
      RETURN
      END
c**********************************************************************
c	 I n t r o   S c r e e n                (subroutine)
C**********************************************************************
      SUBROUTINE Intro Screen

c***  This routine displays the Introduction screen and prompts the
c***  user to press any key when finished reading the Intro.

      CALL Clear Screen
      CALL Set Screen(1)
      CALL Display Frame
      CALL Display Labels
      CALL Flash (20,2,'Press any key to begin the program ')
      RETURN
      END
c**********************************************************************




