clarge
c**********************************************************************
c*** File name: PREP4.FOR (Source code for the program DATAPREP)
c**********************************************************************
c	 D i s p l a y  V a r i a b l e s   (subroutine)
c**********************************************************************
      SUBROUTINE Display Variables (Count)

c*** This subroutine displays the variable names up to six in a row.
c***
c*** VARIABLES:
c*** Count  - Number of variable names to be displayed.
c*** Varnam - Global character array containing the variable names.
c*** Row    - Row location to display variable name.
c*** Col    - Column location to displaye variable name.

       include 'dataprep.dec'
       include '../common/screen.str'

      Integer*4 I, J, Row, Col, Count

      Row = 11
      Col = 2
c***  The do loop iterates one time for each variable name displayed.
      DO 10 I = 1, Count
c***    Determine if variable name is at end of row, if so then increment
c***    to next row, first variable name position.
	IF ( MOD(I,6) .EQ. 0) THEN
     	  CALL Put Str(Varnam(I), 10, Row, Col, 0)
	  Row = Row + 1
	  Col = 2
        ELSE
c***      If not end of row position then place in current position.
	  CALL Put Str(Varnam(I), 10, Row, Col, 0)
c***      Increment to next column position.
	  Col = Col + 12
        ENDIF
  10  CONTINUE

      RETURN
      END
c**********************************************************************
c	 D i s p l a y   C h o s e n   V a r i a b l e s  (subroutine)
c**********************************************************************
      SUBROUTINE Display Chosen Variables (Count)

c*** This subroutine displays the variable names indicated by the
c*** array Startcol.  Up to six names in a row are displayed.
c*** 
c*** VARIABLES:
c*** Count    - Number of variable names to be displayed.
c*** Varnam   - Global character array containing the variable names.
c*** Startcol -	Global array that contains the index of the Varnam array
c***            to be displayed on the screen.
c*** Row      - Row location to display variable name.
c*** Col      - Column location to displaye variable name.

       include 'dataprep.dec'
       include '../common/screen.str'

      Integer*4 I, J, Row, Col, Count

      Row = 11
      Col = 2
c***  The do loop iterates one time for each variable name displayed.
      DO 10 I = 1, Count
c***    Determine if variable name is at end of row, if so then increment
c***    to next row, first variable name position.
	IF ( MOD(I,6) .EQ. 0) THEN
     	  CALL Put Str(Varnam(Startcol(I)), 10, Row, Col, 0)
	  Row = Row + 1
	  Col = 2
        ELSE
c***      If not end of row position then place in current position.
	  CALL Put Str(Varnam(Startcol(I)), 10, Row, Col, 0)
c***      Increment to next column position.
	  Col = Col + 12
        ENDIF
  10  CONTINUE

      RETURN
      END
c**********************************************************************
c	 G e t   E l e m e n t       (function)
c**********************************************************************
      REAL*4 FUNCTION Get Element(Data Unit, Col, Index, Varnum)
 
c*** This routine reads a record from a scratch file and extracts the
c*** requested variable designated by 'Col' and assigns to Get Element.
c***
c*** VARIABLES:
c*** Data Unit - Scratch file number ,ie., 1 for infile1 data, or
c***             2 for infile2 data. 
c*** Col       - variable requested from infile.
c*** Index     - data item (sample data record) requested of 'Col'.
c*** Varnum    - number of variables.

      REAL*4      Value
      INTEGER*4   Varnum, Data Unit, J, Index, I, Col

c***  Extract sample indicated by 'J' from the specified scratch file.
      J = Col + ((Index - 1) * Varnum) 
      READ (Data Unit, REC = J) Value
c***  Assign this sample to 'Get Element'.
      Get Element = Value

      RETURN
      END

c**********************************************************************
c	 G e t  H e a d e r  D a t a             (subroutine)
c**********************************************************************
      SUBROUTINE Get Header Data (File Name, Error, Nvar temp)
 
c***  This routine reads the following data from a Geo-EAS data input 
c***  file:
c***       1.) Title
c***       2.) Number of Variables in data file
c***       3.) Variable names
c***  If an error occurs opening or reading the file then an error
c***  message is displayed.
c***
c***  VARIABLES:
c***  File Name - Name of Geo-EAS data file from which to extract data.
c***  Error	- Returns .true. if error occurs in opening or reading data.
c***  Nvar temp	- Contains number of variables in file.
c***  Prefile	- Global character string that contains the prefix + file
c***              name after call to Build Prefix Filename routine.
c***  MAXVAR	- Global variable containing maximum number of variables
c***              permitted in Dataprep.
c***  Varnam	- Global character array containing variable names.
c***  Units	- Global character array containing measurement descriptions
c***              of each variable.
c***  Formats	- Global character array containing output formats of each
c***              variable.


       include 'dataprep.dec'
       include '../common/screen.str'

      CHARACTER  File Name*(*)
      CHARACTER  Ch*1, Header*80, Readmess*16
      INTEGER*4	 J, I, Nvar temp
      LOGICAL  Error

      Error   = .False.
      Readmess = 'Reading Data ...'

c***  Concatenate prefix to filename ***
      CALL Build Prefix Filename ( File Name)
      CALL Put Str (Readmess, 16, 20, 2, 2)
      OPEN (UNIT = 10, FILE = Prefile, Err = 999)
	
c***  Read description of data file.
      READ (10,'(A)',ERR=999) Header
      Title (1:66) =  Header

c***  Get number of variables in file
      READ (10, *, Err = 999) Nvar temp
c***  Check that (1 <= number of variables <= MAXVAR) else display 
c***  error message. 
      IF (Nvar temp .LT. 1  .OR.  Nvar temp .GT. MAXVAR) GOTO 999

c***  Set format array 'Formats' to blanks

      DO 15 I = 1, Nvar temp
	 Formats(I) = '          '
  15  CONTINUE

c***  Get variable name, measurements description and output format
c***  for each variable in file.

      DO 20 I = 1, Nvar temp
         READ (10, 11, Err = 999) Varnam(I), Units(I), Formats(I)
  11	 FORMAT(2A10,A5)
c***     Check that output format is in acceptable range else assign
c***     a default output format for that variable in question.
	 CALL Input Format Check(I)
  20  CONTINUE
c***  Close input data file.
      CLOSE (10)
      RETURN

c***  Error has been found in data of file - try again

 999  CALL Beep
      CLOSE (10)
cccc      CALL Erase (20, 1, 78)
      CALL Flash (20, 2, 
     +     'Error reading data file...(press any key)')
      Error = .TRUE.

      RETURN
      END
cc**********************************************************************
ccc        G e t  F o r m a t                 (subroutine)
cc**********************************************************************
c      SUBROUTINE Get Format	
c
cc***  Prompt user for the output format width and decimal size.
cc***  If input format is illegal then issue message and use
cc***  G15.7 as the default.
cc***  
cc***  VARIABLES:
cc***  Choice	- contains the name of the screen field to be accessed. 
cc***  Fwidth	- contains the width of the output field.
cc***  Fdecimals	- contains the number of decimals to be in the output field.
cc***  W1	- temporary variable to contain the Fwidth value.
cc***  D1	- temporary variable to contain the Fdecimals value.
c c
c
c       include '../common/screen.str'
c
c      CHARACTER*10 Choice
c      INTEGER*4    W1, D1, Fwidth, Fdecimals, FLD Index
c
cc***  Prompt user for format width.
c      Choice = 'FWIDTH    '
c      CALL Set Integer Value (Choice, 0)
c      Next Field = FLD Index (Choice)
c      CALL Get Field
c      CALL Get Integer Value (Choice, Fwidth)
cc***  Prompt user for number of decimal positions.
c      Choice = 'FDECIMALS '
c      CALL Set Integer Value (Choice, 0)
c      Next Field = FLD Index (Choice)
c      CALL Get Field
c      CALL Get Integer Value (Choice, Fdecimals)
cc***  Assign the values to the temporary variables.
c      W1 = Fwidth
c      D1 = Fdecimals
cc***  Determine if format entered by user is within legal range.
cc***  If not then issue warning and use G15.7 by default,
cc***  else use the format specified by the user.
c      IF ((W1 .GT. 15).OR.(D1 .GT. 7).OR.(W1 .LT. (D1+1)).OR.
c     +    (W1 .LT. 3).OR.(D1 .LT. 0)) THEN
c	CALL Flash(20, 2, 
c     +        'Illegal format, G15.7 used by default (press any key)')
c	Fwidth = 15
c	Fdecimals = 7
c      ENDIF
c      RETURN
c      END
c**********************************************************************


