cc$LARGE
C PlotNeig.for
C-------------------------------------------------------------------
C Plot Neighbors - graph of neighborhood for program KRIGE
C
        SUBROUTINE Plot Neighbors (X0, Y0, Neighbors, N)
	IMPLICIT INTEGER*4 (A-Z)

        INCLUDE  'krige.dec'

C Passed
        REAL*4          X0, Y0
        INTEGER*4       Neighbors (MAXNEIGHBOR), N

C Local Variables
        REAL*4        X, Y, Dx, Dy, Mx, My, Bx, By, Tx, Ty
        REAL*4        Xfudge, Yfudge, Dtr, Factor
        REAL*4        CosA, SinA, CosE, SinE, Tmax
	REAL*4	      Txmin, Txmax, Tymin, Tymax, TxTinc, TyTinc
	REAL*4	      TopMargin, BottomMargin
	        
	INTEGER*4     Tnxtic, Tnytic
        INTEGER*4     Ptr, L, L2, Angl, Ierr, Key
	INTEGER*4     DevMinX, DevMaxX, DevMinY, DevMaxY

        CHARACTER     Plot Title*30, Title2*50, Xlabel*10, Ylabel*10
        character*72 message, menu choice*10
        integer destination

        DATA Dtr /0.0174533/

C Set up labels

        Plot Title      = 'Plot of Neighbors'
        Xlabel = Varnam (XVar)
        Ylabel = Varnam (YVar)
        WRITE (Title2, 5) X0, Y0
 5      FORMAT ('Point: (',F13.3,',',F13.3,')') 

C Compute SQUARE Device space (corrected by device pixel aspect ratio)
 1      DevMinX		= 0
	DevMinY		= 0
	DevMaxY		= YmaxDev
        Factor 	= 29./45.
	DevMaxX = FLOAT (DevMaxY) / Factor
	Aspect 	= FLOAT (Xmaxdev) / FLOAT (Ymaxdev)
  
C  Calculate other scaling information
	Txmin	= Xmin
	Txmax	= Xmax
	Tymin	= Ymin
	Tymax	= Ymax
	CALL Neatax (Txmin, Txmax, TxTinc, Tnxtic)
	CALL Neatax (Tymin, Tymax, TyTinc, Tnytic)
	Dx 	= ABS (TXmax-TXmin)
	Dy 	= ABS (TYmax-TYmin)
	Ratio	= Dx / Dy
	Tmax 	= AMAX1 (Dx, Dy)
	Minx 	= TXmin - Tmax * .265
	MaxX	= TXmin + Tmax * 1.1
	MinY 	= TYmin - Tmax * .265
	Maxy 	= TYmin + Tmax * 1.1
	Mx = dx * .005/Aspect
	My = dy * .005
	Tx = (Maxx - Minx) * .005
	Ty = (Maxy - Miny) * .007

C Shift vertically to center of the screen (for horizontal rectangles)
	IF (Ratio .GT. 1.0) THEN
 10	  TopMargin 	= MaxY - TYmax
	  BottomMargin	= TYmin - Miny
	  IF (TopMargin .GT. BottomMargin) THEN
	     Ty		= AMIN1 (TopMargin, BottomMargin) * .1
	     IF (MaxY - Ty .GT. TYmax) THEN
	       MaxY 	= MaxY - Ty
	       MinY 	= MinY - Ty
	       GOTO 10
	     ENDIF
	  ENDIF
	ENDIF

C Start graphics mode
        CALL Zsetup (Gmode, Ierr)


C Set text attributes (8x8, white on black border)
        CALL Zlbdir (0.0)
	CALL Zlocat (DevMinX, DevMaxX, DevMinY, DevMaxY)

C Set scaling window
        CALL Zscale (Minx, Maxx + Tx, miny, Maxy +Ty)
        call grframe(1,1,blue)
        CALL Zcolor (INT2 (White))

C Draw and label the axes
	CALL Axes (Plot Title, Xlabel, Ylabel, Title2,
     +       TXmin, TXmax, TYmin, TYmax, TXtinc, TYtinc, DevCode)

C Plot data points 
        DO 15 I = 1, NData
           CALL justCross  (XData (I), Ydata (I), Mx, My)
 15     CONTINUE

C Plot Block dimensions
	Tx = AMIN1 (XcellSize/2.0, Dx)
	Ty = AMIN1 (YcellSize/2.0, Dy)
	CALL box (X0, Y0, Tx, Ty, Blue, White)

C Plot discretization points
	Tx = Mx*.5
	Ty = My*.5
	IF (.NOT. Point) THEN
           call zcolor(blue)
	   DO 16 I = 1, Na
	      CALL justPlus (X0+Xa(I), Y0+Ya (I), Tx, Ty)
 16	   CONTINUE
	ENDIF

C Draw search ellipse
	CALL Zcolor (INT2 (Red))
        CosE = COS (SrAngle*Dtr)
        SinE = SIN (SrAngle*Dtr)
        X = X0 + SrMajor * CosE 
        Y = Y0 + SrMajor * SinE 
        CALL Zmove (X, Y)
        DO 20 Angl =  6, 360, 6
           CosA = COS (FLOAT (Angl)*Dtr)
           SinA = SIN (FLOAT (Angl)*Dtr)
           Tx   = SrMajor * CosA
           Ty   = SrMinor * SinA 
           X    = X0 + (Tx*CosE - Ty*SinE)
           Y    = Y0 + (Tx*SinE + Ty*CosE)
           CALL Zplot (X, Y)             
 20     CONTINUE

C Draw Sector division lines
	IF (Nsector .EQ. 1) GOTO 40
        DO 30 Angl =  0, 360, 45
	 IF (MOD (Angl, 90) .NE. 0 .AND. Nsector .EQ. 4) GOTO 30
         CosA = COS (FLOAT (Angl)*Dtr)
         SinA = SIN (FLOAT (Angl)*Dtr)
         Tx   = SrMajor * CosA
         Ty   = SrMinor * SinA 
         X    = X0 + (Tx*CosE - Ty*SinE)
         Y    = Y0 + (Tx*SinE + Ty*CosE)
	 CALL Zmove (X0, Y0)
         CALL Zplot (X, Y)             
 30     CONTINUE

C Plot Neighbors
 40     CALL Zcolor (INT2 (Yellow))
	Mx = Mx * 1.5
	My = My * 1.5
        DO 50 I = 1, N
           Ptr  = Neighbors (I)
           CALL justPlus (XData (Ptr), Ydata (Ptr), Mx, My)
 50     CONTINUE
        call zcolor(white)
 
	CALL getthekey(key)
        if (char(key) .eq. 'r' .or. char(key) .eq. 'R') then
c     redraw screen
           goto 1
        else if (char(key) .eq. 'q' .or. char(key) .eq. 'Q') then
           keep = .false.
        else if (key .eq. 2002 .or. char(key) .eq. 'p') then
cclong11/30/93

           message='neighbor.ps'
           call post script(message,destination)
           if (destination .eq. 1) goto 1

           i = lenstr(message)
           call putstr(message(1:60),60,5,10,2)
           j = lenstr(fileprefix)
           call turn_ps_on(fileprefix(1:j)//message(1:i))
           goto 1
cclong11/30/93
        end if

 666  call reset()

      RETURN
      END
