cc$LARGE
C System.for - system solver for KRIGE
C----------------------------------------------------------------------
C Build System - set up system of equations for program KRIGE
C
        SUBROUTINE Build System (X0, Y0, Neighbors, N,
     +					 TNeighbors, TN, Index)
	IMPLICIT INTEGER*4 (A-Z)

        INCLUDE 'krige.dec'

C Passed Variables 
        REAL*4          X0, Y0 
        INTEGER*4       Neighbors (MAXNEIGHBOR), N, Index
        INTEGER*4       TNeighbors (MAXNEIGHBOR), TN

C Local Variables
        INTEGER*4       I, J, IPtr, JPtr, Neq
        REAL*4          X, Y, Xc, Yc, D1x, D1y, D2x, D2y
	REAL*4		Cbar, Gamma, G

C Initialize 
        Var = Index
        DO 10 I = 1, MAXEQ
           B  (I)    = 0.0
           B2 (I)    = 0.0
           DO 10 J = 1, MAXEQ
               W (I, J) = 0.0
 10     CONTINUE        

C Build list of sample with non-missing values
	TN = 0
	DO 15 I = 1, N 
           IPtr         = Neighbors (I)
	   IF (Data (Iptr, Index) .LT. Missing) THEN
	      TN = TN + 1
	      TNeighbors (TN) = Iptr
	   ENDIF
 15	CONTINUE
	IF (TN .LE. 0) RETURN

C Set up System of Equations for simple, or ordinary kriging
        DO 20 I = 1, TN
           IPtr         = TNeighbors (I)
           D1x          = XData (IPtr) - X0
           D1y          = YData (IPtr) - Y0
           B  (I)       = Cbar (D1x, D1y, Index)
           B2 (I)       = B (I)
           DO 25 J = 1, I-1
                JPtr      = TNeighbors (J)
                D2x       = XData (Iptr) - XData (Jptr)
                D2y       = YData (Iptr) - YData (Jptr)
                G	  = Gamma (D2x, D2y, Index)
	        IF (G .LE. 1.E-30) THEN
	           W (I, J) = GamMax (Index) * .999
	        ELSE
                   W (I, J)  =  GamMax (Index) - G
	        ENDIF
                IF (I .NE. J) W (J, I) = W (I, J)
 25	   CONTINUE
           W (I, I) =  GamMax (Index)
 20     CONTINUE
        Neq = TN

C Fill in for ordinary kriging
        IF (Ktype .GE. ORDINARY) THEN
            Neq = TN + 1
            DO 30 I = 1, TN
               W (TN+1, I) = 1.0
               W (I, TN+1) = 1.0
 30         CONTINUE
            W  (TN+1, TN+1) = 0.0
            B  (TN+1)       = 1.0
            B2 (TN+1)       = B (TN+1)
        ENDIF

        RETURN
        END

C----------------------------------------------------------------------
C Solve System - solve system of equations for program KRIGE
C
        SUBROUTINE Solve System (Neighbors, N, Vk, Sk, Index, SysError)
	IMPLICIT INTEGER*4 (A-Z)

        INCLUDE 'krige.dec'

        LOGICAL         SysError
        INTEGER*4       Neighbors (MAXNEIGHBOR), N, Ndim
        INTEGER*4       K, I, Ptr, Index
        REAL*4          Vk, Sk

        IF      (Ktype .EQ. 0) THEN
            Ndim = N
        ELSE IF (Ktype .EQ. 1) THEN 
            Ndim = N + 1
        ENDIF

        CALL Simeq (Ndim, SysError)

C Message for system error
        IF (SysError) THEN
	   CALL Beep
ccsow	   CALL Locate Cursor (24, 2)
	   CALL Locate Cursor (23, 2)
cc
	   CALL Put Characters
     +	   ('Error in system of equations ...', 32, 0)
	   CALL getthekey (Key)
ccsow	   CALL Erase (24, 2, 35)
	   CALL Erase (23, 2, 35)
cc
	   Vk = Missing
	   Sk = Missing
           RETURN
        ENDIF

        Vk = 0.0
        Sk = 0.0
        DO 10 K = 1, N
           Ptr = Neighbors (K)
           Vk = Vk + (Data (Ptr, Index) - Mglobal (Index)) * B (K)
           Sk = Sk + B2 (K) * B (K)
 10     CONTINUE
        Vk = Vk + Mglobal (Index)
        Sk = Cvv (Index) - Sk - B (N+1)

C compute kriging standard deviation
	IF (ABS (Sk) .LT. 1.E-5*GamMax (Index)) Sk = 0.0
	IF (Sk .GE. 0.) THEN
	    Sk = SQRT (Sk)
	ELSE
	   CALL Beep
ccsow	   CALL Locate Cursor (24, 2)
	   CALL Locate Cursor (23, 2)
cc
	   CALL Put Characters
     +	   ('Negative kriging variance ...', 29, 0)
	   Sk = Missing
	ENDIF  
        RETURN
        END

C----------------------------------------------------------------------
C Simeq - system of equations solver for program KRIGE
C
        SUBROUTINE Simeq (N, Error)
	IMPLICIT INTEGER*4 (A-Z)

        INCLUDE 'krige.dec'

C Passed variables
        INTEGER*4       N
        LOGICAL         Error

C Local variables
        INTEGER*4       I, J, K
        REAL*4          Tiny, Sum
        DATA            Tiny /1.0E-6/

C Solve system
        Error = .TRUE.
        DO 10 J = 1, N
           IF (J .GT. 1) THEN
              DO 20 I = 1, J-1
                 Sum =  W (I, J)
                 IF (I .GT. 1) THEN
                    DO 30 K = 1, I-1
 30                    Sum = Sum - W (I, K) * W (K, J)
                 END IF   
                 W (I, J) = Sum
 20           CONTINUE
           END IF

           DO 40 I = J, N
              Sum = W (I, J)
              IF (J .GT. 1) THEN
                 DO 50 K = 1, J-1
 50                 Sum = Sum - W (I, K) * W (K, J)
              ENDIF 
              W (I, J) = Sum
 40        CONTINUE

           IF (J .LT. N) THEN
              IF (W (J, J) .GE. Tiny) THEN
                 DO 60 I = J+1, N
                    W (I, J) = W (I, J) / W (J, J)      
 60              CONTINUE 
              ELSE
                RETURN 
              ENDIF
           END IF
 10     CONTINUE

        DO 70 I = 1, N
           Sum = B (I)
           IF (I .GT. 1) THEN
              DO 80 J = 1, I-1
 80              Sum = Sum - W (I, J) * B(J)
           ENDIF
           B (I) = Sum
 70     CONTINUE

        DO 90 I = N, 1, -1
           Sum = B (I)
           IF (I .LT. N) THEN
              DO 100 J = I+1, N
 100            Sum = Sum - W (I, J) * B (J)
           ENDIF
           B (I) = Sum / W (I, I)
 90     CONTINUE

        Error = .FALSE.
        RETURN
        END       

C----------------------------------------------------------------------
C Digitize - calculate relative offsets of discretization points from
C            block center. These are stored into global arrays  Xa, Ya
C
        SUBROUTINE Digitize
	IMPLICIT INTEGER*4 (A-Z)

        INCLUDE  'krige.dec'

        INTEGER*4       I, J, L
        REAL*4          X0, Y0, Dx, Dy, Dx2, Dy2

        IF (Point) THEN
           Na     = 1
           Ndiscr = 1
           Xa (1) = 0.0
           Ya (1) = 0.0
        ELSE
           X0 = XcellSize * 0.5
           Y0 = YcellSize * 0.5
           Dx = XcellSize / REAL (Ndiscr)
           Dy = YcellSize / REAL (Ndiscr)
           Dx2 = Dx * 0.5
           Dy2 = Dy * 0.5
           L = 0
           Na = Ndiscr * Ndiscr
           DO 10 I = 1, Ndiscr
              DO 10 J = 1, Ndiscr
                 L = L + 1
                 Xa (L) = (Dx2 + ((I-1) * Dx)) - X0
                 Ya (L) = (Dy2 + ((J-1) * Dy)) - Y0
 10        CONTINUE
        ENDIF
        RETURN
        END

C----------------------------------------------------------------------
C Cbar - calculates point to block covariance for program KRIGE
C
        REAL*4  FUNCTION Cbar (Hx, Hy, Index)
	IMPLICIT INTEGER*4 (A-Z)

        INCLUDE 'krige.dec'

        REAL*4          Hx, Hy,  Gamma, Dx, Dy, Tiny
	REAL*8		Cb
        INTEGER*4       Index, I
	DATA		Tiny /1.E-6/

        Cb = 0.0
        DO 10 I = 1, Na
           Dx = Hx + Xa (I)
           Dy = Hy + Ya (I)
	   Dx = AMAX1 (ABS (Dx), Tiny)
	   Dy = AMAX1 (ABS (Dy), Tiny)
           Cb = Cb + Gamma (Dx, Dy, Index)
 10     CONTINUE
        Cbar =  GamMax (Index) -  (Cb  / REAL (Na))
        RETURN
        END

C----------------------------------------------------------------------
C Gamma - function for calculating variogram value - program KRIGE
C
        REAL*4  FUNCTION Gamma (Dx, Dy, Index)
	IMPLICIT INTEGER*4 (A-Z)

        INCLUDE 'krige.dec'

        REAL*4          Dx, Dy
	REAL*8		Tiny, Teeny, Distance, Slope
        REAL*8          Dtr, Hu, Hv, G, R, H, Angle
	INTEGER*4	Index

        DATA            Dtr 	/0.0174533/
	DATA		Tiny 	/1.01E-6/
	DATA		Teeny 	/1.E-20/

C compute simple distance
	Distance = Dx * Dx + Dy * Dy
	IF (Distance .GT. 0.0) THEN
	   Distance = SQRT (Distance)
	ELSE
	   Distance = 0.0
	ENDIF

C variogram for 'zero' distance
	IF (Distance .LE. Teeny) THEN
	   Gamma = 0.0
	   RETURN

C variogram value for very small distances
	ELSE IF (ABS (Dx) .LE. Tiny .AND. ABS (Dy) .LE. Tiny) THEN
           Gamma = Nugget (Index)
	   RETURN
	ENDIF
	
C variogram value for non-small distances
	G = Nugget (Index)

C Sum the nested structures
        DO 10 I = 1, 4

C Skip the 'null structure'
	   IF (Mtype (I, Index) .EQ. 0) GOTO 10

C Compute rotated distance componments (Hu, Hv) (avoid COS/SIN cost)
	   IF (VrAngle (I, Index) .GT. Teeny) THEN
             Angle      = -Vrangle (I, Index) * Dtr
             Hu         = Dx * COS (Angle) - Dy * SIN (Angle)
             Hv         = Dy * COS (Angle) + Dx * SIN (Angle)
	   ELSE
	     Hu		= Dx 
	     Hv		= Dy
	   ENDIF

C Scale with anisotropy ratio and compute structural distance H 
           Hu           = Hu * (VrMinor (I, Index) / VrMajor (I, Index))
           H            = SQRT (Hu*Hu+Hv*Hv)

C Compute Gamma for structure type
C
C Spherical
           IF      (Mtype (I, Index) .EQ. 1) THEN
              R = H / VrMinor (I, Index)
              IF (R .GT. 1.0) R = 1.0
              G = G + Sill (I, Index) * (1.5*R - 0.5*R*R*R)

C Gaussian (practical range is divided by SQRT (3))
           ELSE IF (Mtype (I, Index) .EQ. 2) THEN
              R = H / (VrMinor (I, Index) / 1.73205)
              G = G + Sill (I, Index) * (1.-EXP(-R*R))

C Exponential (practical range is divided by 3)
           ELSE IF (Mtype (I, Index) .EQ. 3) THEN
              R = H / (VrMinor (I, Index) / 3.0)
              G = G + Sill (I, Index) * (1.- EXP (-R))

C Linear
           ELSE IF (Mtype (I, Index) .EQ. 4) THEN
	      Slope = Sill (I, Index) / VrMinor (I, Index)
              G = G + Slope * H
           ENDIF
 10     CONTINUE

        Gamma = G
        RETURN
        END

C----------------------------------------------------------------------
C Calc CVV - calculates average block covariance for KRIGE

        SUBROUTINE Calc CVV
	IMPLICIT INTEGER*4 (A-Z)
	PARAMETER (NDim = 64)

        INCLUDE 'krige.dec'


	REAL*4		Dx, Dy, X, Y, X0, Y0, X1, Y1, X2, Y2
        REAL*4          Gamma, Tiny, Dx2, Dy2, G
	REAL*4		XDig (NDIM), YDig (NDIM)
	REAL*8		Sum
	INTEGER*4	TN, L, N

	DATA Tiny /1.E-6/

C compute digitized offsets to speed CVV computations
	N 	= Ndiscr
	IF (.NOT. Point) THEN
          X0 	= XcellSize * 0.5
          Y0 	= YcellSize * 0.5
          Dx 	= XcellSize / REAL (N)
          Dy 	= YcellSize / REAL (N)
	  Dx2	= Dx * 0.5
	  Dy2	= Dy * 0.5
	  TN 	= 0
	  DO 5 I = 1, N
	     DO 5 J = 1, N
	       TN = TN + 1
	       XDig (TN) = Dx2 + (I-1)*Dx - X0
	       YDig (TN) = Dy2 + (J-1)*Dy - Y0
 5	  CONTINUE
	ENDIF

C for each variable kriged, calculate average block covariance CVV (I)
        DO 10 K = 1, NkVar

C calculate GammaMax, and check for pure nugget effect
           GamMax (K) = Nugget (K)
	   DO 15 I = 1, 4
	        IF (Mtype (I, K) .NE. 0)
     +           GamMax (K) = GamMax (K) + Sill (I, K)
 15	   CONTINUE

C if model is pure nugget, then Average block covariance will be 0.0
C else calculate CVV for point, and block cases

	   CVV (K) = 0.0
           IF (Point) THEN
	       Cvv (K) = GamMax (K)
           ELSE
	       Sum 	= 0.0
	       Cvv (K) 	= 0.0
	       DO 30 I = TN, 1, -1
	           DO 30 J = 1, TN, 1
	             X = AMAX1 (ABS (Xdig (I) - Xdig (J)), Tiny)
	             Y = AMAX1 (ABS (Ydig (I) - Ydig (J)), Tiny)
		     Sum = Sum + Gamma (X, Y, K)
 30            CONTINUE
	       Sum = Sum / FLOAT (TN*TN)
	       Cvv (K) = GamMax (K) - Sum
           END IF
 10     CONTINUE
        RETURN
        END


	 
               
                    
 

             
 
                


