cc      $LARGE
C----------------------------------------------------------------------
C Batch - print batch statistics for STAT1D               
C		                                                        

	SUBROUTINE Batch              

	IMPLICIT INTEGER*4 (A-Z)
	PARAMETER (NSTATS = 11)

          INCLUDE '../common/screen.str'
          INCLUDE 'stats.dec'

C Table - 2D array to hold stats calculations before they got to file/prn
C Itable
C Printer	- flag to indicate: use printer
C Fexist	- flag for file existance
C Overwrite	- flag for file overwrite
C Error		- error flag
C Kvar		- temporary index for variables in file
C Ipos		- temporary position 
C Batch file	- report file name
C File name	- file name with prefix attached
C Tname		- temporary variable name

	REAL*4		Table (NSTATS, MAXVAR)
	INTEGER*4	Itable (3, MAXVAR)
	LOGICAL		Printer, Fexist
	LOGICAL  	GetYesNo, OverWrite, Error
	INTEGER*4	Fld Index, Kvar, Ipos
	CHARACTER*14	Batch File	
	CHARACTER*65	File Name, Names
cc      Changed as per prevar; was on the line with *65
        CHARACTER*10    Tname

C Force unweighted statistics
	Weighted = .FALSE.
	Wvar	 = Nvar + 1
	CALL Set Toggle Position ('WVARIABLE ', Nvar)
	CALL Display One Field   ('WVARIABLE ') 

C Get choice for printer or file output
  	Printer = (Toggle ('PRINTER   ') .EQ. 0)
	CALL Erase (20, 2, 76)

C If file output, get batch report file name
	IF (.NOT. Printer) THEN
	   Next Field = Fld Index   ('BATCHFILE ')
	   CALL Get Field
	   CALL Erase (20, 2, 76)
	   CALL Get Character Value ('BATCHFILE ', Batch File)

C Build file name
	   N = LENSTR (File Prefix)
	   IF (N .GT. 0) THEN 
	      File Name = File Prefix (1:N) // Batch File
	   ELSE
	      File Name = Batch File
	   ENDIF

C check file existance
	   INQUIRE (FILE = File Name, EXIST = Fexist)  
	   IF (Fexist) THEN
	      CALL Erase (20, 2, 75)
	   Overwrite = GetYesNo (20,2,'File exists, OverWrite <Y/N>?')
	   CALL Erase (20, 2, 76)
	      IF (.NOT. OverWrite) GOTO 999
	   ENDIF

C Open file
cclong added "status='unknown'" due to compile error
	   OPEN (UNIT = 14, FILE = File Name, STATUS='UNKNOWN', ERR = 111)
	   GOTO 300

C error handling for file open error
C----------------------------------------------------------------------
 111 	   CLOSE (14)
	   CALL Beep
	   CALL PutStr ('  ERROR creating report file ... ',33,11,43,2)
	   CALL PutStr ('An ERROR was encountered while   ',33,12,43,0)
	   CALL PutStr ('trying to create the named file. ',33,13,43,0)
	   CALL PutStr ('Check that the disk is not full, ',33,14,43,0)
	   CALL PutStr ('  or that the drive is ready for ',33,15,43,0)
	   CALL PutStr ('  use.                           ',33,16,43,0)
	   CALL FLASH (20, 2, ' Press any key to exit ')
	   GOTO 999

C open communications to printer
C----------------------------------------------------------------------
  	ELSE
 200	   CALL Flash (20, 2, 'Get printer ready, press any key ...') 
	   OPEN (UNIT=14, FILE='PRN', STATUS='UNKNOWN', ERR=222)
	   GOTO 300

C printer not ready error
C----------------------------------------------------------------------
 222 	   CLOSE (14)
	   CALL Beep
	   CALL PutStr ('  ERROR accessing the printer... ',33,11,43,2)
	   CALL PutStr ('An ERROR was encountered while   ',33,12,43,0)
	   CALL PutStr ('trying to access the printer     ',33,13,43,0)
	   CALL PutStr ('Check that the printer is ready, ',33,14,43,0)
	   CALL PutStr ('  connected, and online.         ',33,15,43,0)
	   CALL FLASH (20, 2, ' Press any key to exit ')
	   GOTO 999
C----------------------------------------------------------------------
	ENDIF

C initialize table
C
 300	CONTINUE
	DO 15, K = 1, NSTATS
	   DO 15 J = 1, MAXVAR
 		 TABLE (K, J) = 0.0
 15	CONTINUE

C For each variable, read file, compute stats, store in table
C
	DO 20 IVar = 1, Nvar
	   Var = Ivar
	   CALL Set Toggle Position ('VARIABLE  ', Ivar-1)
	   CALL Display One Field   ('VARIABLE  ')
	   CALL Read Data (Error)
	   IF (Error) GOTO 999
	   CALL Calculate Stats
	   ITable (1,Ivar) = (Nkept)
	   ITable (2,Ivar) = (Nmiss)
	   ITable (3,Ivar) = (NumNeg)
	   Table (1, Ivar) = Mean
	   Table (2, Ivar) = Variance
	   Table (3, Ivar) = Stdev
	   Table (4, Ivar) = CoefVar
	   Table (5, Ivar) = Skewness
	   Table (6, Ivar) = Kurtosis
	   Table (7 ,Ivar) = Min     
	   Table (8 ,Ivar) = Q1
	   Table (9 ,Ivar) = Median
	   Table (10,Ivar) = Q3
	   Table (11,Ivar) = Max
 20     CONTINUE

C Send results to output
	CALL PutStr (' Writing Results ...       ', 27, 20, 2, 2)

C Build file name
	N = LENSTR (File Prefix)
	IF (N .GT. 0) THEN 
	   File Name = File Prefix (1:N) // Data File
	ELSE
	   File Name = Data File
	ENDIF

C Begin writing header info for output
	WRITE (14, 1001, ERR = 400)
	IF (Log) WRITE (14, 1002, ERR = 400)
	WRITE (14, 1003, ERR = 400) File Name

C Write each page of output
	DO 30 Ivar = 1, Nvar, 5
	   Kvar = Ivar + 4
	   IF (Kvar .GT. Nvar) Kvar = Nvar

	   Names = ' '
	   DO 35 Jvar = Ivar, Kvar
              Tname = VarNam (Jvar)
	      CALL Right Justify (Tname, 10)
	      Ipos = (Jvar - Ivar) * 13	+ 1
	      Names (Ipos :Ipos + 9) = Tname
 35	   CONTINUE  

	   WRITE (14, 1005, ERR = 400)  Names

	   WRITE (14, 1010, ERR = 400) 'N used     :',
     +			     (ITable (1, Jvar), Jvar = Ivar, Kvar)  
	   WRITE (14, 1010, ERR = 400) 'N missing  :',
     +			     (ITable (2, Jvar), Jvar = Ivar, Kvar)  
	   WRITE (14, 1010, ERR = 400) 'N .LE. 0   :',
     +			     (ITable (3, Jvar), Jvar = Ivar, Kvar)  
	   WRITE (14, 1030, ERR = 400)
	   WRITE (14, 1020, ERR = 400) 'Mean       :',
     +			     (Table (1,  Jvar), Jvar = Ivar, Kvar)  
	   WRITE (14, 1020, ERR = 400) 'Variance   :',
     +			     (Table (2,  Jvar), Jvar = Ivar, Kvar)  
	   WRITE (14, 1020, ERR = 400) 'Std. Dev.  :',
     +			     (Table (3,  Jvar), Jvar = Ivar, Kvar)  
	   WRITE (14, 1020, ERR = 400) 'Coef. Var. :',
     +			     (Table (4,  Jvar), Jvar = Ivar, Kvar)  
	   WRITE (14, 1020, ERR = 400) 'Skewness   :',
     +			     (Table (5,  Jvar), Jvar = Ivar, Kvar)  
	   WRITE (14, 1020, ERR = 400) 'Kurtosis   :', 
     +			     (Table (6,  Jvar), Jvar = Ivar, Kvar)  
	   WRITE (14, 1030, ERR = 400)
	   WRITE (14, 1020, ERR = 400) 'Minimum    :',
     +			     (Table (7,  Jvar), Jvar = Ivar, Kvar)  
	   WRITE (14, 1020, ERR = 400) '25th %tile :',
     +			     (Table (8,  Jvar), Jvar = Ivar, Kvar)  
	   WRITE (14, 1020, ERR = 400) 'Median     :',
     +			     (Table (9,  Jvar), Jvar = Ivar, Kvar)  
	   WRITE (14, 1020, ERR = 400) '75th %tile :',
     +			     (Table (10, Jvar), Jvar = Ivar, Kvar)  
	   WRITE (14, 1020, ERR = 400) 'Maximum    :',
     +			     (Table (11, Jvar), Jvar = Ivar, Kvar)  
	   WRITE (14, 1030, ERR = 400)
	   IF (MOD (Ivar, 2) .EQ. 0 .AND. Ivar .NE. Nvar) THEN
	      WRITE (14, *) CHAR (12)
	      WRITE (14, 1001, ERR = 400)
	      IF (Log) WRITE (14, 1002, ERR = 400)
	      WRITE (14, 1003, ERR = 400) File Name
	   ENDIF
	   
 30	CONTINUE	     
	WRITE (14, *) CHAR(12)
	CLOSE (14)
        if (printer) call system('lpr PRN;rm -f PRN')
	CALL Flash (20,2,' Batch Statistics completed - press a key ')
	GOTO 999

  400   CLOSE (14)
	CALL PutStr ('    ERROR writing results...     ',33,11,43,2)
	CALL PutStr ('An ERROR was encountered while   ',33,12,43,0)
	CALL PutStr ('trying to WRITE the statistics.  ',33,13,43,0)
	CALL PutStr ('Check that the output device is  ',33,14,43,0)
	CALL PutStr ('  ready, and try again.          ',33,15,43,0)
	CALL FLASH (20, 2, ' Press any key to exit ')
			     
C Subroutine exit
  999	CALL Erase (20, 2, 75)
	RETURN

C Formats for output
 1001   FORMAT (//,25X,' B A T C H   S T A T I S T I C S ')
 1002	FORMAT (/, 25X,'   (for Natural Log of data)')
 1003   FORMAT (//,1X,' Data File: ',A65)
 1005	FORMAT (/,15X, A65,/,1X,78 ('-'))
 1010	FORMAT (1X,A12, 5 (I12,  1X))
 1020	FORMAT (1X,A12, 5 (F12.3,1X))
 1030	FORMAT (1X)
	END

