clarge
c**********************************************************************
c*** File name: TRANSG.FOR (Source code for the program TRANS)
c**********************************************************************
c        O p e n   S c r a t c h   F i l e s   (subroutine)
c**********************************************************************
      SUBROUTINE Open Scratch Files

c***  This routine opens a scratch (temporary) file.
c***
c***  VARIABLES:
c***  File Name - stores the name of the scratch file.
c***  Prefile   - is a global variable that stores the prefix and the
c***              file name.  The routine 'Build Prefix Scratch File'
c***              assigns the prefix + file name to 'Prefile'.

      include 'trans.dec'
      include '../common/screen.str'

      CHARACTER*14 File Name

      File Name = 'ZZSCTCH1.FIL'
      CALL Build Prefix Scratch File(File Name)
c***  Open scratch file to contain 'Data1' 'structure'. ***
      OPEN (UNIT=1,FILE=Prefile,ACCESS='DIRECT',RECL=4
     +     , IOSTAT=I, FORM='unformatted')
ccc     +     , IOSTAT=I, FORM='BINARY')
		
      RETURN
      END
C**********************************************************************
c        C l o s e   S c r a t c h   F i l e s   (subroutine)
C**********************************************************************
      SUBROUTINE Close Scratch Files

c***  This routine closes the scratch (temporary) scratch file.

      include 'trans.dec'
      include '../common/screen.str'

      CHARACTER  Writemess*25

      Writemess = ' Closing scratch files...'
      CALL Put Str (Writemess, 25, 20, 2, 2)
c***  Close scratch file that stores 'Data1' 'structure'. ***
      CLOSE(1, STATUS='DELETE')
c***  Erase message line.
      CALL Erase(20,2,30)
	
      RETURN
      END
c**********************************************************************
c        W r i t e   t o   O u t p u t   F i l e     (subroutine)
c**********************************************************************
      SUBROUTINE Write to Output File 

c*** This subroutine prompts the user for the name of an output
c*** file.  If the file does not exist, one is opened.  The data
c*** data is then written to that output file.
c***
c*** VARIABLES:
c*** Fmt       - An array which is the output format specifier.
c*** Vector48  - Global real array of 48 elements used to store a
c***             a row of data.  
c*** Varnam    - Global array of character strings used to store
c***             variable names.
c*** Units     - Global array of character strings used to store
c***             measurement descriptions of variables.
c*** Formats   - Global array of character strings used to store
c***             variable output formats.
c*** Outfil    - Global variable that stores the name of the 
c***             output file. 
c*** Noinfile  - Global variable that is set to .true. if an
c***             input file has not been accessed.

      include 'trans.dec'
      include '../common/screen.str'

      CHARACTER  Writemess*16, Old File*14, Fname*10
      LOGICAL  Exist, Get Yes No
      INTEGER*4  I, J, K, L, Num Missing
      REAL*4     Get Element

      Writemess = 'Writing data ...'
      Exist     = .FALSE.
      Old File  = '              '
      Fname     = 'OUTFILE   '
      Missing   = 1.E+31

c***  Concatenate prefix to filename.  The global variable 'Prefile' will
c***  contain the prefix + filename.
      CALL Build Prefix Filename (Outfil)

      INQUIRE (FILE = Prefile, EXIST = Exist)
      IF (Exist) THEN
 	 OPEN (UNIT = 11, FILE = Prefile, ERR = 999, STATUS = 'OLD')
      ELSE
	 IF (Outfil .EQ. '              ') THEN
	   CALL Flash (20,2,
     +          'ERROR - Blanks not allowed...(press any key)')
	   Noinfile = .TRUE.
	   RETURN
	 ELSE
            OPEN (UNIT = 11, FILE = Prefile, ERR = 999, STATUS = 'NEW')
	 ENDIF
      ENDIF
      CALL Put Str (Writemess, 16, 20, 2, 2)
c***  Print heading 
      WRITE (11, '(A)', ERR = 799) Title
c***  Print number of variables
      WRITE (11, '(I5)', ERR = 799) Nvar
c***  Write out variable names, measurement descriptions and output formats
c***  to the Geo-EAS output data file.
      DO 20 I = 1, Nvar
        WRITE (11, 25, ERR=799) Varnam(I), Units(I), Formats(I)
  25    FORMAT(2A10,A6)
  20  CONTINUE
c***  Construct the format array 'Fmt' which is the output format 
c***  specifier for the variables.
      Fmt(1:1) = '('
      J = 1
      DO 10 I = 1, Nvar - 1
	K = LENSTR (Formats(I))
	DO 15 L = 1, K
	  J = J + 1
	  Fmt(J:J) = Formats(I)(L:L)
  15	CONTINUE
        J = J + 1
        Fmt(J:J+3) = ',1X,'
	J = J + 3
  10  CONTINUE
      K = LENSTR (Formats(Nvar))
      DO 11 L = 1, K
        J = J + 1
	Fmt(J:J) = Formats(Nvar)(L:L)
  11  CONTINUE
      J = J + 1
      Fmt(J:J) = ')'

      DO 30 I = 1, Ndata
        Num Missing = 0
        DO 40 J = 1, Nvar
          Vector48(J) = Get Element (1,J,I,MAXVAR)
          IF (Vector48 (J) .EQ. Missing) Num Missing = Num Missing + 1
  40    CONTINUE
        IF (Num Missing .GT. 0) THEN
c*** 4/18/90 Modified following line.
c           Write (11,'(48G16.9)',ERR=799) (Vector48(K), K = 1, Nvar)
           Write (11,'(G16.9,47(1X,G16.9))',ERR=799) 
     +                (Vector48(K), K = 1, Nvar)
        ELSE
           Write (11,Fmt,ERR=799) (Vector48(K), K = 1, Nvar)
        ENDIF
  30  CONTINUE

c***  Erase message line.
      CALL Erase (20, 1, 78)
c***  Reset Newdata to indicate that new data has been saved.
      Newdata = .FALSE.
c***  Close ouput file.
      CLOSE (11)
      CALL Flash(20,2,
     +	 'Data has been sent to the output file...(press any key)')
      RETURN

 799  CALL Beep
      CLOSE (11)
      CALL Flash (20, 2,
     +'ERROR - writing data to output file...(Press any key)')
      RETURN

 999  CALL Beep
      CLOSE (11)
      CALL Flash (20, 2,
     + 'ERROR - opening output file...(press any key)')
      RETURN

      END
c**********************************************************************
c	 G e t   E l e m e n t       (function)
c**********************************************************************
      REAL*4 FUNCTION Get Element(Data Unit, Col, Index, Varnum)
 
c*** This routine reads a sample from a scratch file corresponding to the
c*** requested variable designated by 'Col' and assigns to Get Element.
c***
c*** V a r i a b l e   N a m e s
c*** Data Unit - Unit number of scratch file which is always 1.
c*** Col       - variable requested from scratch file.
c*** Index     - data item requested of 'Col'.
c*** Varnum    - number of variables.

      include 'trans.dec'
      include '../common/screen.str'

      REAL*4      Value
      INTEGER*4   Varnum, Data Unit, J, Index, I

      J = Col + ((Index - 1) * Varnum) 
      READ (Data Unit, REC = J) Value
      Get Element = Value

      RETURN
      END

c**********************************************************************
c	 P u t   E l e m e n t       (subroutine)
c**********************************************************************
      SUBROUTINE Put Element (Data Unit,Col,Index,Varnum,Value,Error)
 
c*** This routine replaces a sample in the scratch file. 
c***
c*** V a r i a b l e:
c*** Data Unit - Unit number of scratch file which is always 1.
c*** Col       - variable requested from infile.
c*** Index     - data item requested of 'Col'.
c*** Varnum    - number of variables.
c*** Value     - value replacement
c*** Error     - True if error writing to scratch file.

      include 'trans.dec'
      include '../common/screen.str'

      REAL*4     Value
      INTEGER*4  Varnum, Data Unit, Col, J, Index
      LOGICAL  Error

      J = Col + ((Index - 1) * Varnum) 
      WRITE (Data Unit, REC = J, ERR = 799) Value
      RETURN

  799 CALL Beep
      Error = .TRUE.
      Noinfile = .TRUE.
      CALL Flash (20, 2,
     +'ERROR - writing to scratch file...(Press any key)')

      RETURN
      END
C**********************************************************************
c        Read System Defaults   (subroutine)
C**********************************************************************
      SUBROUTINE Read System Defaults

c***  This routine reads in a Prefix name, an Input File name and a
c***  the logical 'Intro' (if they exist) from a 'geoeas.def' file 
c***  (if it exists) and assigns these names to the corresponding 
c***  variables(character strings).
c***
c***  VARIABLES:
c***  Prefix - global variable that contains the name of the Geo-EAS
c***           data file prefix.
c***  Infil  - global variable that contains the name of the Geo-EAS
c***           input data file.
c***  String - variable that contains the character string 
c***           'INTRO=whatever'.

      include 'trans.dec'
      include '../common/screen.str'

      CHARACTER String*80

      OPEN (UNIT = 10, FILE = 'geoeas.def', STATUS = 'UNKNOWN',
     +		       ERR=999)
      READ (10,'(A)',ERR=999,END=999) Prefix
      READ (10,'(A)',ERR=999,END=999) Infil
      READ (10,'(A)',ERR=999,END=999) String

c***  Intro flag embedded as INTRO=ON, INTRO=anything else.
c***  Intro is a global logical.
      Intro = ((String (7:8) .EQ. 'ON') .OR.
     +         (String (7:8) .EQ. 'on')) 
      CALL Set Character Value ('PREFIX    ', Prefix)    
      CALL Set Character Value ('INFILE    ', Infil)    
      CLOSE (10)
      RETURN

  999 CLOSE (10)
      Intro = .TRUE.
      RETURN
      END
C**********************************************************************
c        Write System Defaults   (subroutine)
C**********************************************************************
      SUBROUTINE Write System Defaults

c***  This routine writes out a Prefix name, Output File name 
c***  and the logical 'Intro' to the 'geoeas.def' file.

c***
c***  VARIABLES:
c***  Prefix - global variable that stores the name of the Geo-EAS
c***           data file prefix.
c***  Outfil - global variable that stores the name of the Geo-EAS
c***           output data file.

      include 'trans.dec'

      OPEN (UNIT = 10, FILE = 'geoeas.def', STATUS = 'UNKNOWN',
     +		       ERR=999)
      WRITE (10,'(A)',ERR = 999) Prefix
      WRITE (10,'(A)',ERR = 999) Outfil
      IF (Intro) THEN
        WRITE (10,'(A)',ERR = 999) 'INTRO=ON'
      ELSE
        WRITE (10,'(A)',ERR = 999) 'INTRO=OFF'
      ENDIF

  999 CLOSE (10)
      RETURN
      END
C**********************************************************************



