clarge
c**********************************************************************
c*** File name: TRANSH.FOR (Source code for the program TRANS)
c**********************************************************************
c	 C l e a r   O p e r a t i o n   B o x  (subroutine)
c**********************************************************************
      SUBROUTINE Clear Operation Box

c*** This subroutine clears the operation box.

      include '../common/screen.str'

      CALL Erase (7, 16,  53)
      CALL Erase (8,  2,  13)
      CALL Erase (8, 27,  50)

      RETURN
      END

c**********************************************************************
c	 V m i s s                            (function)
c**********************************************************************
      LOGICAL FUNCTION Vmiss(R1)

c*** This function determines if 'R1' is a missing value.
c*** If 'R1' is missing then Vmiss = .true. else Vmiss = .false..

      REAL*4  R1

      IF ((ABS(1.E+31 - R1) .LE. 0.0001) .AND. (R1 .GE. 0))THEN
 	 Vmiss = .TRUE.
      ELSE
	 Vmiss = .FALSE.
      ENDIF
 
      RETURN
      END
c**********************************************************************
c        M i n m a x                      (subroutine)
c**********************************************************************
      SUBROUTINE Minmax(Col, Nerrors)

c*** This subroutine determines the minimum and maximum values of 
c*** the data array and then prints out the values in a message
c*** that is in reverse video.
c***
c*** FORMAL ARGUMENTS:
c*** Col     - This integer identifies the column number of the
c***           variable.
c*** Nerrors - Number of missing values noted in the variable.  This
c***           value is passed by the calling subroutine.
c***
c*** VARIABLES:
c*** Minimum - stores the field name of the 'Minimum' screen field.
c*** Maximum - stores the field name of the 'Maximum' screen field.
c*** Min     - stores the current minimum value.
c*** Max     - stores the current maximum value. 
c*** Col     - stores the variable(column) number.
c*** Row     - stores the sample item(record number).
c*** Nerrors - stores the number of missing values.

      include 'trans.dec'
      include '../common/screen.str'

      CHARACTER*10  Minimum, Maximum
      CHARACTER*45  Ch
      REAL*4        Get Element, Max, Min
      INTEGER*4	    J, Row, Col, Nerrors
      LOGICAL	    Vmiss, Error

      Minimum = 'MINIMUM   '
      Maximum = 'MAXIMUM   '
      Min     = +1.E+31
      Max     = -1.E+31 
c***  The following do loop determines the minimum and maximum sample
c***  values other than missing values.
      DO 10 Row = 1, Ndata
	IF (.NOT. Vmiss(Get Element (1,Col,Row,MAXVAR))) THEN
	  Max = AMAX1 (Get Element (1,Col,Row,MAXVAR), Max)
	  Min = AMIN1 (Get Element (1,Col,Row,MAXVAR), Min)
	ENDIF
  10  CONTINUE
c***  The following code displays the minimum and maximum values
c***  on the screen.
      Ch = 'Range of Values--Minimum:            Maximum:'
      CALL Put Str(Ch, 45, 20, 2, 2)
      CALL Set 4 Floating Point Value (Minimum, Min, 3)
      CALL Write Field (FLD Index(Minimum), 2)
c***  If Max .EQ. -1.E+31 then all sample data must be missing
c***  values, ie., the minimum and the maximum are 1.E=31.
      IF (Max .EQ. -1.E+31) Max = 1.E+31
      CALL Set 4 Floating Point Value (Maximum, Max, 3)
      CALL Write Field (FLD Index(Maximum), 2)
      CALL Flash (20, 57,
     +     ' (press any key)')
      CALL Erase (20, 2, 70) 

c***  The following routine prompts the user for an output format
c***  and determines if it can print all samples without overflow.
c***  If overflow does occur then the user is prompted for another
c***  output format.
      CALL Check for Output Overflow (Col, Min, Max, Nerrors)

      RETURN
      END
c**********************************************************************
c	U n a r y  P r o c e s s         (subroutine)
c**********************************************************************
      SUBROUTINE Unary Process

c*** This routine displays a menu that permits the user to quit(return to
c*** the preceding menu) or to pick a unary operation via a toggle field.
c*** After deciding upon an operation a subroutine is called that displays
c*** a menu for operand choices(either a variable or constant).
c*** 
c*** VARIABLES:
c*** Menu	 - stores the name of the Unary Operation menu.
c*** Menu Choice - stores the user's menu selection.
c*** Unary Field - stores a field name. 
c*** Var	 - stores the toggle position of the item selected
c***               by the user.
c*** Calcomp	 - global logical variable indicating if the calculation
c***               was completed.

      include 'trans.dec'
      include '../common/screen.str'

      CHARACTER*10  Unary Field, Menu, Choice
      CHARACTER     Heading1*5, Heading2*22, Equal1*3, Menu Choice*12
      INTEGER*4     Var,Toggle

      Menu        = 'MEUSMAIN  '
      Unary Field = 'UNARY     '
      Heading1    = 'Unary'
      Heading2    = 'Operator       Operand'
      Equal1      = ' = '
c***********  Place headings into the operation box  ******************
      CALL Put Str (Heading1,  5, 8,  6, 0)
      CALL Put Str (Heading2, 22, 7, 30, 0)
      CALL Put Str (  Equal1,  3, 8, 27, 0)
c**********************************************************************
      CALL Put Menu(Menu)
  10  CALL Get Menu Choice (Menu, Menu Choice)
      IF (Menu Choice .EQ. 'Operations  ') THEN
	 Uoperator = .TRUE.
	 Boperator = .FALSE.
c***     Prompt user for the specific unary operation 
c***     using a toggle field.
	 Var = Toggle (Unary Field)
	 CALL Select Unary Operand 
	 IF (Calcomp) THEN
	   RETURN
	 ELSE
	   CALL Put Menu(Menu)
	 ENDIF
      ELSE IF (Menu Choice .EQ. 'Quit        ') THEN
 	 CALL Clear Operation Box
         CALL Erase (20,  2, 70)
	 RETURN
      ENDIF
      GOTO 10

      RETURN
      END
c**********************************************************************
c	 B i n a r y  P r o c e s s            (subroutine)
c**********************************************************************
      SUBROUTINE Binary Process

c*** This routine is called by the routine 'Select Binary Operand 1'.
c*** Binary Process displays a menu that permits the user to pick
c*** a specific binary operation, or to quit (return to previous menu).
c*** After picking a binary operation (via a toggle field) the routine
c*** 'Select Binary Operand 2' is called.
c*** 
c*** VARIABLES:
c*** Binary Field - stores the field name of the binary operation field.
c*** Mebvmain1	  - stores the field name of the Select Binary Operand 1
c***                menu.
c*** Menu	  - stores the field name of the Binary Operation menu.
c*** Menu Choice  - stores the user's menu selection.
c*** Var	  - stores the toggle position of item selected by the 
c***                user.
c*** Boperator	  - global logical variable.
c*** Uoperator	  - global logical variable.
c*** Calcomp	 - global logical variable indicating if the calculation
c***               was completed.

      include 'trans.dec'
      include '../common/screen.str'

      CHARACTER*10  Binary Field, Mebvmain1, Menu, Choice
      CHARACTER     Menu Choice*12
      INTEGER*4     Var, Toggle

      Binary Field = 'BINARY    '
      Menu         = 'MEBSMAIN  '
      Mebvmain1    = 'MEBVMAIN1 '

      CALL Put Menu(Menu)
  10  CALL Get Menu Choice (Menu, Menu Choice)
      IF (Menu Choice .EQ. 'Operations  ') THEN
	 Boperator = .TRUE.
	 Uoperator = .FALSE.
c***     Prompt the user for the specific binary operation using 
c***     a toggle field.
	 Var = Toggle (Binary Field)
	 CALL Select Binary Operand 2
	 IF (Calcomp) THEN 
	   RETURN
	 ELSE
	   CALL Put Menu (Menu)
	 ENDIF
      ELSE IF (Menu Choice .EQ. 'Quit       ') THEN
c***     Display Select Binary Operand 1 menu.
         CALL Put Menu (Mebvmain1)
	 RETURN
      ENDIF
      GOTO 10

      RETURN
      END
c**********************************************************************
c	 Check for Output Overflow (subroutine)
c**********************************************************************
      SUBROUTINE Check for Output Overflow (Col, Min, Max, Nerrors)

c***  This routine prompts the user for an output format and checks
c***  if the output format can print the min, max and missing
c***  values (if applicable).  If not than an error message is 
c***  displayed and the user is prompted for another output format.
c***
c***  FORMAT ARGUMENTS:
c***  Col      - Variable(column) number used for identifying the
c***             specific variable.
c***  Min      - Local variable, represents minimum value.    
c***  Max      - Local variable, represents maximum value.    
c***  Nerrors  - Number of missing values passed by the calling routine.
c***
c***  VARIABLES:
c***  Missing  - Global variable initialized at the beginning 
c***             of the Subroutine Process.                   
c***  M        - A real array which stores the min, max and missing values.

      include 'trans.dec'
      include '../common/screen.str'

      CHARACTER*20 Tmp Input, Tempchar
      CHARACTER*8  Value
      INTEGER*4    J, K, L, Col, Nerrors, Count
      REAL*4       M(3), Min, Max

c***  Check for output overflow for the missing, minimum and maximum 
c***  values generated in the calculation process.		     
      Value = ' F12.5  '
      M(1) = Max
      M(2) = Min
      M(3) = Missing
c***  If missing values exist, as indicated by Nerrors, then check for 
c***  missing values by setting count to 3, ie.,   include M(3) element.
c***  Update: Modification on 11/13/89 to disregard missing value as 
c***  it will be handled at the time the data is saved to an output file.
c      IF (Nerrors .GT. 0) THEN
c        Count = 3
c      ELSE
	Count = 2
c      ENDIF

  10  CALL Get Format(Value)
c      CALL Convert Int to Char Format(Col)
      
      Formats (Col) = Value
c***  Construct the format array 'Fmt'.  ***
      Fmt(1:1) = '('
      J = 1
      K = LENSTR (Formats(Col))
      DO 15 L = 1, K
	J = J + 1
	Fmt(J:J) = Formats(Col)(L:L)
  15  CONTINUE
      J = J + 1
      Fmt(J:J) = ')'
       
    
c***  Check for output overflow using format entered by user.  This
c***  step is easily accomplished using an internal read and write.
      DO 20 J = 1, Count
        WRITE (Tempchar,Fmt,IOSTAT=K) M(J)
        READ (Tempchar,Fmt,IOSTAT=K) M(J)
        IF (K .GT. 0) THEN
          CALL Beep
          CALL Flash (20, 2,
     +      'ERROR - output format causes overflow...(press any key)')
          GOTO 10
        ENDIF
  20  CONTINUE

      RETURN
      END
c**********************************************************************




