cc $LARGE
C Histo.for 
C----------------------------------------------------------------------
C Lag Histogram for VARIO
C
	SUBROUTINE Histo (LagNum)
	IMPLICIT INTEGER*4 (A-Z)
	PARAMETER (HMAXCLASS = 150)

	INCLUDE 'vario.dec'

c ***	local variables
	REAL*4		HistFreq (HMAXCLASS), Class (HMAXCLASS)
	REAL*4		Width, Tmax, Tmin, ClassInt
	REAL*4		Xc, Yc, X, Y, Dx, Dy, Tx, Ty, Mx, My
	REAL*4		TxMin, TxMax, TyMin, TyMax, TxTinc, TyTinc
	REAL*4		TMinx, Tmaxx, Tminy, TmaxY, Xfudge, Yfudge

	INTEGER*4	NclassHist, I, J, TN
	INTEGER*4   	N, lenstr, L, TnX, TnY, LagNum, Ptr
	CHARACTER   	Plot Title*70, Xlabel*40, Ylabel*40, Vlabel*15
        character*72 message, menu choice*10
	integer destination  

C check for no data
	TN = Pairs (LagNum)
	IF (TN .LE. 3) THEN
	   CALL Flash (20, 2,
     +		 'Too few pairs for histogram (press a key)')
	   RETURN
	ENDIF

 1	if (ps_or_no .eq. 0) call pre post script('histogram')

C Initialize classes and frequencies to zero
	DO 10 I = 1, HMAXCLASS
	   Class (I) = 0.0
	   HistFreq (I) = 0.0
 10	CONTINUE

C Compute X axis boundry and tic interval parameters
	TxMin = Difference (1)
	TxMax = Vmax (LagNum)
	IF (TXmin .EQ. TXmax) TxMax = TxMin + 1.0
	CALL NeatAx (TXmin, TXmax, TXtinc, TNx)
	Classint 	= ABS (TxTinc) * .25
	NClassHist 	= INT (ABS (TxMin - TxMax)  / ClassInt +.5)
	IF (NclassHist .LE. 0) NclassHist = 1
	IF (NclassHist .GE. HMAXCLASS) NclassHist = HMAXCLASS-1

C Calculate class cutoff values
	Class (1) = TxMin + ClassInt
	DO 20 I = 2, NclassHist
	   Class (I) = Class (I-1) + ClassInt
	   TxMax = AMAX1 (Txmax, Class (I))
 20	CONTINUE
	

C Calculate frequencies
        I = 1
        DO 30 J = 1, NclassHist
 40        IF (I .LE. TN .AND. Difference (I) .LE. Class (J)) THEN
	      IF (Difference (I) .LT. Missing) THEN
                 HistFreq (J) = HistFreq (J) + 1.0
                 I = I + 1
                 GOTO 40
	      ENDIF 
           ENDIF
 30     CONTINUE
 35     IF (I .LE. TN .AND. Difference (I) .LT. Missing) THEN
           HistFreq (NclassHist+1) = 
     +              HistFreq (NclassHist+1) + 1.0
           I = I + 1
           GOTO 35
        ENDIF 

C Calculate maximum y axis value
	Tymin = 0.0
	Tymax = HistFreq (1)
	DO 50 I = 2, NclassHist+1	
 	  TyMax = AMAX1 (HistFreq (I), TyMax)
 50     CONTINUE        

C Scale axes tickmarks to 'neat' values
	CALL Neatax (Tymin, Tymax, Tytinc,  K)
	TyMax = TyMax + TyTinc * .1
	TyMin = 0.0

C Compute symbol sizes, and scaling stuff
	Dx 	= TXmax - TXmin
	Dy 	= TYmax - TYmin
	Xfudge  = Dx * 0.15
	Yfudge  = Dy * 0.2
	TMinx 	= TXmin - (Xfudge*1.75)
	TMiny 	= TYmin -  Yfudge
	TMaxx 	= TXmax + (Xfudge*3.8)
	TMaxy 	= TYmax + (Yfudge*1.5)
	Mx 	= Dx * .01 / Aspect 
	My 	= Dy * .01

C Set up labels
	IF (Log) THEN
	  L = LENSTR (Varnam (Var))
	  Vlabel = 'LN ('//Varnam (Var) (1:L) //')'
	ELSE
	  Vlabel = VarNam (Var)
	ENDIF
	Plot Title  = 'Lag Histogram for '//Vlabel
	Xlabel = '(Z(x) - Z(x+h)) Squared'
	Ylabel = 'Frequency'

C graphics setup
	CALL Zsetup (INT2 (Gmode), Ierr)
	CALL Zcolor (INT2 (White))
	CALL Zlbdir (0.0)
	CALL Zlocat (INT2 (XminDev), INT2 (Xmaxdev),
     +               INT2 (YminDev), INT2 (YmaxDev))
	CALL Zscale (TMinx, TMaxx, TMiny, TMaxy)
	CALL Grframe (Xmaxdev, Ymaxdev, Blue) 

C Draw and label the axes
	CALL Axes (Plot Title, Xlabel, Ylabel, ' ',
     +		   TXmin, TXmax, TYmin, TYmax, TXtinc, TYtinc, DevCode)

C Plot boxplot above graph area
      Width   = Dy *.04
      K = LagNum
      CALL Boxplot (Vmin (K), VQ1 (K), Vmedian (K), Vmean (K),
     +                VQ3 (K), Vmax (K), .TRUE.,
     +                tYmax+Width*.7, Width, Dx*.01)
      
C Plot histogram
      CALL Zcolor (INT2 (Green))
      DO 60 I = 1,  NclassHist
         Tx = AMIN1 (Class (I) - Classint, TxMax)
         X  = AMIN1 (Class (I), TxMax)
         Y  = HistFreq (I)
         IF (Y .NE. 0.) CALL Hbox (Tx, X, 0.0, Y)	   
 60   CONTINUE
      IF (HistFreq (NclassHist+1) .GT. 0.0)
     +	   CALL Hbox (TxMax, TxMax + Dx*.04, 
     +     0.0, HistFreq (NclassHist+1))
      
      CALL Zcolor (INT2 (White))
      CALL Plot Lstats (LagNum, TXmin, TxMax, TyMin, TyMax)

      if (ps_or_no .eq. 0) call post script(message, destination)
      if (destination .eq. 1) goto 1

 666  call reset()
      RETURN
      END        
      
      
