/************************************************************
 *                                                          *
 *  Permission is hereby granted  to  any  individual   or  *
 *  institution   for  use,  copying, or redistribution of  *
 *  this  code and associated documentation,  provided      *
 *  that   such  code  and documentation are not sold  for  *
 *  profit and the  following copyright notice is retained  *
 *  in the code and documentation:                          *
 *    Copyright (c) 1994 Dianne Cook <dicook@iastate.edu>   *
 *                                                          *
 ************************************************************/

#include <math.h>
#include <stdio.h>
#include "xincludes.h"
#include "xgobitypes.h"
#include "xgobivars.h"

/* Functions used in this file */
Boolean RunWorkProcs();
Boolean current_mode();
Widget CreateToggle(), CreateCommand();
void refresh_vlab(), refresh_vbox();
void norm(), gram_schmidt();
float inner_prod();
void gen_norm_variates();
void show_message();
void find_plot_center(), refresh_vboxes(), world_to_plane();
void plane_to_screen(), update_world();
void plot_once(), set_varsel_label(), tour_var_lines();
void add_sbar_help(), free_txtr_var();
void set_mono();
void reset_var_labels();
void carry_corr_vars();
void update_lims();
/* */

Widget corr_panel, corr_cmd[6], corr_label[3], corr_speed_sbar;

static float **cu0, **cu1, **cu, **cuold;
long **cxi0, **cxi1;
/* static int ncorr_xvars, *corr_xvars, ncorr_yvars, *corr_yvars; */
static float tau_x, tau_y;
static float tinc_x, tinc_y;
static float corr_dv, corr_delta, corr_step;
static Boolean xvar_to_fade = False;
static int xvar_fading;
static Boolean xvar_fade_in_prog = False;
static Boolean yvar_to_fade = False;
static int yvar_fading;
static Boolean yvar_fade_in_prog = False;
static Boolean xvar_to_add = False;
static Boolean yvar_to_add = False;
/* variables to determine whether to stop corr_proc */
static int counting_to_stop;
static Boolean ready_to_stop_now;

static Boolean is_corr_paused;
static XSegment *corr_axes;
static char **corr_lab;
static hist_rec *cfl, *ccurr, *chfirst, *chend;
static Boolean is_corr_bt;
static int cnhist_list, old_cnhist_list, max_cnhist_list, cbt_dir;
static Boolean is_corr_syncd_axes;

#define PAUSE corr_cmd[0]
#define REINIT corr_cmd[1]
#define CBACKTRACK corr_cmd[2]
#define NBASES_HISTORY corr_label[0]
#define CDIRECTION corr_cmd[3]
#define SYNC_AXES corr_cmd[4]

void
alloc_corr(xg)
  xgobidata *xg;
{
  unsigned int nc = (unsigned int) xg->ncols;
  unsigned int nc2 = (unsigned int) (2*xg->ncols);
  unsigned int nr = (unsigned int) xg->nrows;
  int j, k;

  xg->corr_xvars = (int *) XtMalloc(nc * sizeof(int));
  xg->corr_yvars = (int *) XtMalloc(nc * sizeof(int));

  cu0 = (float **) XtMalloc((unsigned int) 2 * sizeof(float *));
  for (k=0; k<2; k++)
    cu0[k] = (float *) XtMalloc(nc * sizeof(float));

  cu1 = (float **) XtMalloc((unsigned int) 2 * sizeof(float *));
  for (k=0; k<2; k++)
    cu1[k] = (float *) XtMalloc(nc * sizeof(float));

  cu = (float **) XtMalloc((unsigned int) 2 * sizeof(float *));
  for (k=0; k<2; k++)
    cu[k] = (float *) XtMalloc(nc * sizeof(float));

  cuold = (float **) XtMalloc((unsigned int) 2 * sizeof(float *));
  for (k=0; k<2; k++)
    cuold[k] = (float *) XtMalloc(nc * sizeof(float));

  cxi0 = (long **) XtMalloc(nr * sizeof(long *));
  for (k=0; k<nr; k++)
    cxi0[k] = (long *) XtMalloc(2 * sizeof(long));

  cxi1 = (long **) XtMalloc(nr * sizeof(long *));
  for (k=0; k<nr; k++)
    cxi1[k] = (long *) XtMalloc(2 * sizeof(long));

/* Axes and axis labels */
  corr_axes = (XSegment *) XtMalloc(nc2 * sizeof(XSegment));
  corr_lab = (char **) XtMalloc(nc * sizeof (char *));
  for (j=0; j<nc; j++)
    corr_lab[j] = (char *) XtMalloc(
      (unsigned int) COLLABLEN * sizeof(char));

}

void
free_corr(xg)
  xgobidata *xg;
{
  int j, k;
  unsigned int nr = (unsigned int) xg->nrows;

  XtFree((XtPointer) xg->corr_xvars);
  XtFree((XtPointer) xg->corr_yvars);
  for (k=0; k<2; k++)
    XtFree((XtPointer) cu[k]);
  XtFree((XtPointer) cu);
  for (k=0; k<2; k++)
    XtFree((XtPointer) cu0[k]);
  XtFree((XtPointer) cu0);
  for (k=0; k<2; k++)
    XtFree((XtPointer) cu1[k]);
  XtFree((XtPointer) cu1);
  for (k=0; k<2; k++)
    XtFree((XtPointer) cuold[k]);
  XtFree((XtPointer) cuold);

  for (k=0; k<nr; k++)
    XtFree((XtPointer) cxi0[k]);
  XtFree((XtPointer) cxi0);
  for (k=0; k<nr; k++)
    XtFree((XtPointer) cxi1[k]);
  XtFree((XtPointer) cxi1);

  XtFree((XtPointer) corr_axes);
  for (j=0; j<xg->ncols; j++)
    XtFree((XtPointer) corr_lab[j]);
  XtFree((XtPointer) corr_lab);

}

void
AllocCorrBox(xg)
/*
 * This routine allocates space for a new node if there are no unused nodes
 * on the free list, fl.
*/
  xgobidata *xg;
{
  if (cfl == NULL)
  {
    ccurr = (hist_rec *) XtMalloc(
      (unsigned int) sizeof(hist_rec));
    ccurr->hist[0] = (float *) XtMalloc(
      (unsigned int) xg->ncols * sizeof(float));
    ccurr->hist[1] = (float *) XtMalloc(
      (unsigned int) xg->ncols * sizeof(float));
  }
  else
  {
    ccurr = cfl;
    cfl = cfl->prev;
  }
}

void
reinit_corr(xg)
  xgobidata *xg;
{
  XtCallCallbacks(REINIT, XtNcallback, (XtPointer) xg);
}

void
nback_update_clabel()
{
  char str[10];

  (void) sprintf(str, "%d", cnhist_list);
  XtVaSetValues(NBASES_HISTORY,
    XtNstring, (String) str,
    NULL);
}

void
reinit_corr_hist()
{
  hist_rec *histp;
/*
 * Free and then reinitialize the tour arrays.
*/
  if (cfl != NULL)
  {
    while (cfl->prev != NULL)
    {
      histp = cfl;
      cfl = cfl->prev;
      XtFree((XtPointer) histp->hist[0]);
      XtFree((XtPointer) histp->hist[1]);
      XtFree((XtPointer) histp);
    }
  }

  if (ccurr != NULL)
  {
    ccurr->next = NULL;
    chfirst->prev = NULL;
    while (ccurr->prev != NULL)
    {
      histp = ccurr;
      ccurr = ccurr->prev;
      XtFree((XtPointer) histp->hist[0]);
      XtFree((XtPointer) histp->hist[1]);
      XtFree((XtPointer) histp);
    }
  }
  chfirst = ccurr; 
}

void
store_corr_basis(xg, basis)
  xgobidata *xg;
  float **basis;
{
/*
 * This routine saves a basis, u0, into the history list.
*/
  int i, j;
  hist_rec *prev;
/*
 * Initial node doubly linked to itself, used when tour is started.
*/
  if (ccurr == NULL)
  {
    AllocCorrBox(xg);
    for (i=0; i<xg->ncols_used; i++)
      for (j=0; j<2; j++)
        ccurr->hist[j][i] = basis[j][i];
    ccurr->next = ccurr;
    ccurr->prev = ccurr;
    chfirst = ccurr;
    old_cnhist_list = cnhist_list;
    cnhist_list++;
  }
  else
  {
/* every other case */
    prev = ccurr;
    if ((cnhist_list <= MAXHIST))
    {
/*
 * Add in new node and remake circular list
*/
      AllocCorrBox(xg);
      for (i=0; i<xg->ncols_used; i++)
        for (j=0; j<2; j++)
          ccurr->hist[j][i] = basis[j][i];
      ccurr->prev = prev;
      ccurr->next = chfirst;
      prev->next = ccurr;
      chfirst->prev = ccurr;
      old_cnhist_list = cnhist_list;
      if (cnhist_list != MAXHIST)
        cnhist_list++;
    }
    else
    {
      /*
       * Case where MAXHIST has been reached. Move pointer to
       * next node in list and overwrite base.
      */
      ccurr = ccurr->next;
      for (i=0; i<xg->ncols_used; i++)
        for (j=0; j<2; j++)
          ccurr->hist[j][i] = basis[j][i];
      chfirst = ccurr->next;
    }
  }
}

void
retrieve_corr_basis(xg)
  xgobidata *xg;
{
  int i, j;

  for (i=0; i<xg->ncols_used; i++)
    for (j=0; j<2; j++)
      cu1[j][i] = ccurr->hist[j][i];
}

void
init_corr(xg)
  xgobidata *xg;
{
  int j;

/* initialize first variable as dependent variable, and
 * second two as independent variables.
*/
  xg->ncorr_xvars = 2;
  xg->ncorr_yvars = 1;

  for (j=0; j<xg->ncols; j++)
    xg->corr_xvars[j] = xg->corr_yvars[j] = 0;
  xg->corr_xvars[0] = 1;
  xg->corr_xvars[1] = 2;
  xg->corr_yvars[0] = 0;

  for (j=0; j<xg->ncols; j++)
    cu[0][j] = cu[1][j] = cu0[0][j] = cu0[1][j] = cu1[0][j] = cu1[1][j] = 
      cuold[0][j] = cuold[1][j] = 0.;

  cu[0][xg->corr_xvars[0]] = cu0[0][xg->corr_xvars[0]] = 
    cu1[0][xg->corr_xvars[0]] = cuold[0][xg->corr_xvars[0]] = 1.;
  cu[1][xg->corr_yvars[0]] = cu0[1][xg->corr_yvars[0]] = 
    cu1[1][xg->corr_yvars[0]] = cuold[1][xg->corr_yvars[0]] = 1.;

  tau_x = tau_y = 0.;
  tinc_x = tinc_y = 0.;
  corr_delta = 0.0;
  corr_dv = 1.0;
  corr_step = TOURSTEP0;

  xg->is_corr_touring = False;
  xg->run_corr_proc = False;
  is_corr_bt = False;
  cnhist_list = 1;
  max_cnhist_list = MAXHIST;
  cbt_dir = BACKWARD;
  cfl = NULL;
  ccurr = NULL;
  chfirst = ccurr;

  is_corr_syncd_axes = False;
}

void
start_corr_proc(xg)
  xgobidata *xg;
{
  if (!is_corr_paused)
  {
    xg->run_corr_proc = True;
    (void) XtAppAddWorkProc(app_con, RunWorkProcs, NULL);
  }
}

void
stop_corr_proc(xg)
  xgobidata *xg;
{
  xg->run_corr_proc = False;
}

void
map_corr_panel(xg, is_corr_on)
  xgobidata *xg;
  Boolean is_corr_on;
{
  if (is_corr_on)
    XtMapWidget(corr_panel);
  else
    XtUnmapWidget(corr_panel);
}

void
corr_var_lines(xg)
  xgobidata *xg;
{
  int j;
  int x, y;
  int r = xg->radius;
  float fr = (float) xg->radius;
  Window vwin;
  XGCValues *gcv, gcv_inst;

  XGetGCValues(display, varpanel_xor_GC, GCCapStyle|GCJoinStyle, &gcv_inst);
  gcv = &gcv_inst;

  if (xg->is_corr_touring)
  {
    for (j=0; j<xg->ncols_used; j++)
    {
      if (xg->varchosen[j])
        XSetLineAttributes(display, varpanel_xor_GC, 2, LineSolid,
          gcv->cap_style, gcv->join_style);

      vwin = XtWindow(xg->vardraww[j]);
      x = INT(fr * cuold[0][j]);
      y = INT(fr * cuold[1][j]);
      ChooseVar(vwin, r, x, y);
      x = INT(fr * cu[0][j]);
      y = INT(fr * cu[1][j]);
      ChooseVar(vwin, r, x, y);
      cuold[0][j] = cu[0][j];
      cuold[1][j] = cu[1][j];

      if (xg->varchosen[j])
        XSetLineAttributes(display, varpanel_xor_GC, 0, LineSolid,
          gcv->cap_style, gcv->join_style);
    }
  }
}

int
more_interp_togo(xg)
  xgobidata *xg;
{
  int ret_val = 1;

  tinc_x += (corr_delta * tau_x);
  tinc_y += (corr_delta * tau_y);

  if ((xg->ncorr_xvars > 1 && tinc_x >= tau_x) || 
    ((xvar_to_fade || xvar_fade_in_prog) &&  xg->ncorr_xvars == 1 && 
      tinc_x >= tau_x) ||
    (xg->ncorr_yvars > 1 && tinc_y >= tau_y) ||
    ((yvar_to_fade || yvar_fade_in_prog) &&  xg->ncorr_yvars == 1 && 
      tinc_y >= tau_y))
    ret_val = 0;

  return(ret_val);
}

void
set_counting_to_stop_corr(logical)
  int logical;
{
  counting_to_stop = logical;
}

void
set_ready_to_stop_corr_now(logical)
  Boolean logical;
{
  ready_to_stop_now = logical;
}

void
increment_corr(xg)
  xgobidata *xg;
{
  world_to_plane(xg);
  plane_to_screen(xg);
  plot_once(xg);
  corr_var_lines(xg);
}

void
do_last_corr_increment(xg)
  xgobidata *xg;
{
  if (tinc_x != tau_x || tinc_y != tau_y)
  {
    tinc_x = tau_x;
    tinc_y = tau_y;
    world_to_plane(xg);
    plane_to_screen(xg);
    plot_once(xg);
    tour_var_lines(xg);
  }
}

void
init_corr_basis(xg)
  xgobidata *xg;
{
  int j, k;

  for (k=0; k<2; k++)
    for (j=0; j<xg->ncols_used; j++)
      cu0[k][j] = cu[k][j];
}

void
new_corr_basis(xg)
  xgobidata *xg;
{
  int j;

  for (j=0; j<xg->ncols_used; j++)
    cu1[0][j] = cu1[1][j] = xg->tv[0][j] = 0.;

  if (xg->ncorr_xvars > 1)
  {
    gen_norm_variates(xg->ncorr_xvars, 1, xg->tv[0]);
    for (j=0; j<xg->ncorr_xvars; j++)
      cu1[0][xg->corr_xvars[j]] = xg->tv[0][j];
    norm(cu1[0], xg->ncols_used);
  }
  else
    cu1[0][xg->corr_xvars[0]] = 1.;

  if (!is_corr_syncd_axes)
  {
    if (xg->ncorr_yvars > 1)
    {
      for (j=0; j<xg->ncols_used; j++)
        xg->tv[0][j] = 0.;
      gen_norm_variates(xg->ncorr_yvars, 1, xg->tv[0]);
      for (j=0; j<xg->ncorr_yvars; j++)
        cu1[1][xg->corr_yvars[j]] = xg->tv[0][j];
      norm(cu1[1], xg->ncols_used);
    }
    else
      cu1[1][xg->corr_yvars[0]] = 1.;
  }
  else
  {
    for (j=0; j<xg->ncorr_yvars; j++)
      cu1[1][xg->corr_yvars[j]] = cu1[0][xg->corr_xvars[j]];
  }

}

void
zero_corr_tincs()
{
  tinc_x = tinc_y = 0.;
}

void
zero_corr_taus()
{
  tau_x = tau_y = 0.;
}

void
corr_span_planes(xg)
  xgobidata *xg;
{
  int i, j, m;

  for (m=0; m<xg->nrows_in_plot; m++)
  {
    i = xg->rows_in_plot[m];
    for (j=0; j<xg->ncols_used; j++)
      xg->tnx[j] = FLOAT(xg->world_data[i][j]);
/*     for (k=0; k<2; k++) */
      cxi0[i][0] = (long)(inner_prod(xg->tnx, cu0[0], xg->ncols_used));
      cxi1[i][0] = (long)(inner_prod(xg->tnx, cu1[0], xg->ncols_used));
/*    if (xg->ncorr_yvars > 2 || (xg->ncorr_yvars == 2 && !xvar_fade_in_prog) ||
      (xg->ncorr_yvars == 1 && yvar_fade_in_prog))
    {*/
/*         for (k=0; k<2; k++) */
          cxi0[i][1] = (long)(inner_prod(xg->tnx, cu0[1], xg->ncols_used));
          cxi1[i][1] = (long)(inner_prod(xg->tnx, cu1[1], xg->ncols_used));
/*     } */
  }
}

void
corr_reproject(xg)
/*
 * This routine uses the data projected into the span of
 * the starting basis and ending basis, and then rotates in this
 * space.
*/
  xgobidata *xg;
{
  int i, j;
  float ctx, stx, cty, sty;
  int ictx, istx, icty, isty;
  float stddev_fn();

  if (xg->ncorr_xvars > 1 || 
    (xg->ncorr_xvars == 1 && xvar_fade_in_prog))
  {
    ctx = cos((double) tinc_x);
    stx = sin((double) tinc_x);
    ictx = (int)(ctx * PRECISION2);
    istx = (int)(stx * PRECISION2);
    for (j=0; j<xg->ncols_used; j++)
      cu[0][j] = ctx * cu0[0][j] + stx * cu1[0][j];
  }
  else
  {
    ictx = PRECISION2;
    istx = 0;
  }

  if (xg->ncorr_yvars > 1 || 
    (xg->ncorr_yvars == 1 && yvar_fade_in_prog))
  {
    cty = cos((double) tinc_y);
    sty = sin((double) tinc_y);
    icty = (int)(cty * PRECISION2);
    isty = (int)(sty * PRECISION2);
    for (j=0; j<xg->ncols_used; j++)
      cu[1][j] = cty * cu0[1][j] + sty * cu1[1][j];
  }
  else
  {
    icty = PRECISION2;
    isty = 0;
  }

  for (i=0; i<xg->nrows; i++)
  {
    xg->planar[i].x = ictx * cxi0[i][0] + istx * cxi1[i][0];
    xg->planar[i].x /= PRECISION2;
    xg->planar[i].y = icty * cxi0[i][1] + isty * cxi1[i][1];
    xg->planar[i].y /= PRECISION2;
  }

}

void
calc_corr_angs(xg)
  xgobidata *xg;
{
  float tmpf;
  int j;

  if (xg->ncorr_xvars > 1 || (xg->ncorr_xvars == 1 && xvar_to_fade))
  {
    tmpf = inner_prod(cu0[0], cu1[0], xg->ncols_used);
    tau_x = (float)(acos((double) tmpf));
    if (tau_x > PI/2.)
    {
      tau_x = (PI - tau_x);
      for (j=0; j<xg->ncols_used; j++)
        cu1[0][j] *= (-1);
    }
  }
  if (xg->ncorr_yvars > 1 || (xg->ncorr_yvars == 1 && yvar_to_fade))
  {
    tmpf = inner_prod(cu0[1], cu1[1], xg->ncols_used);
    tau_y = (float)(acos((double) tmpf));
    if (tau_y > PI/2.)
    {
      tau_y = (PI - tau_y);
      for (j=0; j<xg->ncols_used; j++)
        cu1[1][j] *= (-1);
    }
  }
  else
    tau_y = 0.;

  corr_dv = sqrt(tau_x*tau_x + tau_y*tau_y);
  corr_delta = corr_step/corr_dv;
}

void
corr_path(xg)
  xgobidata *xg;
{
  calc_corr_angs(xg);
  if (xg->ncorr_xvars > 1 || (xg->ncorr_xvars == 1 && xvar_to_fade))
    gram_schmidt(cu0[0], cu1[0], xg->ncols_used);
  if (xg->ncorr_yvars > 1 || (xg->ncorr_yvars == 1 && yvar_to_fade))
    gram_schmidt(cu0[1], cu1[1], xg->ncols_used);
}

void
reset_corr_pause_cmd(xg, call, state)
  xgobidata *xg;
  Boolean call, state;
{
  if (call)
    XtCallCallbacks(PAUSE, XtNcallback, (XtPointer) xg);
  XtVaSetValues(PAUSE, XtNstate, (Boolean) state, NULL);
}

void
set_sens_corr_reinit_cmd(xg, sens)
  xgobidata *xg;
  Boolean sens;
{
  XtVaSetValues(REINIT, XtNsensitive, (Boolean) sens, NULL);
}

void
set_sens_corr_sync_axes_cmd(xg, sens)
  xgobidata *xg;
  Boolean sens;
{
  XtVaSetValues(SYNC_AXES, XtNsensitive, (Boolean) sens, NULL);
}

void
reset_corr_sync_axes_cmd(xg, call, state)
  xgobidata *xg;
  Boolean call, state;
{
  if (call)
    XtCallCallbacks(SYNC_AXES, XtNcallback, (XtPointer) xg);
  XtVaSetValues(SYNC_AXES, XtNstate, (Boolean) state, NULL);
}

void
reset_corr_cycleback_cmd(xg, set, sens, label)
  xgobidata *xg;
  int set, sens;
  char *label;
{
  if (set)
    XtVaSetValues(CDIRECTION,
      XtNsensitive, (Boolean) sens,
      XtNlabel, (String) label,
      NULL);
  else
    XtVaSetValues(CDIRECTION,
      XtNlabel, (String) label,
      NULL);
}

void
determine_new_corr_endbasis_and_path(xg)
  xgobidata *xg;
{
  if (is_corr_bt)
  {
    /*
     * Backtracking control.
    */
    if (cbt_dir == FORWARD)
    {
      init_corr_basis(xg);
      if (ccurr->next == NULL)
      {
        /*
         * Reverse direction and sleep for a second
         * when reaching the end of the history list.
        */
        cbt_dir = BACKWARD;
        ccurr = ccurr->prev;
        reset_corr_cycleback_cmd(xg, 0, 0, "B");
        sleep(1);
      }
      else
        ccurr = ccurr->next;

      retrieve_corr_basis(xg);
      cnhist_list += cbt_dir;
      nback_update_clabel();
    }
    else
    {
      init_corr_basis(xg);
      if (ccurr->prev == NULL)
      {
        /*
         * Reverse direction and sleep for a second
         * when reaching the end of the history list.
        */
        cbt_dir = FORWARD;
        ccurr = ccurr->next;
        reset_corr_cycleback_cmd(xg, 0, 0, "F");
        sleep(1);
      }
      else
        ccurr = ccurr->prev;

      retrieve_corr_basis(xg);
      cnhist_list += cbt_dir;
      nback_update_clabel();
    }
  }
  else
  {
    init_corr_basis(xg);
    new_corr_basis(xg);
    store_corr_basis(xg, cu1);
    nback_update_clabel();
  }
  zero_corr_tincs();
  corr_path(xg);
  corr_span_planes(xg);
}

void
corr_proc(xg)
  xgobidata *xg;
{
  if (more_interp_togo(xg))
  {
    increment_corr(xg);
  }
  else
  {
    if (ready_to_stop_now)
    {
      ready_to_stop_now = False;
      stop_corr_proc(xg);
      reset_corr_pause_cmd(xg, 1, 1);
    }
    else
    {
      do_last_corr_increment(xg);
      determine_new_corr_endbasis_and_path(xg);
      if (xvar_fade_in_prog)
      {
        xvar_fade_in_prog = False;
      }
      if (xvar_to_fade)
      {
        xvar_to_fade = False;
        xvar_fade_in_prog = True;
      }
      if (xvar_to_add)
        xvar_to_add = False;
      if (yvar_fade_in_prog)
      {
        yvar_fade_in_prog = False;
      }
      if (yvar_to_fade)
      {
        yvar_to_fade = False;
        yvar_fade_in_prog = True;
      }
      if (yvar_to_add)
        yvar_to_add = False;
      if (counting_to_stop == 1)
      {
        ready_to_stop_now = True;
        counting_to_stop--;
      }
    }
  }
}

/* ARGSUSED */
XtCallbackProc
corr_tour_on_cback(w, xgobi, callback_data)
  Widget w;
  XtPointer *xgobi;
  caddr_t callback_data;
{
  xgobidata *xg = (xgobidata *) xgobi;
  int j, k;

/*
 *  If this mode is currently selected, turn it off.
*/
  if ((Boolean) current_mode(w))
  {
    map_corr_panel(xg, False);
    stop_corr_proc(xg);
  }
  else
  {    /* Else turn it on */

    if (!xg->is_corr_touring)
    {
      if (xg->is_dotplotting)
        free_txtr_var();
      else if (xg->is_touring && xg->is_princ_comp)
        reset_var_labels(xg, PRINCCOMP_OFF);

      if (xg->carry_vars)
        carry_corr_vars(xg);
      xg->is_dotplotting = xg->is_xyplotting = False;
      xg->is_spinning = False;
      xg->is_touring = False;
      xg->is_corr_touring = True;
  
      find_plot_center(xg); 
      update_lims(xg);
      update_world(xg);
      world_to_plane(xg);
      plane_to_screen(xg);
      plot_once(xg);

      for (j=0; j<xg->ncols_used; j++)
        xg->varchosen[j] = False;
      for (j=0; j<xg->ncorr_xvars; j++)
      {
        k = xg->corr_xvars[j];
        xg->varchosen[k] = True;
      }
      for (j=0; j<xg->ncorr_yvars; j++)
      {
        k = xg->corr_yvars[j];
        xg->varchosen[k] = True;
      }
  
      for (j=0; j<xg->ncols_used; j++)
        refresh_vlab(j, xg);
      refresh_vboxes(xg);

    }
    map_corr_panel(xg, True);
    start_corr_proc(xg);
  }
}

void
set_corr_speed(slidepos, xg)
  float slidepos;
  xgobidata *xg;
{
  double sqrt(), pow();
/*
 * If the slider is near the start of its range, set step = 0.
 * and stop the touring.
*/
  if (slidepos < .05)
  {
    stop_corr_proc(xg);
    corr_step = 0.0;
  }
  else
  {
    start_corr_proc(xg);

    if (slidepos < 0.8)
      corr_step = (slidepos - .05) / 20. ;
    else if ((slidepos >= 0.8) && (slidepos < 0.9))
      corr_step = pow((double)(slidepos-0.8),(double)0.90) + 0.0375;
    else 
      corr_step = sqrt((double)(slidepos-0.8)) + 0.0375;
        /* 0.0375 = (0.8-0.05)/20.*/
/**/
  }
  corr_delta = corr_step/corr_dv;
}

/* ARGSUSED */
XtCallbackProc
corr_speed_cback(w, xgobi, slideposp)
  Widget w;
  XtPointer *xgobi;
  XtPointer slideposp;
{
  float slidepos = * (float *) slideposp;
  xgobidata *xg = (xgobidata *) xgobi;

  set_corr_speed(slidepos, xg);
}

/* ARGSUSED */
XtCallbackProc
corr_pause_cback(w, xgobi, callback_data)
  Widget w;
  XtPointer *xgobi;
  caddr_t callback_data;
{
  xgobidata *xg = (xgobidata *) xgobi;

  if (is_corr_paused)
  {
    if (xg->ncorr_xvars > 1 || xg->ncorr_yvars > 1)
    {
      is_corr_paused = False;
      start_corr_proc(xg);
    }
    else
      reset_corr_pause_cmd(xg, 0, 1);
  }
  else
  {
    is_corr_paused = True;
    stop_corr_proc(xg);
  }
}

/* ARGSUSED */
XtCallbackProc
corr_reinit_cback(w, xgobi, callback_data)
  Widget w;
  XtPointer *xgobi;
  caddr_t callback_data;
{
  int i, j;
  xgobidata *xg = (xgobidata *) xgobi;

/* set u0 to be the first two variables, and zero taus */
  for (i=0; i<2; i++)
    for (j=0; j<xg->ncols_used; j++)
      cu0[i][j] = 0.;
  cu0[1][xg->corr_yvars[0]] = 1.0;
  cu0[0][xg->corr_xvars[0]] = 1.0;
  for (i=0; i<2; i++)
    for (j=0; j<xg->ncols_used; j++)
      cu[i][j] = cu0[i][j];
  for (i=0; i<2; i++)
    for (j=0; j<xg->ncols_used; j++)
      cu1[i][j] = cu0[i][j];

  zero_corr_tincs();
  zero_corr_taus();

  reinit_corr_hist();
  cnhist_list = 0;
  store_corr_basis(xg, cu0);
  nback_update_clabel();
  update_world(xg);
  world_to_plane(xg);
  plane_to_screen(xg);
  plot_once(xg);
  corr_var_lines(xg);
}

Boolean
corr_backtracking()
{
  return(is_corr_bt);
}

/* ARGSUSED */
XtCallbackProc
corr_bt_cback(w, xgobi, callback_data)
  Widget w;
  XtPointer *xgobi;
  caddr_t callback_data;
{
  xgobidata *xg = (xgobidata *) xgobi;

  if (!is_corr_bt)
  {
    if ((cnhist_list > 2) || 
      (!check_proximity(cu, cu0, xg->ncols_used) && 
      cnhist_list == 2))
    {
      cbt_dir = BACKWARD;
      reset_corr_cycleback_cmd(xg, 1, 1, "B");
      /*
       * Store current position if not arbitrarily close to
       * last entry in history list.
      */
      if (!check_proximity(cu, cu0, xg->ncols_used))
      {
        init_corr_basis(xg);
        store_corr_basis(xg, cu0);
        nback_update_clabel();
      }
      /* Have just added 1 to nhist_list with store_basis, when there was 
       * already one more than actually in the list, so decrement nhist_list
       * by one, if there are less than the MAXHIST bases in the list. Also
       * allowing for the situation when there are MAXHIST-1 in the list but
       * heading towards MAXHIST.
      */
      if ((cnhist_list < MAXHIST) || 
        ((cnhist_list == MAXHIST) && 
        (old_cnhist_list == (MAXHIST-1)))) 
      {
        cnhist_list--;
      }
      max_cnhist_list = cnhist_list;
      /*
       * Break circular list and make end pointers NULL
      */
      chend = ccurr;
      ccurr->next = NULL;
      chfirst->prev = NULL;

      zero_corr_tincs();
      zero_corr_taus();
      is_corr_bt = True;
      set_varsel_label(xg);

      /* de/re-sensitize widgets */
      set_sens_corr_sync_axes_cmd(xg, 0);
      set_sens_corr_reinit_cmd(xg, 0);
    }
  }
  else
  {
    if (cbt_dir == FORWARD)
    {
      /*
       * If not at end of curr list save previous base,
       * break links and load remaining bases onto the free list.
      */
      if (ccurr->prev != NULL)
      {
        chend = ccurr->prev;
        cfl = ccurr;
        cfl->prev = NULL;
        while (cfl->next != NULL)
          cfl = cfl->next;
      }
    }
    else
    {
      /*
       * If not at end of curr list save current base,
       * break links and load remaining bases onto the free list.
      */
      if (ccurr->next != NULL)
      {
        chend = ccurr;
        nback_update_clabel();
        ccurr = ccurr->next;
        cfl = ccurr;
        cfl->prev = NULL;
        while (cfl->next != NULL)
          cfl = cfl->next;
      }
      cnhist_list++;
    }
    /* ensure fl ends have NULL pointers */
    if (cfl != NULL)
      cfl->next = NULL;
    ccurr = chend;
    /*
     * re-form circular links
    */
    ccurr->next = chfirst;
    chfirst->prev = ccurr;
    old_cnhist_list = cnhist_list - 1;
    max_cnhist_list = MAXHIST;
    zero_corr_tincs();
    zero_corr_taus();
    reset_corr_cycleback_cmd(xg, 1, 0, "F");
    is_corr_bt = False;
    set_varsel_label(xg);

    /* de/re-sensitize widgets */
    set_sens_corr_sync_axes_cmd(xg, 1);
    set_sens_corr_reinit_cmd(xg, 1);
  }
}

/* ARGSUSED */
XtCallbackProc
corr_cycleback_cback(w, xgobi, callback_data)
  Widget w;
  XtPointer *xgobi;
  caddr_t callback_data;
{
  xgobidata *xg = (xgobidata *) xgobi;
  int do_it = 1;
  char message[200];

  if (is_corr_bt)
  {
    if (cbt_dir == BACKWARD)   /* backwards */
    {
      /* 
       * If at the end of the list, ignore user request.
      */
      if (!ccurr->next)
      {
        do_it = 0;
        sprintf(message, "Can\'t go forward; at end of list.\n");
        show_message(message, xg->shell);
      }
      else
      {
        reset_corr_cycleback_cmd(xg, 0, 0, "F");
        cbt_dir = FORWARD;
        ccurr = ccurr->next;
      }
    }
    else if (cbt_dir == FORWARD)  /* forwards */
    {
      /* 
       * If at the end of the list, ignore user request.
      */
      if (!ccurr->prev)
      {
        do_it = 0;
        sprintf(message, "Can\'t go backward; at end of list.\n");
        show_message(message, xg->shell);
      }
      else
      {
        reset_corr_cycleback_cmd(xg, 0, 0, "B");
        cbt_dir = BACKWARD;
        ccurr = ccurr->prev;
      }
    }
    if (do_it)
    {
      init_corr_basis(xg);
      cnhist_list += cbt_dir;
      retrieve_corr_basis(xg);
      zero_corr_tincs();
      zero_corr_taus();
    }
  }
}

/* ARGSUSED */
XtCallbackProc
corr_sync_axes_cback(w, xgobi, callback_data)
  Widget w;
  XtPointer *xgobi;
  caddr_t callback_data;
{
  xgobidata *xg = (xgobidata *) xgobi;
  int j;
  char message[200];

  if (is_corr_syncd_axes)
  {
    is_corr_syncd_axes = False;
  }
  else
  {
    if (xg->ncorr_xvars == xg->ncorr_yvars)
    {
      is_corr_syncd_axes = True;
      init_corr_basis(xg);
      zero_corr_tincs();
      zero_corr_taus();
      for (j=0; j<xg->ncorr_yvars; j++)
        cu0[1][xg->corr_yvars[j]] = cu0[0][xg->corr_xvars[j]];
      update_world(xg);
      world_to_plane(xg);
      plane_to_screen(xg);
      plot_once(xg);
      corr_var_lines(xg);
    }
    else
    {
        sprintf(message,
         "The number of variables in each group is not equal. \n");
        show_message(message, xg->shell);
        reset_corr_sync_axes_cmd(xg, 0, 0);
     }
  }
}

void
make_corr(xg)
  xgobidata *xg;
{
  char str[30];
  Dimension max_width;

/*
 * Widest button label used in this panel.
*/
  sprintf(str, "What do you no?");
  max_width = XTextWidth(appdata.font, str, strlen(str)) +
    4 * ASCII_TEXT_BORDER_WIDTH + 3 + 2;
  /* borders around text, spacing between widgets, widget borders */

  corr_panel = XtVaCreateManagedWidget("CorrPanel",
    formWidgetClass, xg->box0,
    XtNleft, (XtEdgeType) XtChainLeft,
    XtNright, (XtEdgeType) XtChainLeft,
    XtNtop, (XtEdgeType) XtChainTop,
    XtNbottom, (XtEdgeType) XtChainTop,
    XtNmappedWhenManaged, (Boolean) False,
    NULL);
  if (mono) set_mono(corr_panel);

/*
 * tour speed scrollbar
*/
  corr_speed_sbar = XtVaCreateManagedWidget("Scrollbar",
    scrollbarWidgetClass, corr_panel,
    XtNwidth, (Dimension) max_width,
    XtNorientation, (XtOrientation) XtorientHorizontal,
    NULL);
  if (mono) set_mono(corr_speed_sbar);

  add_sbar_help(&xg->nhelpids.sbar,
    corr_speed_sbar, "Tour_Speed");

  XawScrollbarSetThumb(corr_speed_sbar, 20.*TOURSTEP0 + .05, -1.);
  XtAddCallback(corr_speed_sbar, XtNjumpProc,
    (XtCallbackProc) corr_speed_cback, (XtPointer) xg);

  PAUSE = CreateToggle(xg, "Pause",
    True, (Widget) NULL, (Widget) corr_speed_sbar, (Widget) NULL, 
    False, ANY_OF_MANY, corr_panel, "Corr_Pause");
  XtAddCallback(PAUSE, XtNcallback,
    (XtCallbackProc) corr_pause_cback, (XtPointer) xg);
  XtManageChild(PAUSE);

  REINIT = CreateCommand(xg, "Reinit",
    True, (Widget) PAUSE, (Widget) corr_speed_sbar, 
    (Widget) corr_panel, "Corr_Reinit");
  XtAddCallback(REINIT, XtNcallback,
    (XtCallbackProc) corr_reinit_cback, (XtPointer) xg);
  XtManageChild(REINIT);

/*
 * corr backtrack control: initiate insensitive, turn on
 * when there are more than two or three elements in the
 * history file
*/
  CBACKTRACK = CreateToggle(xg, "Backtrck",
    True, (Widget) NULL, PAUSE, (Widget) NULL, False, ANY_OF_MANY,
    corr_panel, "Corr_Backtrack");
  XtAddCallback(CBACKTRACK, XtNcallback,
    (XtCallbackProc) corr_bt_cback, (XtPointer) xg);
  XtManageChild(CBACKTRACK);

/*
 * label to record number of bases in history file
*/
  sprintf(str, "%d ", MAXHIST);
  max_width = XTextWidth(appdata.font, str, strlen(str));

  NBASES_HISTORY = XtVaCreateManagedWidget("CorrLabel",
    asciiTextWidgetClass, corr_panel,
    XtNfromVert, (Widget) PAUSE,
    XtNfromHoriz, (Widget) CBACKTRACK,
    XtNstring, (String) "2",
    XtNdisplayCaret, (Boolean) False,
    XtNwidth, (Dimension) max_width,
    NULL);
  if (mono) set_mono(NBASES_HISTORY);

/*
 * Button to allow cycling backwards and forwards through backtracking
*/
  CDIRECTION = CreateCommand(xg, "F",
    False, NBASES_HISTORY, PAUSE,
    corr_panel, "Corr_BtrackDir");
  XtAddCallback(CDIRECTION, XtNcallback,
    (XtCallbackProc) corr_cycleback_cback, (XtPointer) xg);
  XtManageChild(CDIRECTION);

  SYNC_AXES = CreateToggle(xg, "Sync Axes",
    True, (Widget) NULL, (Widget) CBACKTRACK, (Widget) NULL, 
    False, ANY_OF_MANY, corr_panel, "Corr_SyncAxes");
  XtAddCallback(SYNC_AXES, XtNcallback,
    (XtCallbackProc) corr_sync_axes_cback, (XtPointer) xg);
  XtManageChild(SYNC_AXES);

}

/*Boolean
check_xvarsel(varno, cvarno)
  int varno, cvarno;
{
  int j;
  Boolean selected = 0;

  for (j=0; j<xg->ncorr_xvars; j++)
    if (varno == xg->corr_xvars[j])
    {
      selected = 1;
      cvarno = j;
      break;
    }
  return(selected);
}*/

/*Boolean
check_yvarsel(varno, cvarno)
  int varno, cvarno;
{
  int j;
  Boolean selected = 0;

  for (j=0; j<xg->ncorr_yvars; j++)
    if (varno == xg->corr_yvars[j])
    {
      selected = 1;
      cvarno = j;
      break;
    }
  return(selected);
}*/

void
alter_target_basis(xg, varno)
  xgobidata *xg;
  int varno;
{
  int i;

  for (i=0; i<2; i++)
    cu1[i][varno] = 0.0;
  norm(cu1[0], xg->ncols_used);
  norm(cu1[1], xg->ncols_used);
  gram_schmidt(cu1[0], cu1[1], xg->ncols_used);
}

/*void
reorg_corr_xvars(cvarno, nxvars)
  int cvarno, nxvars;
{
  int j;

  nxvars = xg->ncorr_xvars;
}

void
reorg_ncorr_yvars(cvarno, nyvars) 
 int cvarno;
{
  int j;

  xg->ncorr_yvars--;

  for (j=cvarno; j<xg->ncorr_yvars; j++) 
    xg->corr_yvars[j] = xg->corr_yvars[j+1];

  nyvars = xg->ncorr_yvars;
}*/

void
set_fading_xvar(set_val, varno)
  Boolean set_val;
  int varno;
{
  xvar_to_fade = set_val;
  xvar_fading = varno;
}

void
set_fading_yvar(set_val, varno)
  Boolean set_val;
  int varno;
{
  yvar_to_fade = set_val;
  yvar_fading = varno;
}

/*
void
set_adding_xvar(set_val, varno)
  Boolean set_val;
  int varno;
{
  xvar_to_add = set_val;
  xvar_adding = varno;
}
*/

/*
void
set_adding_yvar(set_val, varno)
  Boolean set_val;
  int varno;
{
  yvar_to_add = set_val;
  yvar_adding = varno;
}
*/

void
calc_var_xy(r, varno, px, py)
  int r;
  int varno, *px, *py;
{
  int x, y;

  x = *px;
  y = *py;

  x = INT(FLOAT(r) * cuold[0][varno]);
  y = INT(FLOAT(r) * cuold[1][varno]);

  *px = x;
  *py = y;
}

int
corr_varselect(varno, event, xg)
/*
 * This is used only during correlation touring.
*/
  int varno;
  XButtonEvent *event;
  xgobidata *xg;
{
  int newvar = 1, cvarno;
  int i, j;
  int selected;
  int y_cvarno, x_cvarno;
  int found;
  char message[200];
  Boolean y_selected, x_selected, dont_doit;

  if (is_corr_syncd_axes)
    reset_corr_sync_axes_cmd(xg, 1, 0);/* turn off syncing while doing 
                                          variable selection */ 

/*
 * If it's a left click ...
*/
  if (event->button == 1)
  {
    selected = 0;
    for (j=0; j<xg->ncorr_xvars; j++)
      if (varno == xg->corr_xvars[j])
      {
        selected = 1;
        cvarno = j;
        break;
      }
    if (selected)
    {/* selected */
      if (xg->ncorr_xvars > 1)
      {
        xg->varchosen[varno] = False;
        cu1[0][varno] = 0.0;
        norm(cu1[0], xg->ncols_used);
        xg->ncorr_xvars--;
        if (xg->ncorr_xvars == 1 && xg->ncorr_yvars == 1)
          counting_to_stop = 1;
        for (j=cvarno; j<xg->ncorr_xvars; j++) 
          xg->corr_xvars[j] = xg->corr_xvars[j+1];
        refresh_vbox(xg, varno, 1);
        set_fading_xvar(True, varno);
        init_corr_basis(xg);
        zero_corr_tincs();
        zero_corr_taus();
        update_world(xg);
      }
      else
      {
        sprintf(message,
         "Too few X variables to remove it.\n");
        show_message(message, xg->shell);
      }
    }
    else
    {/* !selected */
      y_selected = dont_doit = False;
      for (j=0; j<xg->ncorr_yvars; j++)
        if (varno == xg->corr_yvars[j])
        {
          y_selected = True;
          y_cvarno = j;
          break;
        }
      if (y_selected)/* zero it out now */
      {
        if (xg->ncorr_yvars > 1)
        {
          cu[1][varno] = 0.0;
          norm(cu[1], xg->ncols_used);
          xg->ncorr_yvars--;
          if (xg->ncorr_xvars == 1 && xg->ncorr_yvars == 1)
            counting_to_stop = 1;
          for (j=y_cvarno; j<xg->ncorr_yvars; j++) 
            xg->corr_yvars[j] = xg->corr_yvars[j+1];
          if (xg->ncorr_yvars == 1)
            cu[1][xg->corr_yvars[0]] = 1.;
          reinit_corr_hist();
          cnhist_list = 0;
          store_corr_basis(xg, cu);
          nback_update_clabel();
        }
        else
	{
          dont_doit = True;
          sprintf(message,
            "Too few Y variables to change it to an X variable.\n");
          show_message(message, xg->shell);
	}
      }
      if (!dont_doit)
      {
        xg->varchosen[varno] = True;
        found = 0;
        if (varno < xg->corr_xvars[0])
        {
          for (j=xg->ncorr_xvars; j>0; j--) 
            xg->corr_xvars[j] = xg->corr_xvars[j-1];
          xg->corr_xvars[0] = varno;
          xg->ncorr_xvars += 1;
          found = 1;
        }
        i = 0;
        while (!found && i < xg->ncorr_xvars-1 )
        {
          if ( varno > xg->corr_xvars[i]  &&
               varno < xg->corr_xvars[i+1] )
          {
            for (j=xg->ncorr_xvars; j>i; j--) 
              xg->corr_xvars[j] = xg->corr_xvars[j-1];
            xg->corr_xvars[i+1] = varno;
            xg->ncorr_xvars += 1;
            found = 1;
          }
	  i++;
        }
        if (!found && (varno > xg->corr_xvars[xg->ncorr_xvars-1]))
        {
          xg->corr_xvars[xg->ncorr_xvars] = varno;
          xg->ncorr_xvars += 1;
          found = 1;
        }
        refresh_vbox(xg, varno, 1);
        init_corr_basis(xg);
        zero_corr_tincs();
        zero_corr_taus();
        update_world(xg);
      }
    }
  }
/*
 * If it's a middle click ...
*/
  else if (event->button == 2)
  {
    selected = 0;
    for (j=0; j<xg->ncorr_yvars; j++)
      if (varno == xg->corr_yvars[j])
      {
        selected = 1;
        cvarno = j;
        break;
      }
/*     if (check_yvarsel(varno, cvarno)) */
    if (selected)
    {/* selected */
      if (xg->ncorr_yvars > 1)
      {
        xg->varchosen[varno] = False;
        cu1[1][varno] = 0.0;
        norm(cu1[1], xg->ncols_used);
        xg->ncorr_yvars--;
        if (xg->ncorr_xvars == 1 && xg->ncorr_yvars == 1)
          counting_to_stop = 1;
        for (j=cvarno; j<xg->ncorr_yvars; j++) 
          xg->corr_yvars[j] = xg->corr_yvars[j+1];
        refresh_vbox(xg, varno, 1);
        set_fading_yvar(True, varno);
        init_corr_basis(xg);
        zero_corr_tincs();
        zero_corr_taus();
        update_world(xg);
      }
      else
      {
        sprintf(message,
         "Too few Y variables to remove it.\n");
        show_message(message, xg->shell);
      }
    }
    else
    {/* !selected */
      x_selected = dont_doit = False;
      for (j=0; j<xg->ncorr_xvars; j++)
        if (varno == xg->corr_xvars[j])
        {
          x_selected = True;
          x_cvarno = j;
          break;
        }
      if (x_selected)
      {
        if (xg->ncorr_xvars > 1)
        {
          cu[0][varno] = 0.0;
          norm(cu[0], xg->ncols_used);
          xg->ncorr_xvars--;
          if (xg->ncorr_xvars == 1 && xg->ncorr_yvars == 1)
            counting_to_stop = 1;
          for (j=x_cvarno; j<xg->ncorr_xvars; j++) 
            xg->corr_xvars[j] = xg->corr_xvars[j+1];
          reinit_corr_hist();
          cnhist_list = 0;
          store_corr_basis(xg, cu);
          nback_update_clabel();
        }
        else
	{
          dont_doit = True;
          sprintf(message,
            "Too few X variables to change it to an Y variable.\n");
          show_message(message, xg->shell);
	}
      }
      if (!dont_doit)
      {
        xg->varchosen[varno] = True;
        found = 0;
        if (varno < xg->corr_yvars[0])
        {
          for (j=xg->ncorr_yvars; j>0; j--) 
            xg->corr_yvars[j] = xg->corr_yvars[j-1];
          xg->corr_yvars[0] = varno;
          xg->ncorr_yvars++;
          found = 1;
        }
        i = 0;
        while (!found && i < xg->ncorr_yvars-1 )
        {
          if ( varno > xg->corr_yvars[i]  &&
               varno < xg->corr_yvars[i+1] )
          {
            for (j=xg->ncorr_yvars; j>i; j--) 
              xg->corr_yvars[j] = xg->corr_yvars[j-1];
            xg->corr_yvars[i+1] = varno;
            xg->ncorr_yvars++;
            found = 1;
          }
          i++;
        }
        if (!found && (varno > xg->corr_yvars[xg->ncorr_yvars-1]))
        {
          xg->corr_yvars[xg->ncorr_yvars] = varno;
          xg->ncorr_yvars++;
          found = 1;
        }
        refresh_vbox(xg, varno, 1);
        init_corr_basis(xg);
        zero_corr_tincs();
        zero_corr_taus();
        update_world(xg);
      }
    }
  }
  return(newvar);
}

void
draw_corr_axes(xg)
  xgobidata *xg;
{
  int j, k;
  int naxes;
  int raw_axis_len = MIN(xg->mid.x, xg->mid.y);
  float tol = .01;
  fcoords axs;

  /* Use the width of the plot window to normalize the axes */
  axs.x = raw_axis_len / 2.;
  axs.y = raw_axis_len / 2.;

  naxes = 0;

/*
 * modifications to always plot the axes based on the raw data,
 * regardless of principal components.
*/
  for (j=0; j<xg->ncols_used; j++)
  {
    if (cu[0][j]*cu[0][j] > tol)
    {
      corr_axes[naxes].x1 = xg->cntr.x;
      corr_axes[naxes].x2 = xg->cntr.x +
        (int) (axs.x * cu[0][j]);
      corr_axes[naxes].y1 = xg->cntr.y;
      corr_axes[naxes].y2 = xg->cntr.y;
      strcpy(corr_lab[(int)(naxes/2)], xg->collab_tform[j]);
      naxes++;
      corr_axes[naxes].x1 = corr_axes[naxes].x2 = xg->cntr.x +
        (int) (axs.x * cu[0][j]);
      corr_axes[naxes].y1 = xg->cntr.y + 2;
      corr_axes[naxes].y2 = xg->cntr.y - 2;
      naxes++;
    }
    if (cu[1][j]*cu[1][j] > tol)
    {
      corr_axes[naxes].x1 = xg->cntr.x;
      corr_axes[naxes].x2 = xg->cntr.x;
      corr_axes[naxes].y1 = xg->cntr.y;
      corr_axes[naxes].y2 = xg->cntr.y -
        (int) (axs.y * cu[1][j]);
      strcpy(corr_lab[(int)(naxes/2)], xg->collab_tform[j]);
      naxes++;
      corr_axes[naxes].x1 = xg->cntr.x + 2;
      corr_axes[naxes].x2 = xg->cntr.x - 2;
      corr_axes[naxes].y1 = corr_axes[naxes].y2 = xg->cntr.y -
        (int) (axs.y * cu[1][j]);
      naxes++;
    }
  }

/*
 * Draw axes of rotation.
*/
  XSetForeground(display, copy_GC, plotcolors.fg);
  XDrawSegments(display, xg->pixmap0, copy_GC, corr_axes, naxes);
/*
 * Add axis labels.
*/
  for (k=0; k<((int)naxes/2); k++)
  {
    XDrawString(display, xg->pixmap0, copy_GC,
       corr_axes[k*2].x2 + 6, corr_axes[k*2].y2 - 5,
       corr_lab[k], strlen(corr_lab[k]));
  }
}

#undef PAUSE

